@@ -10,6 +10,8 @@
 #include <memory>
 #include "bytes.hpp"
 
+#include <boost/type_traits/is_polymorphic.hpp>
+
 using namespace boost::python;
 using namespace libtorrent;
 
@@ -19,49 +21,6 @@ bytes get_buffer(read_piece_alert const& rpa)
        : bytes();
 }
 
-tuple endpoint_to_tuple(tcp::endpoint const& ep)
-{
-    return boost::python::make_tuple(ep.address().to_string(), ep.port());
-}
-
-tuple endpoint_to_tuple(udp::endpoint const& ep)
-{
-    return boost::python::make_tuple(ep.address().to_string(), ep.port());
-}
-
-
-tuple peer_alert_ip(peer_alert const& pa)
-{
-    return endpoint_to_tuple(pa.ip);
-}
-
-std::string peer_blocked_alert_ip(peer_blocked_alert const& pa)
-{
-    error_code ec;
-    return pa.ip.to_string(ec);
-}
-
-std::string dht_announce_alert_ip(dht_announce_alert const& pa)
-{
-    error_code ec;
-    return pa.ip.to_string(ec);
-}
-
-tuple incoming_connection_alert_ip(incoming_connection_alert const& ica)
-{
-    return endpoint_to_tuple(ica.ip);
-}
-
-tuple dht_outgoing_get_peers_alert_ip(dht_outgoing_get_peers_alert const& a)
-{
-    return endpoint_to_tuple(a.ip);
-}
-
-std::string external_ip_alert_ip(external_ip_alert const& eia)
-{
-    return eia.external_address.to_string();
-}
-
 list stats_alert_transferred(stats_alert const& alert)
 {
    list result;
@@ -197,13 +156,12 @@ dict session_stats_values(session_stats_alert const& alert)
 list dht_get_peers_reply_alert_peers(dht_get_peers_reply_alert const& a)
 {
     list result;
-
     std::vector<tcp::endpoint> v(a.peers());
 
     for (std::vector<tcp::endpoint>::const_iterator i = v.begin();
          i != v.end(); ++i)
     {
-        result.append(endpoint_to_tuple(*i));
+        result.append(*i);
     }
 
     return result;
@@ -214,13 +172,17 @@ void bind_alert()
     using boost::noncopyable;
 #ifndef TORRENT_NO_DEPRECATE
     typedef boost::shared_ptr<alert> alert_holder;
-#if BOOST_VERSION >= 106000
-    register_ptr_to_python<boost::shared_ptr<alert> >();
-#endif
+
+    if (boost::python::converter::registry::query(
+        boost::python::type_id <boost::shared_ptr<alert> >()) == NULL) {
+            register_ptr_to_python<boost::shared_ptr<alert> >();
+    }
 #else
     typedef alert alert_holder;
 #endif
 
+    typedef return_value_policy<return_by_value> by_value;
+
     {
         scope alert_scope = class_<alert, alert_holder, noncopyable >("alert", no_init)
             .def("message", &alert::message)
@@ -272,7 +234,7 @@ void bind_alert()
 
     class_<torrent_alert, bases<alert>, noncopyable>(
         "torrent_alert", no_init)
-        .def_readonly("handle", &torrent_alert::handle)
+        .add_property("handle", make_getter(&torrent_alert::handle, by_value()))
         ;
 
     class_<tracker_alert, bases<torrent_alert>, noncopyable>(
@@ -283,9 +245,11 @@ void bind_alert()
         .def("tracker_url", &tracker_alert::tracker_url)
         ;
 
+#ifndef TORRENT_NO_DEPRECATE
     class_<torrent_added_alert, bases<torrent_alert>, noncopyable>(
         "torrent_added_alert", no_init)
         ;
+#endif
 
     class_<torrent_removed_alert, bases<torrent_alert>, noncopyable>(
         "torrent_removed_alert", no_init)
@@ -294,6 +258,7 @@ void bind_alert()
 
     class_<read_piece_alert, bases<torrent_alert>, noncopyable>(
         "read_piece_alert", 0, no_init)
+        .def_readonly("ec", &read_piece_alert::ec)
         .add_property("buffer", get_buffer)
         .def_readonly("piece", &read_piece_alert::piece)
         .def_readonly("size", &read_piece_alert::size)
@@ -301,7 +266,7 @@ void bind_alert()
 
     class_<peer_alert, bases<torrent_alert>, noncopyable>(
         "peer_alert", no_init)
-        .add_property("ip", &peer_alert_ip)
+        .add_property("ip", make_getter(&peer_alert::ip, by_value()))
         .def_readonly("pid", &peer_alert::pid)
     ;
     class_<tracker_error_alert, bases<tracker_alert>, noncopyable>(
@@ -392,6 +357,8 @@ void bind_alert()
     class_<storage_moved_failed_alert, bases<torrent_alert>, noncopyable>(
         "storage_moved_failed_alert", no_init)
         .def_readonly("error", &storage_moved_failed_alert::error)
+        .def("file_path", &storage_moved_failed_alert::file_path)
+        .def_readonly("operation", &storage_moved_failed_alert::operation)
         ;
 
     class_<torrent_deleted_alert, bases<torrent_alert>, noncopyable>(
@@ -411,7 +378,9 @@ void bind_alert()
         .def_readonly("url", &url_seed_alert::url)
         .def_readonly("msg", &url_seed_alert::msg)
 #endif
+        .def_readonly("error", &url_seed_alert::error)
         .def("server_url", &url_seed_alert::server_url)
+        .def("error_message", &url_seed_alert::error_message)
         ;
 
     class_<file_error_alert, bases<torrent_alert>, noncopyable>(
@@ -425,14 +394,16 @@ void bind_alert()
         ;
 
     class_<metadata_failed_alert, bases<torrent_alert>, noncopyable>(
-        "metadata_failed_alert", no_init);
+        "metadata_failed_alert", no_init)
+        .def_readonly("error", &metadata_failed_alert::error)
+        ;
 
     class_<metadata_received_alert, bases<torrent_alert>, noncopyable>(
         "metadata_received_alert", no_init);
 
     class_<listen_failed_alert, bases<alert>, noncopyable>(
         "listen_failed_alert", no_init)
-        .def_readonly("endpoint", &listen_failed_alert::endpoint)
+        .add_property("endpoint", make_getter(&listen_failed_alert::endpoint, by_value()))
         .def("listen_interface", &listen_failed_alert::listen_interface)
         .def_readonly("error", &listen_failed_alert::error)
         .def_readonly("operation", &listen_failed_alert::operation)
@@ -441,7 +412,7 @@ void bind_alert()
 
     class_<listen_succeeded_alert, bases<alert>, noncopyable>(
         "listen_succeeded_alert", no_init)
-        .def_readonly("endpoint", &listen_succeeded_alert::endpoint)
+        .add_property("endpoint", make_getter(&listen_succeeded_alert::endpoint, by_value()))
         ;
 
     class_<portmap_error_alert, bases<alert>, noncopyable>(
@@ -484,11 +455,13 @@ void bind_alert()
 #ifndef TORRENT_NO_DEPRECATE
         .def_readonly("msg", &fastresume_rejected_alert::msg)
 #endif
+        .def("file_path", &fastresume_rejected_alert::file_path)
+        .def_readonly("operation", &fastresume_rejected_alert::operation)
         ;
 
     class_<peer_blocked_alert, bases<alert>, noncopyable>(
         "peer_blocked_alert", no_init)
-        .add_property("ip", &peer_blocked_alert_ip)
+        .add_property("ip", make_getter(&peer_blocked_alert::ip, by_value()))
         ;
 
     class_<scrape_reply_alert, bases<tracker_alert>, noncopyable>(
@@ -503,17 +476,18 @@ void bind_alert()
         .def_readonly("msg", &scrape_failed_alert::msg)
 #endif
         .def("error_message", &scrape_failed_alert::error_message)
+        .def_readonly("error", &scrape_failed_alert::error)
         ;
 
     class_<udp_error_alert, bases<alert>, noncopyable>(
         "udp_error_alert", no_init)
-        .def_readonly("endpoint", &udp_error_alert::endpoint)
+        .add_property("endpoint", make_getter(&udp_error_alert::endpoint, by_value()))
         .def_readonly("error", &udp_error_alert::error)
         ;
 
     class_<external_ip_alert, bases<alert>, noncopyable>(
         "external_ip_alert", no_init)
-        .add_property("external_address", &external_ip_alert_ip)
+        .add_property("external_address", make_getter(&external_ip_alert::external_address, by_value()))
         ;
 
     class_<save_resume_data_alert, bases<torrent_alert>, noncopyable>(
@@ -568,7 +542,7 @@ void bind_alert()
 
     class_<dht_announce_alert, bases<alert>, noncopyable>(
         "dht_announce_alert", no_init)
-        .add_property("ip", &dht_announce_alert_ip)
+        .add_property("ip", make_getter(&dht_announce_alert::ip, by_value()))
         .def_readonly("port", &dht_announce_alert::port)
         .def_readonly("info_hash", &dht_announce_alert::info_hash)
     ;
@@ -623,6 +597,7 @@ void bind_alert()
         .def_readonly("msg", &torrent_delete_failed_alert::msg)
 #endif
         .def_readonly("error", &torrent_delete_failed_alert::error)
+        .def_readonly("info_hash", &torrent_delete_failed_alert::info_hash)
         ;
 
     class_<save_resume_data_failed_alert, bases<torrent_alert>, noncopyable>(
@@ -673,6 +648,10 @@ void bind_alert()
 #endif
     ;
 
+    class_<cache_flushed_alert, bases<torrent_alert>, noncopyable>(
+        "cache_flushed_alert", no_init)
+    ;
+
     class_<anonymous_mode_alert, bases<torrent_alert>, noncopyable>(
         "anonymous_mode_alert", no_init)
         .def_readonly("kind", &anonymous_mode_alert::kind)
@@ -686,11 +665,11 @@ void bind_alert()
     class_<incoming_connection_alert, bases<alert>, noncopyable>(
         "incoming_connection_alert", no_init)
         .def_readonly("socket_type", &incoming_connection_alert::socket_type)
-        .add_property("ip", &incoming_connection_alert_ip)
+        .add_property("ip", make_getter(&incoming_connection_alert::ip, by_value()))
         ;
     class_<torrent_need_cert_alert, bases<torrent_alert>, noncopyable>(
         "torrent_need_cert_alert", no_init)
-        .def_readonly("error", &torrent_need_cert_alert::error) 
+        .def_readonly("error", &torrent_need_cert_alert::error)
         ;
 
     class_<add_torrent_alert, bases<torrent_alert>, noncopyable>(
@@ -709,7 +688,7 @@ void bind_alert()
        "dht_outgoing_get_peers_alert", no_init)
         .def_readonly("info_hash", &dht_outgoing_get_peers_alert::info_hash)
         .def_readonly("obfuscated_info_hash", &dht_outgoing_get_peers_alert::obfuscated_info_hash)
-        .add_property("ip", &dht_outgoing_get_peers_alert_ip)
+        .add_property("ip", make_getter(&dht_outgoing_get_peers_alert::ip, by_value()))
         ;
 
 #ifndef TORRENT_DISABLE_LOGGING
@@ -750,18 +729,30 @@ void bind_alert()
 
     class_<dht_immutable_item_alert, bases<alert>, noncopyable>(
        "dht_immutable_item_alert", no_init)
+        .def_readonly("target", &dht_immutable_item_alert::target)
         .add_property("item", &dht_immutable_item)
         ;
 
     class_<dht_mutable_item_alert, bases<alert>, noncopyable>(
        "dht_mutable_item_alert", no_init)
+        .def_readonly("key", &dht_mutable_item_alert::key)
+        .def_readonly("signature", &dht_mutable_item_alert::signature)
+        .def_readonly("seq", &dht_mutable_item_alert::seq)
+        .def_readonly("salt", &dht_mutable_item_alert::salt)
         .add_property("item", &dht_mutable_item)
+        .def_readonly("authoritative", &dht_mutable_item_alert::authoritative)
         ;
 
     class_<dht_put_alert, bases<alert>, noncopyable>(
        "dht_put_alert", no_init)
-        .add_property("item", &dht_put_item)
+        .def_readonly("target", &dht_put_alert::target)
+        .def_readonly("public_key", &dht_put_alert::public_key)
+        .def_readonly("signature", &dht_put_alert::signature)
+        .def_readonly("salt", &dht_put_alert::salt)
+        .def_readonly("seq", &dht_put_alert::seq)
+        .def_readonly("num_success", &dht_put_alert::num_success)
         ;
+
     class_<session_stats_alert, bases<alert>, noncopyable>(
         "session_stats_alert", no_init)
         .add_property("values", &session_stats_values)
