@@ -10,6 +10,9 @@ void bind_fingerprint()
     using namespace boost::python;
     using namespace libtorrent;
 
+    def("generate_fingerprint", &generate_fingerprint);
+
+#ifndef TORRENT_NO_DEPRECATE
     class_<fingerprint>("fingerprint", no_init)
         .def(
             init<char const*,int,int,int,int>(
@@ -23,5 +26,6 @@ void bind_fingerprint()
         .def_readonly("revision_version", &fingerprint::revision_version)
         .def_readonly("tag_version", &fingerprint::tag_version)
         ;
+#endif
 }
 
