@@ -2,15 +2,10 @@
 // subject to the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#include <boost/python.hpp>
-#include "libtorrent/config.hpp"
-#include "libtorrent/utf8.hpp"
+#include "boost_python.hpp"
 #include <string>
 
-#if TORRENT_USE_WSTRING
-
 using namespace boost::python;
-using namespace libtorrent;
 
 struct unicode_from_python
 {
@@ -32,36 +27,35 @@ struct unicode_from_python
 
     static void construct(PyObject* x, converter::rvalue_from_python_stage1_data* data)
     {
-        using libtorrent::wchar_utf8;
         void* storage = ((converter::rvalue_from_python_storage<
             std::string>*)data)->storage.bytes;
 
         if (PyUnicode_Check(x))
         {
-            std::wstring str;
-            str.resize(PyUnicode_GetSize(x) + 1, 0);
-#if PY_VERSION_HEX >= 0x03020000
-            int len = PyUnicode_AsWideChar(x, &str[0], str.size());
-#else
-            int len = PyUnicode_AsWideChar((PyUnicodeObject*)x, &str[0], str.size());
-#endif
-            if (len > -1)
+            PyObject* utf8 = PyUnicode_AsUTF8String(x);
+            if (utf8 == NULL)
             {
-               assert(len < int(str.size()));
-               str[len] = 0;
+               new (storage) std::string();
+            }
+            else
+            {
+#if PY_VERSION_HEX >= 0x03000000
+               new (storage) std::string(PyBytes_AsString(utf8)
+                  , PyBytes_Size(utf8));
+#else
+               new (storage) std::string(PyString_AsString(utf8)
+                     , PyString_Size(utf8));
+#endif
+               Py_DECREF(utf8);
             }
-            else str[str.size()-1] = 0;
-
-            std::string utf8;
-            wchar_utf8(str, utf8);
-            new (storage) std::string(utf8);
         }
         else
         {
 #if PY_VERSION_HEX >= 0x03000000
-            new (storage) std::string(PyBytes_AsString(x));
+            new (storage) std::string(PyBytes_AsString(x), PyBytes_Size(x));
 #else
-            new (storage) std::string(PyString_AsString(x));
+            new (storage) std::string(PyString_AsString(x)
+               , PyString_Size(x));
 #endif
         }
         data->convertible = storage;
@@ -73,5 +67,3 @@ void bind_unicode_string_conversion()
     unicode_from_python();
 }
 
-#endif // TORRENT_USE_WSTRING
-
