@@ -41,6 +41,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <libtorrent/alert_types.hpp>
 #include <libtorrent/bencode.hpp>
 #include <libtorrent/torrent_status.hpp>
+#include <libtorrent/magnet_uri.hpp>
+#include <libtorrent/error_code.hpp>
 
 namespace lt = libtorrent;
 using clk = std::chrono::steady_clock;
@@ -82,7 +84,12 @@ int main(int argc, char const* argv[])
 	ifs.unsetf(std::ios_base::skipws);
 	atp.resume_data.assign(std::istream_iterator<char>(ifs)
 		, std::istream_iterator<char>());
-	atp.url = argv[1];
+	lt::error_code ec;
+	lt::parse_magnet_uri(argv[1], atp, ec);
+	if (ec) {
+		std::cerr << "invalid magnet URI: " << ec.message() << std::endl;
+		return 1;
+	}
 	atp.save_path = "."; // save in current dir
 	ses.async_add_torrent(atp);
 
