@@ -101,7 +101,7 @@ namespace libtorrent
 		char const* torrent_name() const;
 
 #ifndef TORRENT_NO_DEPRECATE
-		std::string name;
+		std::string TORRENT_DEPRECATED_MEMBER name;
 #endif
 
 	protected:
@@ -149,7 +149,7 @@ namespace libtorrent
 
 #ifndef TORRENT_NO_DEPRECATE
 		// The tracker URL
-		std::string url;
+		std::string TORRENT_DEPRECATED_MEMBER url;
 #endif
 	private:
 		int m_url_idx;
@@ -195,11 +195,14 @@ namespace libtorrent
 #define TORRENT_DEFINE_ALERT_PRIO(name, seq) \
 	TORRENT_DEFINE_ALERT_IMPL(name, seq, 1)
 
+#ifndef TORRENT_NO_DEPRECATE
 	// The ``torrent_added_alert`` is posted once every time a torrent is successfully
 	// added. It doesn't contain any members of its own, but inherits the torrent handle
 	// from its base class.
 	// It's posted when the ``status_notification`` bit is set in the alert_mask.
-	struct TORRENT_EXPORT torrent_added_alert TORRENT_FINAL : torrent_alert
+	// deprecated in 1.1.3
+	// use add_torrent_alert instead
+	struct TORRENT_DEPRECATED_EXPORT torrent_added_alert TORRENT_FINAL : torrent_alert
 	{
 		// internal
 		torrent_added_alert(aux::stack_allocator& alloc, torrent_handle const& h);
@@ -208,6 +211,7 @@ namespace libtorrent
 		static const int static_category = alert::status_notification;
 		virtual std::string message() const TORRENT_OVERRIDE;
 	};
+#endif
 
 	// The ``torrent_removed_alert`` is posted whenever a torrent is removed. Since
 	// the torrent handle in its base class will always be invalid (since the torrent
@@ -290,7 +294,7 @@ namespace libtorrent
 		static const int static_category = alert::storage_notification;
 		virtual std::string message() const TORRENT_OVERRIDE;
 #ifndef TORRENT_NO_DEPRECATE
-		std::string name;
+		std::string TORRENT_DEPRECATED_MEMBER name;
 #endif
 
 		char const* new_name() const;
@@ -469,7 +473,7 @@ namespace libtorrent
 		int status_code;
 		error_code error;
 #ifndef TORRENT_NO_DEPRECATE
-		std::string msg;
+		std::string TORRENT_DEPRECATED_MEMBER msg;
 #endif
 
 		// the message associated with this error
@@ -497,7 +501,7 @@ namespace libtorrent
 
 #ifndef TORRENT_NO_DEPRECATE
 		// contains the warning message from the tracker.
-		std::string msg;
+		std::string TORRENT_DEPRECATED_MEMBER msg;
 #endif
 
 		// the message associated with this warning
@@ -549,7 +553,7 @@ namespace libtorrent
 
 #ifndef TORRENT_NO_DEPRECATE
 		// contains a message describing the error.
-		std::string msg;
+		std::string TORRENT_DEPRECATED_MEMBER msg;
 #endif
 
 		// the error itself. This may indicate that the tracker sent an error
@@ -581,7 +585,7 @@ namespace libtorrent
 		virtual std::string message() const TORRENT_OVERRIDE;
 
 		// tells how many peers the tracker returned in this response. This is
-		// not expected to be more thant the ``num_want`` settings. These are not necessarily
+		// not expected to be greater than the ``num_want`` settings. These are not necessarily
 		// all new peers, some of them may already be connected.
 		int num_peers;
 	};
@@ -704,7 +708,7 @@ namespace libtorrent
 		error_code error;
 
 #ifndef TORRENT_NO_DEPRECATE
-		std::string msg;
+		std::string TORRENT_DEPRECATED_MEMBER msg;
 #endif
 	};
 
@@ -752,7 +756,7 @@ namespace libtorrent
 		close_reason_t reason;
 
 #ifndef TORRENT_NO_DEPRECATE
-		std::string msg;
+		std::string TORRENT_DEPRECATED_MEMBER msg;
 #endif
 	};
 
@@ -782,7 +786,7 @@ namespace libtorrent
 		bool peer_interested;
 
 		// if this is true, the peer is not allowed to download this piece because
-		// of superseeding rules.
+		// of super-seeding rules.
 		bool withheld;
 	};
 
@@ -886,7 +890,7 @@ namespace libtorrent
 		virtual std::string message() const TORRENT_OVERRIDE;
 
 #ifndef TORRENT_NO_DEPRECATE
-		char const* peer_speedmsg;
+		char const* TORRENT_DEPRECATED_MEMBER peer_speedmsg;
 #endif
 		int block_index;
 		int piece_index;
@@ -909,23 +913,24 @@ namespace libtorrent
 		int piece_index;
 	};
 
-	// The ``storage_moved_alert`` is generated when all the disk IO has completed and the
-	// files have been moved, as an effect of a call to ``torrent_handle::move_storage``. This
-	// is useful to synchronize with the actual disk. The ``path`` member is the new path of
-	// the storage.
+	// The ``storage_moved_alert`` is generated when all the disk IO has
+	// completed and the files have been moved, as an effect of a call to
+	// ``torrent_handle::move_storage``. This is useful to synchronize with the
+	// actual disk. The ``storage_path()`` member return the new path of the
+	// storage.
 	struct TORRENT_EXPORT storage_moved_alert TORRENT_FINAL : torrent_alert
 	{
 		// internal
 		storage_moved_alert(aux::stack_allocator& alloc
 			, torrent_handle const& h, std::string const& p);
 
-		TORRENT_DEFINE_ALERT(storage_moved_alert, 33)
+		TORRENT_DEFINE_ALERT_PRIO(storage_moved_alert, 33)
 
 		static const int static_category = alert::storage_notification;
 		virtual std::string message() const TORRENT_OVERRIDE;
 
 #ifndef TORRENT_NO_DEPRECATE
-		std::string path;
+		std::string TORRENT_DEPRECATED_MEMBER path;
 #endif
 
 		// the path the torrent was moved to
@@ -946,7 +951,7 @@ namespace libtorrent
 			, std::string const& file
 			, char const* op);
 
-		TORRENT_DEFINE_ALERT(storage_moved_failed_alert, 34)
+		TORRENT_DEFINE_ALERT_PRIO(storage_moved_failed_alert, 34)
 
 		static const int static_category = alert::storage_notification;
 		virtual std::string message() const TORRENT_OVERRIDE;
@@ -955,7 +960,7 @@ namespace libtorrent
 
 #ifndef TORRENT_NO_DEPRECATE
 		// If the error happened for a specific file, ``file`` is its path.
-		std::string file;
+		std::string TORRENT_DEPRECATED_MEMBER file;
 #endif
 
 		// If the error happened for a specific file, this returns its path.
@@ -1013,7 +1018,7 @@ namespace libtorrent
 		sha1_hash info_hash;
 
 #ifndef TORRENT_NO_DEPRECATE
-		std::string msg;
+		std::string TORRENT_DEPRECATED_MEMBER msg;
 #endif
 	};
 
@@ -1053,7 +1058,7 @@ namespace libtorrent
 		error_code error;
 
 #ifndef TORRENT_NO_DEPRECATE
-		std::string msg;
+		std::string TORRENT_DEPRECATED_MEMBER msg;
 #endif
 	};
 
@@ -1113,10 +1118,10 @@ namespace libtorrent
 
 #ifndef TORRENT_NO_DEPRECATE
 		// the HTTP seed that failed
-		std::string url;
+		std::string TORRENT_DEPRECATED_MEMBER url;
 
 		// the error message, potentially from the server
-		std::string msg;
+		std::string TORRENT_DEPRECATED_MEMBER msg;
 #endif
 
 		// the error the web seed encountered. If this is not set, the server
@@ -1155,7 +1160,7 @@ namespace libtorrent
 
 #ifndef TORRENT_NO_DEPRECATE
 		// the path to the file that was accessed when the error occurred.
-		std::string file;
+		std::string TORRENT_DEPRECATED_MEMBER file;
 #endif
 
 		// the error code describing the error.
@@ -1166,7 +1171,7 @@ namespace libtorrent
 		char const* filename() const;
 
 #ifndef TORRENT_NO_DEPRECATE
-		std::string msg;
+		std::string TORRENT_DEPRECATED_MEMBER msg;
 #endif
 	private:
 		int m_file_idx;
@@ -1373,7 +1378,7 @@ namespace libtorrent
 		// tells you what failed.
 		error_code error;
 #ifndef TORRENT_NO_DEPRECATE
-		std::string msg;
+		std::string TORRENT_DEPRECATED_MEMBER msg;
 #endif
 	};
 
@@ -1432,7 +1437,7 @@ namespace libtorrent
 		int map_type;
 
 #ifndef TORRENT_NO_DEPRECATE
-		std::string msg;
+		std::string TORRENT_DEPRECATED_MEMBER msg;
 #endif
 
 		// the message associated with this log line
@@ -1440,7 +1445,7 @@ namespace libtorrent
 
 	private:
 
-		// TODO: 2 should the alert baseclass have this object instead?
+		// TODO: 2 should the alert base class have this object instead?
 		aux::stack_allocator const& m_alloc;
 
 		int m_log_idx;
@@ -1470,7 +1475,7 @@ namespace libtorrent
 
 #ifndef TORRENT_NO_DEPRECATE
 		// If the error happened to a specific file, ``file`` is the path to it.
-		std::string file;
+		std::string TORRENT_DEPRECATED_MEMBER file;
 #endif
 
 		// If the error happened to a specific file, this returns the path to it.
@@ -1481,7 +1486,7 @@ namespace libtorrent
 		char const* operation;
 
 #ifndef TORRENT_NO_DEPRECATE
-		std::string msg;
+		std::string TORRENT_DEPRECATED_MEMBER msg;
 #endif
 	private:
 		int m_path_idx;
@@ -1559,6 +1564,8 @@ namespace libtorrent
 	// This alert is posted approximately once every second, and it contains
 	// byte counters of most statistics that's tracked for torrents. Each active
 	// torrent posts these alerts regularly.
+	// This alert has been superceded by calling ``post_torrent_updates()``
+	// regularly on the session object. This alert will be removed
 	struct TORRENT_EXPORT stats_alert TORRENT_FINAL : torrent_alert
 	{
 		// internal
@@ -1578,16 +1585,16 @@ namespace libtorrent
 			download_protocol,
 			upload_ip_protocol,
 #ifndef TORRENT_NO_DEPRECATE
-			upload_dht_protocol,
-			upload_tracker_protocol,
+			upload_dht_protocol TORRENT_DEPRECATED_ENUM,
+			upload_tracker_protocol TORRENT_DEPRECATED_ENUM,
 #else
 			deprecated1,
 			deprecated2,
 #endif
 			download_ip_protocol,
 #ifndef TORRENT_NO_DEPRECATE
-			download_dht_protocol,
-			download_tracker_protocol,
+			download_dht_protocol TORRENT_DEPRECATED_ENUM,
+			download_tracker_protocol TORRENT_DEPRECATED_ENUM,
 #else
 			deprecated3,
 			deprecated4,
@@ -1680,7 +1687,7 @@ namespace libtorrent
 
 #ifndef TORRENT_NO_DEPRECATE
 		// The tracker ID returned by the tracker
-		std::string trackerid;
+		std::string TORRENT_DEPRECATED_MEMBER trackerid;
 #endif
 
 		// The tracker ID returned by the tracker
@@ -1767,7 +1774,7 @@ namespace libtorrent
 
 #ifndef TORRENT_NO_DEPRECATE
 		// the filename (or object) the error occurred on.
-		std::string error_file;
+		std::string TORRENT_DEPRECATED_MEMBER error_file;
 #endif
 
 		// the filename (or object) the error occurred on.
@@ -1878,7 +1885,8 @@ namespace libtorrent
 		std::vector<torrent_status> status;
 	};
 
-	struct TORRENT_EXPORT mmap_cache_alert TORRENT_FINAL : alert
+#ifndef TORRENT_NO_DEPRECATE
+	struct TORRENT_DEPRECATED_EXPORT mmap_cache_alert TORRENT_FINAL : alert
 	{
 		mmap_cache_alert(aux::stack_allocator& alloc
 			, error_code const& ec);
@@ -1889,6 +1897,7 @@ namespace libtorrent
 
 		error_code error;
 	};
+#endif
 
 	// The session_stats_alert is posted when the user requests session statistics by
 	// calling post_session_stats() on the session object. Its category is
@@ -1999,7 +2008,7 @@ namespace libtorrent
 		virtual std::string message() const TORRENT_OVERRIDE;
 
 		// the target hash of the immutable item. This must
-		// match the sha-1 hash of the bencoded form of ``item``.
+		// match the SHA-1 hash of the bencoded form of ``item``.
 		sha1_hash target;
 
 		// the data for this item
@@ -2446,8 +2455,10 @@ namespace libtorrent
 	// this is posted when one or more blocks are picked by the piece picker,
 	// assuming the verbose piece picker logging is enabled (see
 	// picker_log_notification).
-	struct TORRENT_EXPORT picker_log_alert : peer_alert
+	struct TORRENT_EXPORT picker_log_alert TORRENT_FINAL : peer_alert
 	{
+		TORRENT_DEFINE_ALERT(picker_log_alert, 89)
+
 #ifndef TORRENT_DISABLE_LOGGING
 
 		// internal
@@ -2455,8 +2466,6 @@ namespace libtorrent
 			, tcp::endpoint const& ep, peer_id const& peer_id, boost::uint32_t flags
 			, piece_block const* blocks, int num_blocks);
 
-		TORRENT_DEFINE_ALERT(picker_log_alert, 89)
-
 		static const int static_category = alert::picker_log_notification;
 		virtual std::string message() const TORRENT_OVERRIDE;
 
