@@ -92,9 +92,19 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/aux_/disable_warnings_pop.hpp"
 
+// ====== CLANG ========
+
+#if defined __clang__
+
+# if !defined TORRENT_BUILDING_LIBRARY
+// TODO: figure out which version of clang this is supported in
+#  define TORRENT_DEPRECATED_ENUM __attribute__ ((deprecated))
+#  define TORRENT_DEPRECATED_MEMBER __attribute__ ((deprecated))
+# endif
+
 // ======= GCC =========
 
-#if defined __GNUC__
+#elif defined __GNUC__
 
 // deprecation markup is only enabled when libtorrent
 // headers are included by clients, not while building
@@ -103,6 +113,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #  define TORRENT_DEPRECATED __attribute__ ((deprecated))
 # endif
 
+# if __GNUC__ >= 6 && !defined TORRENT_BUILDING_LIBRARY
+#  define TORRENT_DEPRECATED_ENUM __attribute__ ((deprecated))
+#  define TORRENT_DEPRECATED_MEMBER __attribute__ ((deprecated))
+# endif
+
 // ======= SUNPRO =========
 
 #elif defined __SUNPRO_CC
@@ -179,13 +194,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #if MAC_OS_X_VERSION_MIN_REQUIRED >= 1050
 #define TORRENT_USE_EXECINFO 1
 #endif
-
-#else // __APPLE__
-// FreeBSD has a reasonable iconv signature
-// unless we're on glibc
-#ifndef __GLIBC__
-# define TORRENT_ICONV_ARG (const char**)
-#endif
 #endif // __APPLE__
 
 #define TORRENT_HAVE_MMAP 1
@@ -216,6 +224,8 @@ POSSIBILITY OF SUCH DAMAGE.
 
 // ===== ANDROID ===== (almost linux, sort of)
 #if defined __ANDROID__
+#define TORRENT_USE_PREADV 0
+#define TORRENT_USE_PREAD 1
 #define TORRENT_ANDROID
 #define TORRENT_HAS_FALLOCATE 0
 #define TORRENT_USE_ICONV 0
@@ -263,7 +273,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_USE_PWRITEV 1
 
 // ==== WINDOWS ===
-#elif defined WIN32
+#elif defined _WIN32
 #define TORRENT_WINDOWS
 #ifndef TORRENT_USE_GETIPFORWARDTABLE
 # define TORRENT_USE_GETIPFORWARDTABLE 1
@@ -327,7 +337,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_USE_IFCONF 1
 #define TORRENT_USE_SYSCTL 1
 #define TORRENT_USE_IPV6 0
-#define TORRENT_ICONV_ARG (const char**)
 #define TORRENT_USE_WRITEV 0
 #define TORRENT_USE_READV 0
 
@@ -439,10 +448,6 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 #define TORRENT_OVERRIDE override
 #endif
 
-#ifndef TORRENT_ICONV_ARG
-#define TORRENT_ICONV_ARG (char**)
-#endif
-
 #if defined __GNUC__ || defined __clang__
 #define TORRENT_FORMAT(fmt, ellipsis) __attribute__((__format__(__printf__, fmt, ellipsis)))
 #else
@@ -530,6 +535,14 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 #define TORRENT_DEPRECATED
 #endif
 
+#ifndef TORRENT_DEPRECATED_ENUM
+#define TORRENT_DEPRECATED_ENUM
+#endif
+
+#ifndef TORRENT_DEPRECATED_MEMBER
+#define TORRENT_DEPRECATED_MEMBER
+#endif
+
 #ifndef TORRENT_USE_COMMONCRYPTO
 #define TORRENT_USE_COMMONCRYPTO 0
 #endif
@@ -695,7 +708,7 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 #if (defined _M_AMD64 || defined _M_IX86 || defined _M_X64 \
 	|| defined __amd64__ || defined __i386 || defined __i386__ \
 	|| defined __x86_64__ || defined __x86_64) \
-	&& (defined __GNUC__ || defined _MSC_VER)
+	&& (defined __GNUC__ || (defined _MSC_VER && _MSC_VER >= 1600))
 #define TORRENT_HAS_SSE 1
 #else
 #define TORRENT_HAS_SSE 0
@@ -703,6 +716,9 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 
 #endif // TORRENT_HAS_SSE
 
+namespace libtorrent {}
+
+namespace lt = libtorrent;
 
 #endif // TORRENT_CONFIG_HPP_INCLUDED
 
