@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2009-2014, Arvid Norberg
+Copyright (c) 2009-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -33,69 +33,24 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef TORRENT_DEADLINE_TIMER_HPP_INCLUDED
 #define TORRENT_DEADLINE_TIMER_HPP_INCLUDED
 
-#ifdef __OBJC__
-#define Protocol Protocol_
+#include "libtorrent/config.hpp"
+
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+
+#include <boost/asio/high_resolution_timer.hpp>
+
+#if defined TORRENT_BUILD_SIMULATOR
+#include "simulator/simulator.hpp"
 #endif
 
-#if __GNUC__ < 3
-// in GCC 2.95 templates seems to have all symbols
-// resolved as they are parsed, so the time_traits
-// template actually needs the definitions it uses,
-// even though it's never instantiated
-#include <boost/date_time/posix_time/posix_time_types.hpp>
-#else
-#include <boost/date_time/posix_time/posix_time_duration.hpp>
-#endif
-
-#if BOOST_VERSION < 103500
-#include <asio/basic_deadline_timer.hpp>
-#else
-#include <boost/asio/basic_deadline_timer.hpp>
-#endif
-
-#ifdef __OBJC__ 
-#undef Protocol
-#endif
-
-#include "libtorrent/time.hpp"
-
-// asio time_traits
-#if !TORRENT_USE_BOOST_DATE_TIME
-#if BOOST_VERSION >= 103500
-namespace boost { 
-#endif
-namespace asio
-{
-	template<>
-	struct time_traits<libtorrent::ptime>
-	{
-		typedef libtorrent::ptime time_type;
-		typedef libtorrent::time_duration duration_type;
-		static time_type now()
-		{ return time_type(libtorrent::time_now_hires()); }
-		static time_type add(time_type t, duration_type d)
-		{ return time_type(t.time + d.diff);}
-		static duration_type subtract(time_type t1, time_type t2)
-		{ return duration_type(t1 - t2); }
-		static bool less_than(time_type t1, time_type t2)
-		{ return t1 < t2; }
-		static boost::posix_time::time_duration to_posix_duration(
-			duration_type d)
-		{ return boost::posix_time::microseconds(libtorrent::total_microseconds(d)); }
-	};
-}
-#if BOOST_VERSION >= 103500
-}
-#endif
-#endif
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
 
 namespace libtorrent
 {
-
-#if BOOST_VERSION < 103500
-	typedef ::asio::basic_deadline_timer<libtorrent::ptime> deadline_timer;
+#if defined TORRENT_BUILD_SIMULATOR
+	typedef sim::asio::high_resolution_timer deadline_timer;
 #else
-	typedef boost::asio::basic_deadline_timer<libtorrent::ptime> deadline_timer;
+	typedef boost::asio::high_resolution_timer deadline_timer;
 #endif
 }
 
