@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2009-2014, Arvid Norberg
+Copyright (c) 2009-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -41,21 +41,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <winsock2.h>
 #endif
 
-#if BOOST_VERSION < 103500
-#include <asio/error.hpp>
-#else
 #include <boost/asio/error.hpp>
-#endif
 
 namespace libtorrent
 {
-
-#if BOOST_VERSION < 103500
-	namespace error = asio::error;
-#else
 	namespace error = boost::asio::error;
-#endif
-
 }
 
 #endif
