@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007, Arvid Norberg
+Copyright (c) 2007-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -34,20 +34,23 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_INSTANTIATE_CONNECTION
 
 #include "libtorrent/socket_type.hpp"
-#include <boost/shared_ptr.hpp>
 
 namespace libtorrent
 {
-	struct proxy_settings;
+	namespace aux {
+		struct proxy_settings;
+	}
+
 	struct utp_socket_manager;
 	struct socket_type;
 
 	// instantiate a socket_type (s) according to the specified criteria
 	TORRENT_EXTRA_EXPORT bool instantiate_connection(io_service& ios
-		, proxy_settings const& ps, socket_type& s
-		, void* ssl_context = 0
-		, utp_socket_manager* sm = 0
-		, bool peer_connection = false);
+		, aux::proxy_settings const& ps, socket_type& s
+		, void* ssl_context
+		, utp_socket_manager* sm
+		, bool peer_connection
+		, bool tracker_connection);
 }
 
 #endif
