@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2006-2014, Arvid Norberg
+Copyright (c) 2006-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -37,56 +37,19 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/socket.hpp"
 #include "libtorrent/address.hpp"
 #include "libtorrent/union_endpoint.hpp"
-#include "libtorrent/time.hpp" // for time_now()
-
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-#include "libtorrent/time.hpp"
-#endif
+#include "libtorrent/time.hpp" // for time_point
 
 namespace libtorrent { namespace dht
 {
 
-struct node_entry
+struct TORRENT_EXTRA_EXPORT node_entry
 {
 	node_entry(node_id const& id_, udp::endpoint ep, int roundtriptime = 0xffff
-		, bool pinged = false)
-		: last_queried(pinged ? time_now() : min_time())
-		, id(id_)
-		, a(ep.address().to_v4().to_bytes())
-		, p(ep.port())
-		, rtt(roundtriptime & 0xffff)
-		, timeout_count(pinged ? 0 : 0xff)
-	{
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-		first_seen = time_now();
-#endif
-	}
+		, bool pinged = false);
+	node_entry(udp::endpoint ep);
+	node_entry();
+	void update_rtt(int new_rtt);
 
-	node_entry(udp::endpoint ep)
-		: last_queried(min_time())
-		, id(0)
-		, a(ep.address().to_v4().to_bytes())
-		, p(ep.port())
-		, rtt(0xffff)
-		, timeout_count(0xff)
-	{
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-		first_seen = time_now();
-#endif
-	}
-
-	node_entry()
-		: last_queried(min_time())
-		, id(0)
-		, p(0)
-		, rtt(0xffff)
-		, timeout_count(0xff)
-	{
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-		first_seen = time_now();
-#endif
-	}
-	
 	bool pinged() const { return timeout_count != 0xff; }
 	void set_pinged() { if (timeout_count == 0xff) timeout_count = 0; }
 	void timed_out() { if (pinged() && timeout_count < 0xfe) ++timeout_count; }
@@ -94,23 +57,15 @@ struct node_entry
 	void reset_fail_count() { if (pinged()) timeout_count = 0; }
 	udp::endpoint ep() const { return udp::endpoint(address_v4(a), p); }
 	bool confirmed() const { return timeout_count == 0; }
-	void update_rtt(int new_rtt)
-	{
-		TORRENT_ASSERT(new_rtt <= 0xffff);
-		TORRENT_ASSERT(new_rtt >= 0);
-		if (new_rtt == 0xffff) return;
-		if (rtt == 0xffff) rtt = new_rtt;
-		else rtt = int(rtt) * 2 / 3 + int(new_rtt) / 3;
-	}
 	address addr() const { return address_v4(a); }
 	int port() const { return p; }
 
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-	ptime first_seen;
+#ifndef TORRENT_DISABLE_LOGGING
+	time_point first_seen;
 #endif
 
 	// the time we last received a response for a request to this peer
-	ptime last_queried;
+	time_point last_queried;
 
 	node_id id;
 
