@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2006-2014, Arvid Norberg & Daniel Wallin
+Copyright (c) 2006-2016, Arvid Norberg & Daniel Wallin
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -38,27 +38,27 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include <libtorrent/kademlia/node_id.hpp>
 #include <libtorrent/kademlia/routing_table.hpp>
-#include <libtorrent/kademlia/logging.hpp>
 #include <libtorrent/kademlia/observer.hpp>
 #include <libtorrent/address.hpp>
 
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+
 #include <boost/noncopyable.hpp>
 #include <boost/intrusive_ptr.hpp>
 #include <boost/bind.hpp>
 #include <boost/pool/pool.hpp>
 
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
+
 namespace libtorrent { struct dht_lookup; }
 namespace libtorrent { namespace dht
 {
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-TORRENT_DECLARE_LOG(traversal);
-#endif
 
 class rpc_manager;
-class node_impl;
+class node;
 
 // this class may not be instantiated as a stack object
-struct traversal_algorithm : boost::noncopyable
+struct TORRENT_EXTRA_EXPORT traversal_algorithm : boost::noncopyable
 {
 	void traverse(node_id const& id, udp::endpoint addr);
 	void finished(observer_ptr o);
@@ -67,7 +67,6 @@ struct traversal_algorithm : boost::noncopyable
 	void failed(observer_ptr o, int flags = 0);
 	virtual ~traversal_algorithm();
 	void status(dht_lookup& l);
-	void abort();
 
 	void* allocate_observer();
 	void free_observer(void* ptr);
@@ -80,11 +79,11 @@ struct traversal_algorithm : boost::noncopyable
 	void resort_results();
 	void add_entry(node_id const& id, udp::endpoint addr, unsigned char flags);
 
-	traversal_algorithm(node_impl& node, node_id target);
+	traversal_algorithm(node & node, node_id target);
 	int invoke_count() const { return m_invoke_count; }
 	int branch_factor() const { return m_branch_factor; }
 
-	node_impl& node() const { return m_node; }
+	node& get_node() const { return m_node; }
 
 protected:
 
@@ -100,10 +99,11 @@ protected:
 	virtual observer_ptr new_observer(void* ptr
 		, udp::endpoint const& ep, node_id const& id);
 
-	virtual bool invoke(observer_ptr o) { return false; }
+	virtual bool invoke(observer_ptr) { return false; }
 
 	friend void intrusive_ptr_add_ref(traversal_algorithm* p)
 	{
+		TORRENT_ASSERT(p->m_ref_count < 0xffff);
 		p->m_ref_count++;
 	}
 
@@ -113,16 +113,19 @@ protected:
 			delete p;
 	}
 
-	int m_ref_count;
-
-	node_impl& m_node;
-	node_id const m_target;
+	node & m_node;
 	std::vector<observer_ptr> m_results;
-	int m_invoke_count;
-	int m_branch_factor;
-	int m_responses;
-	int m_timeouts;
-	int m_num_target_nodes;
+	node_id const m_target;
+	boost::uint16_t m_ref_count;
+	boost::int16_t m_invoke_count;
+	boost::int16_t m_branch_factor;
+	boost::int16_t m_responses;
+	boost::int16_t m_timeouts;
+
+	// the IP addresses of the nodes in m_results
+	std::set<boost::uint32_t> m_peer4_prefixes;
+// no IPv6 support yet anyway
+//	std::set<boost::uint64_t> m_peer6_prefixes;
 };
 
 struct traversal_observer : observer
