@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2009-2014, Arvid Norberg
+Copyright (c) 2009-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -32,13 +32,14 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/allocator.hpp"
 #include "libtorrent/config.hpp"
-#include "libtorrent/assert.hpp"
+#include "libtorrent/assert.hpp" // for print_backtrace
+#include <boost/cstdint.hpp>
 
 #if defined TORRENT_BEOS
 #include <kernel/OS.h>
 #include <stdlib.h> // malloc/free
 #elif !defined TORRENT_WINDOWS
-#include <stdlib.h> // valloc/free
+#include <stdlib.h> // posix_memalign/free
 #include <unistd.h> // _SC_PAGESIZE
 #endif
 
@@ -49,7 +50,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #ifdef TORRENT_WINDOWS
 // windows.h must be included after stdlib.h under mingw
-#include <windows.h> 
+#include <windows.h>
 #endif
 
 #ifdef TORRENT_MINGW
@@ -61,11 +62,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef TORRENT_WINDOWS
 #include <sys/mman.h>
 #endif
-#include "libtorrent/size_type.hpp"
 
 struct alloc_header
 {
-	libtorrent::size_type size;
+	boost::int64_t size;
 	int magic;
 	char stack[3072];
 };
@@ -80,14 +80,16 @@ namespace libtorrent
 		static int s = 0;
 		if (s != 0) return s;
 
-#ifdef TORRENT_WINDOWS
+#ifdef TORRENT_BUILD_SIMULATOR
+		s = 4096;
+#elif defined TORRENT_WINDOWS
 		SYSTEM_INFO si;
 		GetSystemInfo(&si);
 		s = si.dwPageSize;
 #elif defined TORRENT_BEOS
 		s = B_PAGE_SIZE;
 #else
-		s = sysconf(_SC_PAGESIZE);
+		s = int(sysconf(_SC_PAGESIZE));
 #endif
 		// assume the page size is 4 kiB if we
 		// fail to query it
@@ -95,7 +97,7 @@ namespace libtorrent
 		return s;
 	}
 
-	char* page_aligned_allocator::malloc(size_type bytes)
+	char* page_aligned_allocator::malloc(page_aligned_allocator::size_type bytes)
 	{
 		TORRENT_ASSERT(bytes > 0);
 		// just sanity check (this needs to be pretty high
@@ -104,32 +106,32 @@ namespace libtorrent
 
 		TORRENT_ASSERT(int(bytes) >= page_size());
 #ifdef TORRENT_DEBUG_BUFFERS
-		int page = page_size();
-		int num_pages = (bytes + (page-1)) / page + 2;
-		int orig_bytes = bytes;
+		const int page = page_size();
+		const int num_pages = (bytes + (page-1)) / page + 2;
+		const int orig_bytes = bytes;
 		bytes = num_pages * page;
 #endif
 
-		char* ret;
+		void* ret;
 #if TORRENT_USE_POSIX_MEMALIGN
-		if (posix_memalign((void**)&ret, page_size(), bytes) != 0) ret = NULL;
+		if (posix_memalign(&ret, page_size(), bytes)
+			!= 0) ret = NULL;
 #elif TORRENT_USE_MEMALIGN
-		ret = (char*)memalign(page_size(), bytes);
+		ret = memalign(page_size(), bytes);
 #elif defined TORRENT_WINDOWS
-		ret = (char*)_aligned_malloc(bytes, page_size());
+		ret = _aligned_malloc(bytes, page_size());
 #elif defined TORRENT_BEOS
 		area_id id = create_area("", &ret, B_ANY_ADDRESS
 			, (bytes + page_size() - 1) & (page_size()-1), B_NO_LOCK, B_READ_AREA | B_WRITE_AREA);
 		if (id < B_OK) return NULL;
-		ret = (char*)ret;
 #else
-		ret = (char*)valloc(bytes);
+		ret = valloc(size_t(bytes));
 #endif
 		if (ret == NULL) return NULL;
 
 #ifdef TORRENT_DEBUG_BUFFERS
 		// make the two surrounding pages non-readable and -writable
-		alloc_header* h = (alloc_header*)ret;
+		alloc_header* h = static_cast<alloc_header*>(ret);
 		h->size = orig_bytes;
 		h->magic = 0x1337;
 		print_backtrace(h->stack, sizeof(h->stack));
@@ -139,7 +141,7 @@ namespace libtorrent
 #define PROT_READ PAGE_READONLY
 #endif
 		mprotect(ret, page, PROT_READ);
-		mprotect(ret + (num_pages-1) * page, page, PROT_READ);
+		mprotect(static_cast<char*>(ret) + (num_pages-1) * page, page, PROT_READ);
 
 #ifdef TORRENT_WINDOWS
 #undef mprotect
@@ -147,10 +149,10 @@ namespace libtorrent
 #endif
 //		fprintf(stderr, "malloc: %p head: %p tail: %p size: %d\n", ret + page, ret, ret + page + bytes, int(bytes));
 
-		return ret + page;
+		return static_cast<char*>(ret) + page;
+#else
+		return static_cast<char*>(ret);
 #endif // TORRENT_DEBUG_BUFFERS
-
-		return ret;
 	}
 
 	void page_aligned_allocator::free(char* block)
@@ -164,11 +166,11 @@ namespace libtorrent
 #define PROT_READ PAGE_READONLY
 #define PROT_WRITE PAGE_READWRITE
 #endif
-		int page = page_size();
+		const int page = page_size();
 		// make the two surrounding pages non-readable and -writable
 		mprotect(block - page, page, PROT_READ | PROT_WRITE);
-		alloc_header* h = (alloc_header*)(block - page);
-		int num_pages = (h->size + (page-1)) / page + 2;
+		alloc_header* h = reinterpret_cast<alloc_header*>(block - page);
+		const int num_pages = (h->size + (page-1)) / page + 2;
 		TORRENT_ASSERT(h->magic == 0x1337);
 		mprotect(block + (num_pages-2) * page, page, PROT_READ | PROT_WRITE);
 //		fprintf(stderr, "free: %p head: %p tail: %p size: %d\n", block, block - page, block + h->size, int(h->size));
@@ -196,6 +198,14 @@ namespace libtorrent
 #endif // TORRENT_WINDOWS
 	}
 
+#ifdef TORRENT_DEBUG_BUFFERS
+	bool page_aligned_allocator::in_use(char const* block)
+	{
+		const int page = page_size();
+		alloc_header const* h = reinterpret_cast<alloc_header const*>(block - page);
+		return h->magic == 0x1337;
+	}
+#endif
 
 }
 
