@@ -354,7 +354,7 @@ TORRENT_EXPORT void assert_fail(char const* expr, int line
 #ifndef TORRENT_PRODUCTION_ASSERTS
 	// send SIGINT to the current process
 	// to break into the debugger
-	raise(SIGINT);
+	raise(SIGABRT);
 	abort();
 #endif
 }
