@@ -126,10 +126,12 @@ namespace libtorrent
 	} // anonymous namespace
 
 
-	// fills in 'val' with what the string between start and the
-	// first occurance of the delimiter is interpreted as an int.
-	// return the pointer to the delimiter, or 0 if there is a
-	// parse error. val should be initialized to zero
+	// reads the string between start and end, or up to the first occurrance of
+	// 'delimiter', whichever comes first. This string is interpreted as an
+	// integer which is assigned to 'val'. If there's a non-delimiter and
+	// non-digit in the range, a parse error is reported in 'ec'. If the value
+	// cannot be represented by the variable 'val' and overflow error is reported
+	// by 'ec'.
 	char const* parse_int(char const* start, char const* end, char delimiter
 		, boost::int64_t& val, bdecode_errors::error_code_enum& ec)
 	{
@@ -155,8 +157,6 @@ namespace libtorrent
 			val += digit;
 			++start;
 		}
-		if (*start != delimiter)
-			ec = bdecode_errors::expected_colon;
 		return start;
 	}
 
@@ -193,17 +193,22 @@ namespace libtorrent
 		return msgs[ev];
 	}
 
-	boost::system::error_category& get_bdecode_category()
+	boost::system::error_category& bdecode_category()
 	{
 		static bdecode_error_category bdecode_category;
 		return bdecode_category;
 	}
 
+#ifndef TORRENT_NO_DEPRECATED
+	boost::system::error_category& get_bdecode_category()
+	{ return bdecode_category(); }
+#endif
+
 	namespace bdecode_errors
 	{
 		boost::system::error_code make_error_code(error_code_enum e)
 		{
-			return boost::system::error_code(e, get_bdecode_category());
+			return boost::system::error_code(e, bdecode_category());
 		}
 	}
 
@@ -346,7 +351,7 @@ namespace libtorrent
 	}
 
 	std::string bdecode_node::list_string_value_at(int i
-		, char const* default_val)
+		, char const* default_val) const
 	{
 		bdecode_node n = list_at(i);
 		if (n.type() != bdecode_node::string_t) return default_val;
@@ -354,7 +359,7 @@ namespace libtorrent
 	}
 
 	boost::int64_t bdecode_node::list_int_value_at(int i
-		, boost::int64_t default_val)
+		, boost::int64_t default_val) const
 	{
 		bdecode_node n = list_at(i);
 		if (n.type() != bdecode_node::int_t) return default_val;
@@ -613,16 +618,18 @@ namespace libtorrent
 		bdecode_token const& t = m_root_tokens[m_token_idx];
 		int size = m_root_tokens[m_token_idx + 1].offset - t.offset;
 		TORRENT_ASSERT(t.type == bdecode_token::integer);
-	
+
 		// +1 is to skip the 'i'
 		char const* ptr = m_buffer + t.offset + 1;
 		boost::int64_t val = 0;
 		bool negative = false;
 		if (*ptr == '-') negative = true;
 		bdecode_errors::error_code_enum ec = bdecode_errors::no_error;
-		parse_int(ptr + negative
+		char const* end = parse_int(ptr + negative
 			, ptr + size, 'e', val, ec);
 		if (ec) return 0;
+		TORRENT_UNUSED(end);
+		TORRENT_ASSERT(end < ptr + size);
 		if (negative) val = -val;
 		return val;
 	}
@@ -689,7 +696,7 @@ namespace libtorrent
 	}
 
 #define TORRENT_FAIL_BDECODE(code) do { \
-	ec = make_error_code(code); \
+	ec = code; \
 	if (error_pos) *error_pos = start - orig_start; \
 	goto done; \
 	} TORRENT_WHILE_0
@@ -703,7 +710,7 @@ namespace libtorrent
 		if (end - start > bdecode_token::max_offset)
 		{
 			if (error_pos) *error_pos = 0;
-			ec = make_error_code(bdecode_errors::limit_exceeded);
+			ec = bdecode_errors::limit_exceeded;
 			return -1;
 		}
 
@@ -834,10 +841,13 @@ namespace libtorrent
 					boost::int64_t len = t - '0';
 					char const* str_start = start;
 					++start;
+					if (start >= end) TORRENT_FAIL_BDECODE(bdecode_errors::unexpected_eof);
 					bdecode_errors::error_code_enum e = bdecode_errors::no_error;
 					start = parse_int(start, end, ':', len, e);
 					if (e)
 						TORRENT_FAIL_BDECODE(e);
+					if (start == end)
+						TORRENT_FAIL_BDECODE(bdecode_errors::expected_colon);
 
 					// remaining buffer size excluding ':'
 					const ptrdiff_t buff_size = end - start - 1;
