@@ -271,13 +271,6 @@ namespace libtorrent
 
 #endif
 			open_multicast_socket(ios, i->interface_address, loopback, ec);
-#ifdef TORRENT_DEBUG
-			fprintf(stderr, "broadcast socket [ if: %s group: %s mask: %s ] %s\n"
-				, i->interface_address.to_string().c_str()
-				, m_multicast_endpoint.address().to_string().c_str()
-				, i->netmask.to_string().c_str()
-				, ec.message().c_str());
-#endif
 			open_unicast_socket(ios, i->interface_address
 				, i->netmask.is_v4() ? i->netmask.to_v4() : address_v4());
 		}
