@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2008-2014, Arvid Norberg
+Copyright (c) 2008-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -37,34 +37,59 @@ POSSIBILITY OF SUCH DAMAGE.
 namespace libtorrent
 {
 
-	disk_buffer_holder::disk_buffer_holder(aux::session_impl& ses, char* buf)
-		: m_disk_pool(ses.m_disk_thread), m_buf(buf)
+	disk_buffer_holder::disk_buffer_holder(buffer_allocator_interface& alloc, char* buf)
+		: m_allocator(alloc), m_buf(buf)
 	{
-		TORRENT_ASSERT(buf == 0 || m_disk_pool.is_disk_buffer(buf));
+		m_ref.storage = 0;
+		m_ref.piece = -1;
+		m_ref.block = -1;
 	}
 
-	disk_buffer_holder::disk_buffer_holder(disk_buffer_pool& iothread, char* buf)
-		: m_disk_pool(iothread), m_buf(buf)
+	disk_buffer_holder::disk_buffer_holder(buffer_allocator_interface& alloc, disk_io_job const& j)
+		: m_allocator(alloc), m_buf(j.buffer.disk_block), m_ref(j.d.io.ref)
 	{
-		TORRENT_ASSERT(buf == 0 || m_disk_pool.is_disk_buffer(buf));
+		TORRENT_ASSERT(m_ref.storage == 0 || m_ref.piece >= 0);
+		TORRENT_ASSERT(m_ref.storage == 0 || m_ref.block >= 0);
+		TORRENT_ASSERT(m_ref.storage == 0 || m_ref.piece < static_cast<piece_manager*>(m_ref.storage)->files()->num_pieces());
+		TORRENT_ASSERT(m_ref.storage == 0 || m_ref.block <= static_cast<piece_manager*>(m_ref.storage)->files()->piece_length() / 0x4000);
+		TORRENT_ASSERT(j.action != disk_io_job::save_resume_data);
+		TORRENT_ASSERT(j.action != disk_io_job::rename_file);
+		TORRENT_ASSERT(j.action != disk_io_job::move_storage);
+	}
+
+	void disk_buffer_holder::reset(disk_io_job const& j)
+	{
+		if (m_ref.storage) m_allocator.reclaim_block(m_ref);
+		else if (m_buf) m_allocator.free_disk_buffer(m_buf);
+		m_buf = j.buffer.disk_block;
+		m_ref = j.d.io.ref;
+
+		TORRENT_ASSERT(m_ref.piece >= 0);
+		TORRENT_ASSERT(m_ref.storage != 0);
+		TORRENT_ASSERT(m_ref.block >= 0);
+		TORRENT_ASSERT(m_ref.piece < static_cast<piece_manager*>(m_ref.storage)->files()->num_pieces());
+		TORRENT_ASSERT(m_ref.block <= static_cast<piece_manager*>(m_ref.storage)->files()->piece_length() / 0x4000);
+		TORRENT_ASSERT(j.action != disk_io_job::save_resume_data);
+		TORRENT_ASSERT(j.action != disk_io_job::rename_file);
+		TORRENT_ASSERT(j.action != disk_io_job::move_storage);
 	}
 
 	void disk_buffer_holder::reset(char* buf)
 	{
-		if (m_buf) m_disk_pool.free_buffer(m_buf);
+		if (m_ref.storage) m_allocator.reclaim_block(m_ref);
+		else if (m_buf) m_allocator.free_disk_buffer(m_buf);
 		m_buf = buf;
+		m_ref.storage = 0;
 	}
 
 	char* disk_buffer_holder::release()
 	{
 		char* ret = m_buf;
 		m_buf = 0;
+		m_ref.storage = 0;
 		return ret;
 	}
 
-	disk_buffer_holder::~disk_buffer_holder()
-	{
-		if (m_buf) m_disk_pool.free_buffer(m_buf);
-	}
+	disk_buffer_holder::~disk_buffer_holder() { reset(); }
 }
 
