@@ -102,6 +102,7 @@ const unsigned long siocgifmtu = SIOCGIFMTU;
 namespace libtorrent { namespace
 {
 
+#if !defined TORRENT_BUILD_SIMULATOR
 	address inaddr_to_address(in_addr const* ina, int len = 4)
 	{
 		typedef boost::asio::ip::address_v4::bytes_type bytes_t;
@@ -157,7 +158,7 @@ namespace libtorrent { namespace
 			int read_len = recv(sock, buf, bufsize - msg_len, 0);
 			if (read_len < 0) return -1;
 
-			nl_hdr = (nlmsghdr*)buf;
+			nl_hdr = reinterpret_cast<nlmsghdr*>(buf);
 
 			if ((NLMSG_OK(nl_hdr, read_len) == 0) || (nl_hdr->nlmsg_type == NLMSG_ERROR))
 				return -1;
@@ -175,7 +176,7 @@ namespace libtorrent { namespace
 
 	bool parse_route(int s, nlmsghdr* nl_hdr, ip_route* rt_info)
 	{
-		rtmsg* rt_msg = (rtmsg*)NLMSG_DATA(nl_hdr);
+		rtmsg* rt_msg = reinterpret_cast<rtmsg*>(NLMSG_DATA(nl_hdr));
 
 		if((rt_msg->rtm_family != AF_INET && rt_msg->rtm_family != AF_INET6) || (rt_msg->rtm_table != RT_TABLE_MAIN
 			&& rt_msg->rtm_table != RT_TABLE_LOCAL))
@@ -183,40 +184,47 @@ namespace libtorrent { namespace
 
 		int if_index = 0;
 		int rt_len = RTM_PAYLOAD(nl_hdr);
-		for (rtattr* rt_attr = (rtattr*)RTM_RTA(rt_msg);
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wcast-align"
+#endif
+		for (rtattr* rt_attr = reinterpret_cast<rtattr*>(RTM_RTA(rt_msg));
 			RTA_OK(rt_attr,rt_len); rt_attr = RTA_NEXT(rt_attr,rt_len))
 		{
 			switch(rt_attr->rta_type)
 			{
 				case RTA_OIF:
-					if_index = *(int*)RTA_DATA(rt_attr);
+					if_index = *reinterpret_cast<int*>(RTA_DATA(rt_attr));
 					break;
 				case RTA_GATEWAY:
 #if TORRENT_USE_IPV6
 					if (rt_msg->rtm_family == AF_INET6)
 					{
-						rt_info->gateway = inaddr6_to_address((in6_addr*)RTA_DATA(rt_attr));
+						rt_info->gateway = inaddr6_to_address(reinterpret_cast<in6_addr*>(RTA_DATA(rt_attr)));
 					}
 					else
 #endif
 					{
-						rt_info->gateway = inaddr_to_address((in_addr*)RTA_DATA(rt_attr));
+						rt_info->gateway = inaddr_to_address(reinterpret_cast<in_addr*>(RTA_DATA(rt_attr)));
 					}
 					break;
 				case RTA_DST:
 #if TORRENT_USE_IPV6
 					if (rt_msg->rtm_family == AF_INET6)
 					{
-						rt_info->destination = inaddr6_to_address((in6_addr*)RTA_DATA(rt_attr));
+						rt_info->destination = inaddr6_to_address(reinterpret_cast<in6_addr*>(RTA_DATA(rt_attr)));
 					}
 					else
 #endif
 					{
-						rt_info->destination = inaddr_to_address((in_addr*)RTA_DATA(rt_attr));
+						rt_info->destination = inaddr_to_address(reinterpret_cast<in_addr*>(RTA_DATA(rt_attr)));
 					}
 					break;
 			}
 		}
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
 
 		if_indextoname(if_index, rt_info->name);
 		ifreq req;
@@ -230,7 +238,8 @@ namespace libtorrent { namespace
 //		}
 		return true;
 	}
-#endif
+#endif // TORRENT_USE_NETLINK
+#endif // !BUILD_SIMULATOR
 
 #if TORRENT_USE_SYSCTL && !defined TORRENT_BUILD_SIMULATOR
 #ifdef TORRENT_OS2
@@ -375,7 +384,7 @@ namespace libtorrent
 			typedef boost::asio::ip::address_v4::bytes_type bytes_t;
 			bytes_t b;
 			std::memset(&b[0], 0xff, b.size());
-			for (int i = sizeof(bytes_t)/8-1; i > 0; --i)
+			for (int i = int(sizeof(bytes_t)) / 8 - 1; i > 0; --i)
 			{
 				if (bits < 8)
 				{
@@ -393,7 +402,7 @@ namespace libtorrent
 			typedef boost::asio::ip::address_v6::bytes_type bytes_t;
 			bytes_t b;
 			std::memset(&b[0], 0xff, b.size());
-			for (int i = sizeof(bytes_t)/8-1; i > 0; --i)
+			for (int i = int(sizeof(bytes_t)) / 8 - 1; i > 0; --i)
 			{
 				if (bits < 8)
 				{
@@ -417,10 +426,9 @@ namespace libtorrent
 	{
 		TORRENT_UNUSED(ios); // this may be unused depending on configuration
 		std::vector<ip_interface> ret;
+		ec.clear();
 #if defined TORRENT_BUILD_SIMULATOR
 
-		TORRENT_UNUSED(ec);
-
 		std::vector<address> ips = ios.get_ips();
 
 		for (int i = 0; i < int(ips.size()); ++i)
@@ -465,9 +473,9 @@ namespace libtorrent
 				if (iface_from_ifaddrs(ifa, iface))
 				{
 					ifreq req;
-					memset(&req, 0, sizeof(req));
-					// -1 to leave a null terminator
-					strncpy(req.ifr_name, iface.name, IF_NAMESIZE - 1);
+					std::memset(&req, 0, sizeof(req));
+					// -1 to leave a 0-terminator
+					std::strncpy(req.ifr_name, iface.name, IF_NAMESIZE - 1);
 
 					// ignore errors here. This is best-effort
 					ioctl(s, siocgifmtu, &req);
@@ -490,7 +498,7 @@ namespace libtorrent
 		// make sure the buffer is aligned to hold ifreq structs
 		ifreq buf[40];
 		ifc.ifc_len = sizeof(buf);
-		ifc.ifc_buf = (char*)buf;
+		ifc.ifc_buf = reinterpret_cast<char*>(buf);
 		if (ioctl(s, SIOCGIFCONF, &ifc) < 0)
 		{
 			ec = error_code(errno, system_category());
@@ -498,7 +506,7 @@ namespace libtorrent
 			return ret;
 		}
 
-		char *ifr = (char*)ifc.ifc_req;
+		char *ifr = reinterpret_cast<char*>(ifc.ifc_req);
 		int remaining = ifc.ifc_len;
 
 		while (remaining > 0)
@@ -527,7 +535,7 @@ namespace libtorrent
 
 				ifreq req;
 				memset(&req, 0, sizeof(req));
-				// -1 to leave a null terminator
+				// -1 to leave a 0-terminator
 				strncpy(req.ifr_name, item.ifr_name, IF_NAMESIZE - 1);
 				if (ioctl(s, siocgifmtu, &req) < 0)
 				{
@@ -618,14 +626,19 @@ namespace libtorrent
 				strncpy(r.name, adapter->AdapterName, sizeof(r.name));
 				r.name[sizeof(r.name)-1] = 0;
 				r.mtu = adapter->Mtu;
-				IP_ADAPTER_UNICAST_ADDRESS* unicast = adapter->FirstUnicastAddress;
-				while (unicast)
+				for (IP_ADAPTER_UNICAST_ADDRESS* unicast = adapter->FirstUnicastAddress;
+					unicast; unicast = unicast->Next)
 				{
+					if (unicast->Address.lpSockaddr->sa_family != AF_INET
+#if TORRENT_USE_IPV6
+						&& unicast->Address.lpSockaddr->sa_family != AF_INET6
+#endif
+					)
+						continue;
+
 					r.interface_address = sockaddr_to_address(unicast->Address.lpSockaddr);
 
 					ret.push_back(r);
-
-					unicast = unicast->Next;
 				}
 			}
 
@@ -684,6 +697,7 @@ namespace libtorrent
 		for (;i != udp::resolver_iterator(); ++i)
 		{
 			iface.interface_address = i->endpoint().address();
+			iface.name[0] = '\0';
 			iface.mtu = 1500;
 			if (iface.interface_address.is_v4())
 				iface.netmask = address_v4::netmask(iface.interface_address.to_v4());
@@ -886,7 +900,8 @@ namespace libtorrent
 	{
 		rtm = reinterpret_cast<rt_msghdr*>(next);
 		if (rtm->rtm_version != RTM_VERSION
-			|| rtm->rtm_type != RTM_ADD)
+			|| (rtm->rtm_type != RTM_ADD
+			&& rtm->rtm_type != RTM_GET))
 		{
 			continue;
 		}
@@ -1081,7 +1096,7 @@ namespace libtorrent
 
 		char msg[BUFSIZE];
 		memset(msg, 0, BUFSIZE);
-		nlmsghdr* nl_msg = (nlmsghdr*)msg;
+		nlmsghdr* nl_msg = reinterpret_cast<nlmsghdr*>(msg);
 
 		nl_msg->nlmsg_len = NLMSG_LENGTH(sizeof(rtmsg));
 		nl_msg->nlmsg_type = RTM_GETROUTE;
@@ -1110,11 +1125,18 @@ namespace libtorrent
 			ec = error_code(errno, system_category());
 			return std::vector<ip_route>();
 		}
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wcast-align"
+#endif
 		for (; NLMSG_OK(nl_msg, len); nl_msg = NLMSG_NEXT(nl_msg, len))
 		{
 			ip_route r;
 			if (parse_route(s, nl_msg, &r)) ret.push_back(r);
 		}
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
 		close(s);
 		close(sock);
 
@@ -1122,17 +1144,6 @@ namespace libtorrent
 		return ret;
 	}
 
-	// returns true if the given device exists
-	bool has_interface(char const* name, io_service& ios, error_code& ec)
-	{
-		std::vector<ip_interface> ifs = enum_net_interfaces(ios, ec);
-		if (ec) return false;
-
-		for (int i = 0; i < int(ifs.size()); ++i)
-			if (ifs[i].name == name) return true;
-		return false;
-	}
-
 	// returns the device name whose local address is ``addr``. If
 	// no such device is found, an empty string is returned.
 	std::string device_for_address(address addr, io_service& ios, error_code& ec)
