@@ -37,7 +37,9 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #ifdef __clang__
 #pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunknown-pragmas"
 #pragma clang diagnostic ignored "-Wunused-macros"
+#pragma clang diagnostic ignored "-Wreserved-id-macro"
 #endif
 
 // these defines are just in case the system we're on needs them for 64 bit file
@@ -129,6 +131,9 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifdef TORRENT_ANDROID
 #include <sys/syscall.h>
 #define lseek lseek64
+#define pread pread64
+#define pwrite pwrite64
+#define ftruncate ftruncate64
 #endif
 
 #elif defined __APPLE__ && defined __MACH__ && MAC_OS_X_VERSION_MIN_REQUIRED >= 1050
@@ -138,8 +143,6 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #endif
 
-#undef _FILE_OFFSET_BITS
-
 // make sure the _FILE_OFFSET_BITS define worked
 // on this platform. It's supposed to make file
 // related functions support 64-bit offsets.
@@ -537,7 +540,7 @@ namespace libtorrent
 		// something failed. Does the filesystem not support hard links?
 		// TODO: 3 find out what error code is reported when the filesystem
 		// does not support hard links.
-		DWORD error = GetLastError();
+		DWORD const error = GetLastError();
 		if (error != ERROR_NOT_SUPPORTED && error != ERROR_ACCESS_DENIED)
 		{
 			// it's possible CreateHardLink will copy the file internally too,
@@ -699,27 +702,6 @@ namespace libtorrent
 		}
 
 		rename(inf, newf, ec);
-
-		// on OSX, the error when trying to rename a file across different
-		// volumes is EXDEV, which will make it fall back to copying.
-
-		if (ec)
-		{
-			if (ec != boost::system::errc::no_such_file_or_directory
-				&& ec != boost::system::errc::invalid_argument
-				&& ec != boost::system::errc::permission_denied)
-			{
-				ec.clear();
-				copy_file(inf, newf, ec);
-
-				if (!ec)
-				{
-					// ignore errors when removing
-					error_code ignore;
-					remove(inf, ignore);
-				}
-			}
-		}
 	}
 
 	std::string split_path(std::string const& f)
@@ -1681,6 +1663,7 @@ typedef struct _FILE_ALLOCATED_RANGE_BUFFER {
 
 	namespace {
 
+#if !TORRENT_USE_PREADV
 	void gather_copy(file::iovec_t const* bufs, int num_bufs, char* dst)
 	{
 		std::size_t offset = 0;
@@ -1701,7 +1684,6 @@ typedef struct _FILE_ALLOCATED_RANGE_BUFFER {
 		}
 	}
 
-#if !TORRENT_USE_PREADV
 	bool coalesce_read_buffers(file::iovec_t const*& bufs, int& num_bufs
 		, file::iovec_t* tmp)
 	{
@@ -2093,49 +2075,7 @@ typedef struct _FILE_ALLOCATED_RANGE_BUFFER {
 				ec.assign(GetLastError(), system_category());
 				return false;
 			}
-		}
-
-#if _WIN32_WINNT >= 0x0600 // only if Windows Vista or newer
-		if ((m_open_mode & sparse) == 0)
-		{
-			typedef DWORD (WINAPI *GetFileInformationByHandleEx_t)(HANDLE hFile
-				, FILE_INFO_BY_HANDLE_CLASS FileInformationClass
-				, LPVOID lpFileInformation
-				, DWORD dwBufferSize);
-
-			static GetFileInformationByHandleEx_t GetFileInformationByHandleEx_ = NULL;
-
-			static bool failed_kernel32 = false;
-
-			if ((GetFileInformationByHandleEx_ == NULL) && !failed_kernel32)
-			{
-				HMODULE kernel32 = LoadLibraryA("kernel32.dll");
-				if (kernel32)
-				{
-					GetFileInformationByHandleEx_ = (GetFileInformationByHandleEx_t)GetProcAddress(kernel32, "GetFileInformationByHandleEx");
-				}
-				else
-				{
-					failed_kernel32 = true;
-				}
-			}
-
-			offs.QuadPart = 0;
-			if (GetFileInformationByHandleEx_)
-			{
-				// only allocate the space if the file
-				// is not fully allocated
-				FILE_STANDARD_INFO inf;
-				if (GetFileInformationByHandleEx_(native_handle()
-					, FileStandardInfo, &inf, sizeof(inf)) == FALSE)
-				{
-					ec.assign(GetLastError(), system_category());
-					if (ec) return false;
-				}
-				offs = inf.AllocationSize;
-			}
-
-			if (offs.QuadPart != s)
+			if ((m_open_mode & sparse) == 0)
 			{
 				// if the user has permissions, avoid filling
 				// the file with zeroes, but just fill it with
@@ -2143,7 +2083,6 @@ typedef struct _FILE_ALLOCATED_RANGE_BUFFER {
 				set_file_valid_data(m_file_handle, s);
 			}
 		}
-#endif // if Windows Vista
 #else // NON-WINDOWS
 		struct stat st;
 		if (fstat(native_handle(), &st) != 0)
