@@ -47,10 +47,8 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #if defined(TORRENT_WINDOWS) || defined(TORRENT_OS2)
 #define TORRENT_SEPARATOR '\\'
-#define TORRENT_SEPARATOR_STR "\\"
 #else
 #define TORRENT_SEPARATOR '/'
-#define TORRENT_SEPARATOR_STR "/"
 #endif
 
 namespace libtorrent
@@ -138,13 +136,14 @@ namespace libtorrent
 			return fe1.size < fe2.size;
 		}
 
-		bool compare_file_offset(internal_file_entry const& lhs
-			, internal_file_entry const& rhs)
-		{
-			return lhs.offset < rhs.offset;
-		}
+	bool compare_file_offset(internal_file_entry const& lhs
+		, internal_file_entry const& rhs)
+	{
+		return lhs.offset < rhs.offset;
 	}
 
+}
+
 	// path is not supposed to include the name of the torrent itself.
 	void file_storage::update_path_index(internal_file_entry& e
 		, std::string const& path, bool set_name)
@@ -179,10 +178,10 @@ namespace libtorrent
 		}
 
 		if (branch_len >= m_name.size()
-			&& std::memcmp(branch_path, m_name.c_str(), m_name.size()) == 0)
+			&& std::memcmp(branch_path, m_name.c_str(), m_name.size()) == 0
+			&& branch_path[m_name.size()] == TORRENT_SEPARATOR)
 		{
-			// the +1 is to skip the trailing '/' (or '\')
-			int offset = m_name.size()
+			int const offset = m_name.size()
 				+ (m_name.size() == branch_len?0:1);
 			branch_path += offset;
 			branch_len -= offset;
@@ -193,6 +192,16 @@ namespace libtorrent
 			e.no_root_dir = true;
 		}
 
+		e.path_index = get_or_add_path(branch_path, branch_len);
+		if (set_name) e.set_name(leaf);
+	}
+
+	int file_storage::get_or_add_path(char const* branch_path, int branch_len)
+	{
+		// trim trailing slashes
+		if (branch_len > 0 && branch_path[branch_len-1] == TORRENT_SEPARATOR)
+			--branch_len;
+
 		// do we already have this path in the path list?
 		std::vector<std::string>::reverse_iterator p
 			= std::find_if(m_paths.rbegin(), m_paths.rend()
@@ -201,23 +210,16 @@ namespace libtorrent
 		if (p == m_paths.rend())
 		{
 			// no, we don't. add it
-			e.path_index = m_paths.size();
-			TORRENT_ASSERT(branch_path[0] != '/');
-
-			// trim trailing slashes
-			if (branch_len > 0 && branch_path[branch_len-1] == TORRENT_SEPARATOR)
-				--branch_len;
-
-			// poor man's emplace back
-			m_paths.resize(m_paths.size() + 1);
-			m_paths.back().assign(branch_path, branch_len);
+			int const ret = int(m_paths.size());
+			TORRENT_ASSERT(branch_len == 0 || branch_path[0] != TORRENT_SEPARATOR);
+			m_paths.push_back(std::string(branch_path, branch_len));
+			return ret;
 		}
 		else
 		{
 			// yes we do. use it
-			e.path_index = p.base() - m_paths.begin() - 1;
+			return int(p.base() - m_paths.begin() - 1);
 		}
-		if (set_name) e.set_name(leaf);
 	}
 
 #ifndef TORRENT_NO_DEPRECATE
@@ -396,6 +398,8 @@ namespace libtorrent
 
 	int file_storage::file_index_at_offset(boost::int64_t offset) const
 	{
+		TORRENT_ASSERT_PRECOND(offset >= 0);
+		TORRENT_ASSERT_PRECOND(offset < m_total_size);
 		// find the file iterator and file offset
 		internal_file_entry target;
 		target.offset = offset;
@@ -425,6 +429,8 @@ namespace libtorrent
 		, boost::int64_t const offset
 		, int size) const
 	{
+		TORRENT_ASSERT_PRECOND(piece >= 0);
+		TORRENT_ASSERT_PRECOND(piece < num_pieces());
 		TORRENT_ASSERT_PRECOND(num_files() > 0);
 		std::vector<file_slice> ret;
 
@@ -850,7 +856,7 @@ namespace libtorrent
 		if (index >= int(m_file_hashes.size())) return sha1_hash(0);
 		return sha1_hash(m_file_hashes[index]);
 	}
-	
+
 	std::string const& file_storage::symlink(internal_file_entry const& fe) const
 	{
 		TORRENT_ASSERT_PRECOND(fe.symlink_index < int(m_symlinks.size()));
@@ -1041,11 +1047,13 @@ namespace libtorrent
 				++i;
 
 				// tail-padding is enabled, and the offset after this file is not
-				// aligned and it's not the last file. The last file must be padded
-				// too, in order to match an equivalent tail-padded file.
+				// aligned. The last file must be padded too, in order to match an
+				// equivalent tail-padded file.
 				add_pad_file(alignment - (off % alignment), i, off, padding_file);
 
 				TORRENT_ASSERT((off % alignment) == 0);
+
+				if (i == m_files.end()) break;
 			}
 		}
 		m_total_size = off;
@@ -1056,8 +1064,8 @@ namespace libtorrent
 		, boost::int64_t& offset
 		, int& pad_file_counter)
 	{
-		int cur_index = i - m_files.begin();
-		int index = m_files.size();
+		int const cur_index = i - m_files.begin();
+		int const index = m_files.size();
 		m_files.push_back(internal_file_entry());
 		++m_num_files;
 		internal_file_entry& e = m_files.back();
@@ -1065,11 +1073,10 @@ namespace libtorrent
 		i = m_files.begin() + cur_index;
 		e.size = size;
 		e.offset = offset;
-		char name[30];
-		snprintf(name, sizeof(name), ".pad" TORRENT_SEPARATOR_STR "%d"
-			, pad_file_counter);
-		std::string path = combine_path(m_name, name);
-		e.set_name(path.c_str());
+		e.path_index = get_or_add_path(".pad", 4);
+		char name[15];
+		snprintf(name, sizeof(name), "%d", pad_file_counter);
+		e.set_name(name);
 		e.pad_file = true;
 		offset += size;
 		++pad_file_counter;
@@ -1080,7 +1087,7 @@ namespace libtorrent
 		if (!m_file_base.empty()) m_file_base.resize(index + 1, 0);
 #endif
 
-		reorder_file(index, cur_index);
+		if (index != cur_index) reorder_file(index, cur_index);
 	}
 
 	void file_storage::unload()
