@@ -294,7 +294,7 @@ namespace libtorrent
 							, error_msg);
 					}
 					received_bytes(0, bytes_transferred);
-					disconnect(error_code(m_parser.status_code(), get_http_category()), op_bittorrent, 1);
+					disconnect(error_code(m_parser.status_code(), http_category()), op_bittorrent, 1);
 					return;
 				}
 				if (!m_parser.header_finished())
@@ -430,7 +430,7 @@ namespace libtorrent
 				received_bytes(0, bytes_transferred);
 				// temporarily unavailable, retry later
 				t->retry_web_seed(this, retry_time);
-				disconnect(error_code(m_parser.status_code(), get_http_category()), op_bittorrent, 1);
+				disconnect(error_code(m_parser.status_code(), http_category()), op_bittorrent, 1);
 				return;
 			}
 
