@@ -77,17 +77,22 @@ namespace libtorrent
 	};
 
 
-	boost::system::error_category& get_i2p_category()
+	boost::system::error_category& i2p_category()
 	{
 		static i2p_error_category i2p_category;
 		return i2p_category;
 	}
 
+#ifndef TORRENT_NO_DEPRECATE
+	boost::system::error_category& get_i2p_category()
+	{ return i2p_category(); }
+#endif
+
 	namespace i2p_error
 	{
 		boost::system::error_code make_error_code(i2p_error_code e)
 		{
-			return error_code(e, get_i2p_category());
+			return error_code(e, i2p_category());
 		}
 	}
 
@@ -325,7 +330,7 @@ namespace libtorrent
 		}
 
 		error_code invalid_response(i2p_error::parse_failed
-			, get_i2p_category());
+			, i2p_category());
 
 		// null-terminate the string and parse it
 		m_buffer.push_back(0);
@@ -414,7 +419,7 @@ namespace libtorrent
 			}
 		}
 
-		error_code ec(result, get_i2p_category());
+		error_code ec(result, i2p_category());
 		switch (result)
 		{
 			case i2p_error::no_error:
