@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2006-2014, Arvid Norberg
+Copyright (c) 2006-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,43 +30,45 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
+#include "libtorrent/config.hpp"
 #include "libtorrent/socket.hpp"
 
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+
 #include <boost/bind.hpp>
 
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
+
 #include <libtorrent/io.hpp>
 #include <libtorrent/random.hpp>
 #include <libtorrent/invariant_check.hpp>
 #include <libtorrent/kademlia/node_id.hpp> // for generate_random_id
 #include <libtorrent/kademlia/rpc_manager.hpp>
-#include <libtorrent/kademlia/logging.hpp>
 #include <libtorrent/kademlia/routing_table.hpp>
 #include <libtorrent/kademlia/find_data.hpp>
+#include <libtorrent/kademlia/put_data.hpp>
 #include <libtorrent/kademlia/refresh.hpp>
 #include <libtorrent/kademlia/node.hpp>
 #include <libtorrent/kademlia/observer.hpp>
+#include <libtorrent/kademlia/dht_observer.hpp>
+#include <libtorrent/kademlia/direct_request.hpp>
+#include <libtorrent/kademlia/get_item.hpp>
+
+#include <libtorrent/socket_io.hpp> // for print_endpoint
 #include <libtorrent/hasher.hpp>
 #include <libtorrent/session_settings.hpp> // for dht_settings
 #include <libtorrent/time.hpp>
-#include <time.h> // time()
-
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-#include <fstream>
-#endif
+#include <libtorrent/aux_/time.hpp> // for aux::time_now
 
 namespace libtorrent { namespace dht
 {
 
 namespace io = libtorrent::detail;
 
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-TORRENT_DEFINE_LOG(rpc)
-#endif
-
 void intrusive_ptr_add_ref(observer const* o)
 {
 	TORRENT_ASSERT(o != 0);
-	TORRENT_ASSERT(o->m_refs >= 0);
+	TORRENT_ASSERT(o->m_refs < 0xffff);
 	++o->m_refs;
 }
 
@@ -76,20 +78,21 @@ void intrusive_ptr_release(observer const* o)
 	TORRENT_ASSERT(o->m_refs > 0);
 	if (--o->m_refs == 0)
 	{
-		boost::intrusive_ptr<traversal_algorithm> ta = o->m_algorithm;
+		boost::intrusive_ptr<traversal_algorithm> ta = o->algorithm();
 		(const_cast<observer*>(o))->~observer();
 		ta->free_observer(const_cast<observer*>(o));
 	}
 }
 
+// TODO: 3 move this into it's own .cpp file
+dht_observer* observer::get_observer() const
+{
+	return m_algorithm->get_node().observer();
+}
+
 void observer::set_target(udp::endpoint const& ep)
 {
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-	// use high resolution timers for logging
-	m_sent = time_now_hires();
-#else
-	m_sent = time_now();
-#endif
+	m_sent = clock_type::now();
 
 	m_port = ep.port();
 #if TORRENT_USE_IPV6
@@ -141,8 +144,6 @@ void observer::short_timeout()
 	m_algorithm->failed(observer_ptr(this), traversal_algorithm::short_timeout);
 }
 
-// this is called when no reply has been received within
-// some timeout
 void observer::timeout()
 {
 	if (flags & flag_done) return;
@@ -157,61 +158,43 @@ void observer::set_id(node_id const& id)
 	if (m_algorithm) m_algorithm->resort_results();
 }
 
-enum { observer_size = max3<
+enum { observer_size = max9<
 	sizeof(find_data_observer)
 	, sizeof(announce_observer)
+	, sizeof(put_data_observer)
+	, sizeof(direct_observer)
+	, sizeof(get_item_observer)
+	, sizeof(get_peers_observer)
+	, sizeof(obfuscated_get_peers_observer)
 	, sizeof(null_observer)
+	, sizeof(traversal_observer)
 	>::value
 };
 
 rpc_manager::rpc_manager(node_id const& our_id
-	, routing_table& table, udp_socket_interface* sock)
+	, dht_settings const& settings
+	, routing_table& table, udp_socket_interface* sock
+	, dht_logger* log)
 	: m_pool_allocator(observer_size, 10)
 	, m_sock(sock)
+	, m_log(log)
+	, m_settings(settings)
 	, m_table(table)
-	, m_timer(time_now())
+	, m_timer(aux::time_now())
 	, m_our_id(our_id)
 	, m_allocated_observers(0)
 	, m_destructing(false)
-{
-	std::srand((unsigned int)time(0));
-
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-	TORRENT_LOG(rpc) << "Constructing";
-
-#define PRINT_OFFSETOF(x, y) TORRENT_LOG(rpc) << "  +" << offsetof(x, y) << ": " #y
-
-	TORRENT_LOG(rpc) << " observer: " << sizeof(observer);
-	PRINT_OFFSETOF(dht::observer, m_sent);
-	PRINT_OFFSETOF(dht::observer, m_refs);
-	PRINT_OFFSETOF(dht::observer, m_algorithm);
-	PRINT_OFFSETOF(dht::observer, m_id);
-	PRINT_OFFSETOF(dht::observer, m_addr);
-	PRINT_OFFSETOF(dht::observer, m_port);
-	PRINT_OFFSETOF(dht::observer, m_transaction_id);
-	PRINT_OFFSETOF(dht::observer, flags);
-
-	TORRENT_LOG(rpc) << " announce_observer: " << sizeof(announce_observer);
-	TORRENT_LOG(rpc) << " null_observer: " << sizeof(null_observer);
-	TORRENT_LOG(rpc) << " find_data_observer: " << sizeof(find_data_observer);
-
-#undef PRINT_OFFSETOF
-#endif
-
-}
+{}
 
 rpc_manager::~rpc_manager()
 {
 	TORRENT_ASSERT(!m_destructing);
 	m_destructing = true;
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-	TORRENT_LOG(rpc) << "Destructing";
-#endif
 	
 	for (transactions_t::iterator i = m_transactions.begin()
 		, end(m_transactions.end()); i != end; ++i)
 	{
-		(*i)->abort();
+		i->second->abort();
 	}
 }
 
@@ -243,37 +226,36 @@ void rpc_manager::check_invariant() const
 	for (transactions_t::const_iterator i = m_transactions.begin()
 		, end(m_transactions.end()); i != end; ++i)
 	{
-		TORRENT_ASSERT(*i);
+		TORRENT_ASSERT(i->second);
 	}
 }
 #endif
 
 void rpc_manager::unreachable(udp::endpoint const& ep)
 {
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-	TORRENT_LOG(rpc) << time_now_string() << " PORT_UNREACHABLE [ ip: " << ep << " ]";
+#ifndef TORRENT_DISABLE_LOGGING
+	m_log->log(dht_logger::rpc_manager, "PORT_UNREACHABLE [ ip: %s ]"
+		, print_endpoint(ep).c_str());
 #endif
 
 	for (transactions_t::iterator i = m_transactions.begin();
 		i != m_transactions.end();)
 	{
-		TORRENT_ASSERT(*i);
-		observer_ptr const& o = *i;
+		TORRENT_ASSERT(i->second);
+		observer_ptr const& o = i->second;
 		if (o->target_ep() != ep) { ++i; continue; }
-		observer_ptr ptr = *i;
+		observer_ptr ptr = i->second;
 		i = m_transactions.erase(i);
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-		TORRENT_LOG(rpc) << "  found transaction [ tid: " << ptr->transaction_id() << " ]";
+#ifndef TORRENT_DISABLE_LOGGING
+		m_log->log(dht_logger::rpc_manager, "found transaction [ tid: %d ]"
+			, int(ptr->transaction_id()));
 #endif
 		ptr->timeout();
 		break;
 	}
 }
 
-// defined in node.cpp
-void incoming_error(entry& e, char const* msg, int error_code = 203);
-
-bool rpc_manager::incoming(msg const& m, node_id* id, libtorrent::dht_settings const& settings)
+bool rpc_manager::incoming(msg const& m, node_id* id)
 {
 	INVARIANT_CHECK;
 
@@ -289,31 +271,24 @@ bool rpc_manager::incoming(msg const& m, node_id* id, libtorrent::dht_settings c
 	std::string transaction_id = m.message.dict_find_string_value("t");
 	if (transaction_id.empty()) return false;
 
-	std::string::const_iterator i = transaction_id.begin();	
-	int tid = transaction_id.size() != 2 ? -1 : io::read_uint16(i);
+	std::string::const_iterator ptr = transaction_id.begin();
+	int tid = transaction_id.size() != 2 ? -1 : io::read_uint16(ptr);
 
 	observer_ptr o;
-
-	for (transactions_t::iterator i = m_transactions.begin()
-		, end(m_transactions.end()); i != end;)
+	std::pair<transactions_t::iterator, transactions_t::iterator> range = m_transactions.equal_range(tid);
+	for (transactions_t::iterator i = range.first; i != range.second; ++i)
 	{
-		TORRENT_ASSERT(*i);
-		if ((*i)->transaction_id() != tid
-			|| m.addr.address() != (*i)->target_addr())
-		{
-			++i;
-			continue;
-		}
-		o = *i;
+		if (m.addr.address() != i->second->target_addr()) continue;
+		o = i->second;
 		i = m_transactions.erase(i);
 		break;
 	}
 
 	if (!o)
 	{
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-		TORRENT_LOG(rpc) << "Reply with unknown transaction id size: " 
-			<< transaction_id.size() << " from " << m.addr;
+#ifndef TORRENT_DISABLE_LOGGING
+		m_log->log(dht_logger::rpc_manager, "reply with unknown transaction id size: %d from %s"
+			, int(transaction_id.size()), print_endpoint(m.addr).c_str());
 #endif
 		// this isn't necessarily because the other end is doing
 		// something wrong. This can also happen when we restart
@@ -326,52 +301,71 @@ bool rpc_manager::incoming(msg const& m, node_id* id, libtorrent::dht_settings c
 		return false;
 	}
 
-	ptime now = time_now_hires();
+	time_point now = clock_type::now();
 
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-	std::ofstream reply_stats("round_trip_ms.log", std::ios::app);
-	reply_stats << m.addr << "\t" << total_milliseconds(now - o->sent())
-		<< std::endl;
+#ifndef TORRENT_DISABLE_LOGGING
+	m_log->log(dht_logger::rpc_manager, "round trip time(ms): %" PRId64 " from %s"
+		, total_milliseconds(now - o->sent()), print_endpoint(m.addr).c_str());
 #endif
 
-	lazy_entry const* ret_ent = m.message.dict_find_dict("r");
-	if (ret_ent == 0)
+	if (m.message.dict_find_string_value("y") == "e")
 	{
-		// it may be an error
-		ret_ent = m.message.dict_find("e");
-		o->timeout();
-		if (ret_ent == NULL)
+		// It's an error.
+#ifndef TORRENT_DISABLE_LOGGING
+		bdecode_node err = m.message.dict_find_list("e");
+		if (err && err.list_size() >= 2
+			&& err.list_at(0).type() == bdecode_node::int_t
+			&& err.list_at(1).type() == bdecode_node::string_t)
 		{
-			entry e;
-			incoming_error(e, "missing 'r' key");
-			m_sock->send_packet(e, m.addr, 0);
+			m_log->log(dht_logger::rpc_manager, "reply with error from %s: (%" PRId64 ") %s"
+				, print_endpoint(m.addr).c_str()
+				, err.list_int_value_at(0)
+				, err.list_string_value_at(1).c_str());
 		}
+		else
+		{
+			m_log->log(dht_logger::rpc_manager, "reply with (malformed) error from %s"
+				, print_endpoint(m.addr).c_str());
+		}
+#endif
+		// Logically, we should call o->reply(m) since we get a reply.
+		// a reply could be "response" or "error", here the reply is an "error".
+		// if the reply is an "error", basically the observer could/will
+		// do nothing with it, especially when observer::reply() is intended to
+		// handle a "response", not an "error".
+		// A "response" should somehow call algorithm->finished(), and an error/timeout
+		// should call algorithm->failed(). From this point of view,
+		// we should call o->timeout() instead of o->reply(m) because o->reply()
+		// will call algorithm->finished().
+		o->timeout();
 		return false;
 	}
 
-	lazy_entry const* node_id_ent = ret_ent->dict_find_string("id");
-	if (node_id_ent == 0 || node_id_ent->string_length() != 20)
+	bdecode_node ret_ent = m.message.dict_find_dict("r");
+	if (!ret_ent)
 	{
 		o->timeout();
-		entry e;
-		incoming_error(e, "missing 'id' key");
-		m_sock->send_packet(e, m.addr, 0);
 		return false;
 	}
 
-	node_id nid = node_id(node_id_ent->string_ptr());
-	if (settings.enforce_node_id && !verify_id(nid, m.addr.address()))
+	bdecode_node node_id_ent = ret_ent.dict_find_string("id");
+	if (!node_id_ent || node_id_ent.string_length() != 20)
 	{
 		o->timeout();
-		entry e;
-		incoming_error(e, "invalid node ID");
-		m_sock->send_packet(e, m.addr, 0);
 		return false;
 	}
 
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-	TORRENT_LOG(rpc) << "[" << o->m_algorithm.get() << "] Reply with transaction id: " 
-		<< tid << " from " << m.addr;
+	node_id nid = node_id(node_id_ent.string_ptr());
+	if (m_settings.enforce_node_id && !verify_id(nid, m.addr.address()))
+	{
+		o->timeout();
+		return false;
+	}
+
+#ifndef TORRENT_DISABLE_LOGGING
+	m_log->log(dht_logger::rpc_manager, "[%p] reply with transaction id: %d from %s"
+		, static_cast<void*>(o->algorithm()), int(transaction_id.size())
+		, print_endpoint(m.addr).c_str());
 #endif
 	o->reply(m);
 	*id = nid;
@@ -387,79 +381,60 @@ time_duration rpc_manager::tick()
 {
 	INVARIANT_CHECK;
 
-	const static int short_timeout = 1;
-	const static int timeout = 15;
+	static const int short_timeout = 1;
+	static const int timeout = 15;
 
-	//	look for observers that have timed out
+	// look for observers that have timed out
 
 	if (m_transactions.empty()) return seconds(short_timeout);
 
-	std::list<observer_ptr> timeouts;
+	std::vector<observer_ptr> timeouts;
+	std::vector<observer_ptr> short_timeouts;
 
 	time_duration ret = seconds(short_timeout);
-	ptime now = time_now();
-
-#if TORRENT_USE_ASSERTS
-	ptime last = min_time();
-	for (transactions_t::iterator i = m_transactions.begin();
-		i != m_transactions.end(); ++i)
-	{
-		TORRENT_ASSERT((*i)->sent() >= last);
-		last = (*i)->sent();
-	}
-#endif
+	time_point now = aux::time_now();
 
 	for (transactions_t::iterator i = m_transactions.begin();
 		i != m_transactions.end();)
 	{
-		observer_ptr o = *i;
+		observer_ptr o = i->second;
 
-		// if we reach an observer that hasn't timed out
-		// break, because every observer after this one will
-		// also not have timed out yet
 		time_duration diff = now - o->sent();
-		if (diff < seconds(timeout))
+		if (diff >= seconds(timeout))
 		{
-			ret = seconds(timeout) - diff;
-			break;
-		}
-		
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-		TORRENT_LOG(rpc) << "[" << o->m_algorithm.get() << "] Timing out transaction id: " 
-			<< (*i)->transaction_id() << " from " << o->target_ep();
+#ifndef TORRENT_DISABLE_LOGGING
+			m_log->log(dht_logger::rpc_manager, "[%p] timing out transaction id: %d from: %s"
+				, static_cast<void*>(o->algorithm()), o->transaction_id()
+				, print_endpoint(o->target_ep()).c_str());
 #endif
-		i = m_transactions.erase(i);
-		timeouts.push_back(o);
-	}
-	
-	std::for_each(timeouts.begin(), timeouts.end(), boost::bind(&observer::timeout, _1));
-	timeouts.clear();
-
-	for (transactions_t::iterator i = m_transactions.begin();
-		i != m_transactions.end(); ++i)
-	{
-		observer_ptr o = *i;
-
-		// if we reach an observer that hasn't timed out
-		// break, because every observer after this one will
-		// also not have timed out yet
-		time_duration diff = now - o->sent();
-		if (diff < seconds(short_timeout))
-		{
-			ret = seconds(short_timeout) - diff;
-			break;
+			m_transactions.erase(i++);
+			timeouts.push_back(o);
+			continue;
 		}
-		
+
 		// don't call short_timeout() again if we've
 		// already called it once
-		if (o->has_short_timeout()) continue;
+		if (diff >= seconds(short_timeout) && !o->has_short_timeout())
+		{
+#ifndef TORRENT_DISABLE_LOGGING
+			m_log->log(dht_logger::rpc_manager, "[%p] short-timing out transaction id: %d from: %s"
+				, static_cast<void*>(o->algorithm()), o->transaction_id()
+				, print_endpoint(o->target_ep()).c_str());
+#endif
+			++i;
 
-		timeouts.push_back(o);
+			short_timeouts.push_back(o);
+			continue;
+		}
+
+		ret = std::min(seconds(timeout) - diff, ret);
+		++i;
 	}
 
-	std::for_each(timeouts.begin(), timeouts.end(), boost::bind(&observer::short_timeout, _1));
-	
-	return ret;
+	std::for_each(timeouts.begin(), timeouts.end(), boost::bind(&observer::timeout, _1));
+	std::for_each(short_timeouts.begin(), short_timeouts.end(), boost::bind(&observer::short_timeout, _1));
+
+	return (std::max)(ret, duration_cast<time_duration>(milliseconds(200)));
 }
 
 void rpc_manager::add_our_id(entry& e)
@@ -484,18 +459,23 @@ bool rpc_manager::invoke(entry& e, udp::endpoint target_addr
 	int tid = (random() ^ (random() << 5)) & 0xffff;
 	io::write_uint16(tid, out);
 	e["t"] = transaction_id;
-		
+
+	// When a DHT node enters the read-only state, in each outgoing query message,
+	// places a 'ro' key in the top-level message dictionary and sets its value to 1.
+	if (m_settings.read_only) e["ro"] = 1;
+
 	o->set_target(target_addr);
 	o->set_transaction_id(tid);
 
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-	TORRENT_LOG(rpc) << "[" << o->m_algorithm.get() << "] invoking "
-		<< e["q"].string() << " -> " << target_addr;
+#ifndef TORRENT_DISABLE_LOGGING
+	m_log->log(dht_logger::rpc_manager, "[%p] invoking %s -> %s"
+		, static_cast<void*>(o->algorithm()), e["q"].string().c_str()
+		, print_endpoint(target_addr).c_str());
 #endif
 
 	if (m_sock->send_packet(e, target_addr, 1))
 	{
-		m_transactions.push_back(o);
+		m_transactions.insert(std::make_pair(tid, o));
 #if TORRENT_USE_ASSERTS
 		o->m_was_sent = true;
 #endif
