@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2006-2014, Arvid Norberg
+Copyright (c) 2006-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,32 +30,33 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
+#ifndef TORRENT_NO_DEPRECATE
 #ifndef TORRENT_DISABLE_EXTENSIONS
 
-#ifdef _MSC_VER
-#pragma warning(push, 1)
-#endif
+#include "libtorrent/aux_/disable_warnings_push.hpp"
 
 #include <boost/shared_ptr.hpp>
-
-#ifdef _MSC_VER
-#pragma warning(pop)
-#endif
+#include <boost/bind.hpp>
 
 #include <vector>
 #include <utility>
 #include <numeric>
 #include <algorithm> // count
 
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
+
 #include "libtorrent/peer_connection.hpp"
 #include "libtorrent/bt_peer_connection.hpp"
+#include "libtorrent/peer_connection_handle.hpp"
 #include "libtorrent/hasher.hpp"
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/torrent.hpp"
+#include "libtorrent/torrent_handle.hpp"
 #include "libtorrent/extensions.hpp"
 #include "libtorrent/extensions/metadata_transfer.hpp"
 #include "libtorrent/alert_types.hpp"
 #include "libtorrent/buffer.hpp"
+#include "libtorrent/io.hpp"
 
 namespace libtorrent { namespace
 {
@@ -82,21 +83,22 @@ namespace libtorrent { namespace
 		int size = (offset.first + offset.second) * 256 / total_size - start;
 
 		std::pair<int, int> ret(start, size);
-	
+
 		TORRENT_ASSERT(start >= 0);
 		TORRENT_ASSERT(size > 0);
 		TORRENT_ASSERT(start <= 256);
 		TORRENT_ASSERT(start + size <= 256);
 
 		// assert the identity of this function
-#ifndef NDEBUG
+#if TORRENT_USE_ASSERTS
 		std::pair<int, int> identity = req_to_offset(ret, total_size);
 		TORRENT_ASSERT(offset == identity);
 #endif
 		return ret;
 	}
 
-	struct metadata_plugin : torrent_plugin
+	struct metadata_plugin TORRENT_FINAL
+		: torrent_plugin
 	{
 		metadata_plugin(torrent& t)
 			: m_torrent(t)
@@ -106,7 +108,23 @@ namespace libtorrent { namespace
 			m_requested_metadata.resize(256, 0);
 		}
 
-		virtual void on_files_checked()
+/*
+		bool need_loaded()
+		{ return m_torrent.need_loaded(); }
+*/
+		virtual void on_unload() TORRENT_OVERRIDE
+		{
+			m_metadata.reset();
+		}
+
+		virtual void on_load() TORRENT_OVERRIDE
+		{
+			// initialize m_metadata_size
+			TORRENT_ASSERT(m_torrent.is_loaded());
+			metadata();
+		}
+
+		virtual void on_files_checked() TORRENT_OVERRIDE
 		{
 			// if the torrent is a seed, make a reference to
 			// the metadata from the torrent before it is deallocated
@@ -114,8 +132,8 @@ namespace libtorrent { namespace
 		}
 
 		virtual boost::shared_ptr<peer_plugin> new_connection(
-			peer_connection* pc);
-		
+			peer_connection_handle const& pc) TORRENT_OVERRIDE;
+
 		buffer::const_interval metadata() const
 		{
 			if (!m_metadata)
@@ -146,13 +164,13 @@ namespace libtorrent { namespace
 			std::pair<int, int> req = offset_to_req(std::make_pair(offset, size)
 				, total_size);
 
-			TORRENT_ASSERT(req.first + req.second <= (int)m_have_metadata.size());
+			TORRENT_ASSERT(req.first + req.second <= int(m_have_metadata.size()));
 
 			std::fill(
 				m_have_metadata.begin() + req.first
 				, m_have_metadata.begin() + req.first + req.second
 				, true);
-		
+
 			bool have_all = std::count(
 				m_have_metadata.begin()
 				, m_have_metadata.end()
@@ -201,7 +219,7 @@ namespace libtorrent { namespace
 			m_torrent.set_progress_ppm(boost::int64_t(m_metadata_progress) * 1000000 / m_metadata_size);
 		}
 
-		void on_piece_pass(int)
+		void on_piece_pass(int) TORRENT_OVERRIDE
 		{
 			// if we became a seed, copy the metadata from
 			// the torrent before it is deallocated
@@ -209,7 +227,9 @@ namespace libtorrent { namespace
 				metadata();
 		}
 
+/*
 		int metadata_size() const { return m_metadata_size; }
+*/
 
 	private:
 		torrent& m_torrent;
@@ -232,10 +252,13 @@ namespace libtorrent { namespace
 		// this vector keeps track of how many times each meatdata
 		// block has been requested
 		std::vector<int> m_requested_metadata;
+
+		// explicitly disallow assignment, to silence msvc warning
+		metadata_plugin& operator=(metadata_plugin const&);
 	};
 
-
-	struct metadata_peer_plugin : peer_plugin
+	struct metadata_peer_plugin TORRENT_FINAL
+		: peer_plugin
 	{
 		metadata_peer_plugin(torrent& t, peer_connection& pc
 			, metadata_plugin& tp)
@@ -249,24 +272,24 @@ namespace libtorrent { namespace
 			, m_tp(tp)
 		{}
 
-		virtual char const* type() const { return "LT_metadata"; }
+		virtual char const* type() const TORRENT_OVERRIDE { return "LT_metadata"; }
 
 		// can add entries to the extension handshake
-		virtual void add_handshake(entry& h)
+		virtual void add_handshake(entry& h) TORRENT_OVERRIDE
 		{
 			entry& messages = h["m"];
 			messages["LT_metadata"] = 14;
 		}
 
 		// called when the extension handshake from the other end is received
-		virtual bool on_extension_handshake(lazy_entry const& h)
+		virtual bool on_extension_handshake(bdecode_node const& h) TORRENT_OVERRIDE
 		{
 			m_message_index = 0;
-			if (h.type() != lazy_entry::dict_t) return false;
-			lazy_entry const* messages = h.dict_find("m");
-			if (!messages || messages->type() != lazy_entry::dict_t) return false;
+			if (h.type() != bdecode_node::dict_t) return false;
+			bdecode_node messages = h.dict_find("m");
+			if (!messages || messages.type() != bdecode_node::dict_t) return false;
 
-			int index = int(messages->dict_find_int_value("LT_metadata", -1));
+			int index = int(messages.dict_find_int_value("LT_metadata", -1));
 			if (index == -1) return false;
 			m_message_index = index;
 			return true;
@@ -286,9 +309,9 @@ namespace libtorrent { namespace
 			// abort if the peer doesn't support the metadata extension
 			if (m_message_index == 0) return;
 
-#ifdef TORRENT_VERBOSE_LOGGING
-			m_pc.peer_log("==> METADATA_REQUEST  [ start: %d | size: %d ]\n"
-				, start, size);
+#ifndef TORRENT_DISABLE_LOGGING
+			m_pc.peer_log(peer_log_alert::outgoing_message, "METADATA_REQUEST"
+				, "start: %d size: %d", start, size);
 #endif
 
 			char msg[9];
@@ -319,13 +342,14 @@ namespace libtorrent { namespace
 			if (m_torrent.valid_metadata())
 			{
 				std::pair<int, int> offset
-					= req_to_offset(req, (int)m_tp.metadata().left());
+					= req_to_offset(req, int(m_tp.metadata().left()));
 
 				char msg[15];
 				char* ptr = msg;
 
-#ifdef TORRENT_VERBOSE_LOGGING
-				m_pc.peer_log("==> METADATA [ start: %d | total_size: %d | offset: %d | data_size: %d ]"
+#ifndef TORRENT_DISABLE_LOGGING
+				m_pc.peer_log(peer_log_alert::outgoing_message, "METADATA"
+					, "start: %d total_size: %d offset: %d data_size: %d"
 					, req.first, req.second, offset.first, offset.second);
 #endif
 				// yes, we have metadata, send it
@@ -334,16 +358,20 @@ namespace libtorrent { namespace
 				detail::write_uint8(m_message_index, ptr);
 				// means 'data packet'
 				detail::write_uint8(1, ptr);
-				detail::write_uint32((int)m_tp.metadata().left(), ptr);
+				detail::write_uint32(int(m_tp.metadata().left()), ptr);
 				detail::write_uint32(offset.first, ptr);
 				m_pc.send_buffer(msg, sizeof(msg));
+
+				// TODO: this is not safe. The torrent could be unloaded while
+				// we're still sending the metadata
 				char const* metadata = m_tp.metadata().begin;
 				m_pc.append_const_send_buffer(metadata + offset.first, offset.second);
 			}
 			else
 			{
-#ifdef TORRENT_VERBOSE_LOGGING
-				m_pc.peer_log("==> DONT HAVE METADATA\n");
+#ifndef TORRENT_DISABLE_LOGGING
+				m_pc.peer_log(peer_log_alert::outgoing_message, "METADATA"
+					, "don't have metadata");
 #endif
 				char msg[4+3];
 				char* ptr = msg;
@@ -361,14 +389,14 @@ namespace libtorrent { namespace
 		}
 
 		virtual bool on_extended(int length
-			, int msg, buffer::const_interval body)
+			, int msg, buffer::const_interval body) TORRENT_OVERRIDE
 		{
 			if (msg != 14) return false;
 			if (m_message_index == 0) return false;
 
 			if (length > 500 * 1024)
 			{
-				m_pc.disconnect(errors::metadata_too_large, 2);
+				m_pc.disconnect(errors::metadata_too_large, op_bittorrent, 2);
 				return true;
 			}
 
@@ -383,15 +411,15 @@ namespace libtorrent { namespace
 					int start = detail::read_uint8(body.begin);
 					int size = detail::read_uint8(body.begin) + 1;
 
-#ifdef TORRENT_VERBOSE_LOGGING
-					m_pc.peer_log("<== METADATA_REQUEST  [ start: %d | size: %d ]\n"
-						, start, size);
+#ifndef TORRENT_DISABLE_LOGGING
+					m_pc.peer_log(peer_log_alert::incoming_message, "METADATA_REQUEST"
+						, "start: %d size: %d", start, size);
 #endif
 
 					if (length != 3)
 					{
 						// invalid metadata request
-						m_pc.disconnect(errors::invalid_metadata_request, 2);
+						m_pc.disconnect(errors::invalid_metadata_request, op_bittorrent, 2);
 						return true;
 					}
 
@@ -406,29 +434,30 @@ namespace libtorrent { namespace
 					int offset = detail::read_int32(body.begin);
 					int data_size = length - 9;
 
-#ifdef TORRENT_VERBOSE_LOGGING
-					m_pc.peer_log("<== METADATA [ total_size: %d | offset: %d | data_size: %d ]"
+#ifndef TORRENT_DISABLE_LOGGING
+					m_pc.peer_log(peer_log_alert::incoming_message, "METADATA"
+						, "total_size: %d | offset: %d | data_size: %d"
 						,total_size, offset, data_size);
 #endif
 
-					if (total_size > m_torrent.session().settings().max_metadata_size)
+					if (total_size > m_torrent.session().settings().get_int(settings_pack::max_metadata_size))
 					{
-						m_pc.disconnect(errors::metadata_too_large, 2);
+						m_pc.disconnect(errors::metadata_too_large, op_bittorrent, 2);
 						return true;
 					}
 					if (total_size <= 0)
 					{
-						m_pc.disconnect(errors::invalid_metadata_size, 2);
+						m_pc.disconnect(errors::invalid_metadata_size, op_bittorrent, 2);
 						return true;
 					}
 					if (offset > total_size || offset < 0)
 					{
-						m_pc.disconnect(errors::invalid_metadata_offset, 2);
+						m_pc.disconnect(errors::invalid_metadata_offset, op_bittorrent, 2);
 						return true;
 					}
 					if (offset + data_size > total_size)
 					{
-						m_pc.disconnect(errors::invalid_metadata_message, 2);
+						m_pc.disconnect(errors::invalid_metadata_message, op_bittorrent, 2);
 						return true;
 					}
 
@@ -445,23 +474,24 @@ namespace libtorrent { namespace
 				}
 				break;
 			case 2: // have no data
-				m_no_metadata = time_now();
+				m_no_metadata = aux::time_now();
 				if (m_waiting_metadata_request)
 					m_tp.cancel_metadata_request(m_last_metadata_request);
 				m_waiting_metadata_request = false;
-#ifdef TORRENT_VERBOSE_LOGGING
-				m_pc.peer_log("<== DONT HAVE METADATA\n");
+#ifndef TORRENT_DISABLE_LOGGING
+				m_pc.peer_log(peer_log_alert::incoming_message, "METADATA"
+					, "don't have metadata");
 #endif
 				break;
 			default:
 				{
-					m_pc.disconnect(errors::invalid_metadata_message, 2);
+					m_pc.disconnect(errors::invalid_metadata_message, op_bittorrent, 2);
 				}
 			}
 			return true;
 		}
 
-		virtual void tick()
+		virtual void tick() TORRENT_OVERRIDE
 		{
 			if (m_pc.is_disconnecting()) return;
 
@@ -477,13 +507,13 @@ namespace libtorrent { namespace
 				m_last_metadata_request = m_tp.metadata_request();
 				write_metadata_request(m_last_metadata_request);
 				m_waiting_metadata_request = true;
-				m_metadata_request = time_now();
+				m_metadata_request = aux::time_now();
 			}
 		}
 
 		bool has_metadata() const
 		{
-			return time_now() - m_no_metadata > minutes(5);
+			return aux::time_now() - minutes(5) > m_no_metadata;
 		}
 
 	private:
@@ -492,7 +522,7 @@ namespace libtorrent { namespace
 		// request to this peer, and reset to false when
 		// we receive a reply to our request.
 		bool m_waiting_metadata_request;
-		
+
 		// this is the message index the remote peer uses
 		// for metadata extension messages.
 		int m_message_index;
@@ -506,11 +536,11 @@ namespace libtorrent { namespace
 
 		// this is set to the current time each time we get a
 		// "I don't have metadata" message.
-		ptime m_no_metadata;
+		time_point m_no_metadata;
 
 		// this is set to the time when we last sent
 		// a request for metadata to this peer
-		ptime m_metadata_request;
+		time_point m_metadata_request;
 
 		// if we're waiting for a metadata request
 		// this was the request we sent
@@ -519,15 +549,18 @@ namespace libtorrent { namespace
 		torrent& m_torrent;
 		peer_connection& m_pc;
 		metadata_plugin& m_tp;
+
+		// explicitly disallow assignment, to silence msvc warning
+		metadata_peer_plugin& operator=(metadata_peer_plugin const&);
 	};
 
 	boost::shared_ptr<peer_plugin> metadata_plugin::new_connection(
-		peer_connection* pc)
+		peer_connection_handle const& pc)
 	{
-		if (pc->type() != peer_connection::bittorrent_connection)
+		if (pc.type() != peer_connection::bittorrent_connection)
 			return boost::shared_ptr<peer_plugin>();
 
-		return boost::shared_ptr<peer_plugin>(new metadata_peer_plugin(m_torrent, *pc, *this));
+		return boost::shared_ptr<peer_plugin>(new metadata_peer_plugin(m_torrent, *pc.native_handle().get(), *this));
 	}
 
 	std::pair<int, int> metadata_plugin::metadata_request()
@@ -543,7 +576,7 @@ namespace libtorrent { namespace
 			int min = *std::min_element(m_requested_metadata.begin() + i
 				, m_requested_metadata.begin() + i + num_blocks);
 			min += std::accumulate(m_requested_metadata.begin() + i
-				, m_requested_metadata.begin() + i + num_blocks, (int)0);
+				, m_requested_metadata.begin() + i + num_blocks, int(0));
 
 			if (min_element > min)
 			{
@@ -569,8 +602,9 @@ namespace libtorrent { namespace
 namespace libtorrent
 {
 
-	boost::shared_ptr<torrent_plugin> create_metadata_plugin(torrent* t, void*)
+	boost::shared_ptr<torrent_plugin> create_metadata_plugin(torrent_handle const& th, void*)
 	{
+		torrent* t = th.native_handle().get();
 		// don't add this extension if the torrent is private
 		if (t->valid_metadata() && t->torrent_file().priv()) return boost::shared_ptr<torrent_plugin>();
 		return boost::shared_ptr<torrent_plugin>(new metadata_plugin(*t));
@@ -579,4 +613,5 @@ namespace libtorrent
 }
 
 #endif
+#endif
 
