@@ -117,11 +117,6 @@ namespace libtorrent
 		// connect to 5 peers per second
 		set.set_int(settings_pack::connection_speed, 5);
 
-		// be extra nice on the hard drive when running
-		// on embedded devices. This might slow down
-		// torrent checking
-		set.set_int(settings_pack::file_checks_delay_per_block, 5);
-
 		// only have 4 files open at a time
 		set.set_int(settings_pack::file_pool_size, 4);
 
@@ -296,7 +291,6 @@ namespace libtorrent
 	TORRENT_EXPORT session_settings min_memory_usage()
 	{
 		aux::session_settings def;
-		initialize_default_settings(def);
 		settings_pack pack;
 		min_memory_usage(pack);
 		apply_pack(&pack, def, 0);
@@ -308,7 +302,6 @@ namespace libtorrent
 	TORRENT_EXPORT session_settings high_performance_seed()
 	{
 		aux::session_settings def;
-		initialize_default_settings(def);
 		settings_pack pack;
 		high_performance_seed(pack);
 		apply_pack(&pack, def, 0);
@@ -421,7 +414,6 @@ namespace libtorrent
 	session_settings::session_settings(std::string const& user_agent_)
 	{
 		aux::session_settings def;
-		initialize_default_settings(def);
 		def.set_str(settings_pack::user_agent, user_agent_);
 		load_struct_from_settings(def, *this);
 	}
