@@ -172,6 +172,7 @@ namespace libtorrent
 	void session_handle::async_add_torrent(add_torrent_params const& params)
 	{
 		add_torrent_params* p = new add_torrent_params(params);
+		p->save_path = complete(p->save_path);
 #ifndef TORRENT_NO_DEPRECATE
 		if (params.tracker_url)
 		{
@@ -370,6 +371,7 @@ namespace libtorrent
 #endif
 	}
 
+#ifndef TORRENT_NO_DEPRECATE
 	void session_handle::add_dht_router(std::pair<std::string, int> const& node)
 	{
 #ifndef TORRENT_DISABLE_DHT
@@ -378,6 +380,7 @@ namespace libtorrent
 		TORRENT_UNUSED(node);
 #endif
 	}
+#endif // TORRENT_NO_DEPRECATE
 
 	void session_handle::dht_get_item(sha1_hash const& target)
 	{
@@ -782,14 +785,19 @@ namespace libtorrent
 		set_proxy(s);
 	}
 
-	void session_handle::set_web_seed_proxy(proxy_settings const& s)
+	void session_handle::set_web_seed_proxy(proxy_settings const&)
 	{
-		set_proxy(s);
+		// NO-OP
 	}
 
 	void session_handle::set_tracker_proxy(proxy_settings const& s)
 	{
-		set_proxy(s);
+		// if the tracker proxy is enabled, set the "proxy_tracker_connections"
+		// setting
+		settings_pack pack;
+		pack.set_bool(settings_pack::proxy_tracker_connections
+			, s.type != aux::proxy_settings::none);
+		apply_settings(pack);
 	}
 
 	proxy_settings session_handle::peer_proxy() const
@@ -804,12 +812,14 @@ namespace libtorrent
 
 	proxy_settings session_handle::tracker_proxy() const
 	{
-		return proxy();
+		settings_pack const sett = get_settings();
+		return sett.get_bool(settings_pack::proxy_tracker_connections)
+			? proxy_settings(sett) : proxy_settings();
 	}
 
-	void session_handle::set_dht_proxy(proxy_settings const& s)
+	void session_handle::set_dht_proxy(proxy_settings const&)
 	{
-		set_proxy(s);
+		// NO-OP
 	}
 
 	proxy_settings session_handle::dht_proxy() const
