@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2009-2014, Arvid Norberg
+Copyright (c) 2009-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,7 +30,9 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
+#include "libtorrent/config.hpp"
 #include "libtorrent/socket_type.hpp"
+#include "libtorrent/aux_/openssl.hpp"
 
 #ifdef TORRENT_USE_OPENSSL
 #include <boost/asio/ssl/context.hpp>
@@ -41,6 +43,10 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #endif
 
+#if defined TORRENT_ASIO_DEBUGGING
+#include "libtorrent/debug.hpp"
+#endif
+
 namespace libtorrent
 {
 
@@ -50,7 +56,7 @@ namespace libtorrent
 #define CASE(t) case socket_type_int_impl<ssl_stream<t> >::value:
 		switch (s.type())
 		{
-			CASE(stream_socket)
+			CASE(tcp::socket)
 			CASE(socks5_stream)
 			CASE(http_stream)
 			CASE(utp_stream)
@@ -59,6 +65,7 @@ namespace libtorrent
 		};
 #undef CASE
 #else
+		TORRENT_UNUSED(s);
 		return false;
 #endif
 	}
@@ -89,15 +96,18 @@ namespace libtorrent
 		// for SSL connections, make sure to authenticate the hostname
 		// of the certificate
 #define CASE(t) case socket_type_int_impl<ssl_stream<t> >::value: \
-		s.get<ssl_stream<t> >()->set_verify_callback(asio::ssl::rfc2818_verification(hostname), ec); \
-		ctx = SSL_get_SSL_CTX(s.get<ssl_stream<t> >()->native_handle()); \
+		s.get<ssl_stream<t> >()->set_verify_callback( \
+			boost::asio::ssl::rfc2818_verification(hostname), ec); \
+		ssl = s.get<ssl_stream<t> >()->native_handle(); \
+		ctx = SSL_get_SSL_CTX(ssl); \
 		break;
 
+		SSL* ssl = 0;
 		SSL_CTX* ctx = 0;
 
 		switch(s.type())
 		{
-			CASE(stream_socket)
+			CASE(tcp::socket)
 			CASE(socks5_stream)
 			CASE(http_stream)
 			CASE(utp_stream)
@@ -107,15 +117,29 @@ namespace libtorrent
 #if OPENSSL_VERSION_NUMBER >= 0x90812f
 		if (ctx)
 		{
-			SSL_CTX_set_tlsext_servername_callback(ctx, 0);
-			SSL_CTX_set_tlsext_servername_arg(ctx, 0);
+			aux::openssl_set_tlsext_servername_callback(ctx, 0);
+			aux::openssl_set_tlsext_servername_arg(ctx, 0);
 		}
 #endif // OPENSSL_VERSION_NUMBER
 
+#ifdef SSL_CTRL_SET_TLSEXT_HOSTNAME
+		if (ssl)
+		{
+			aux::openssl_set_tlsext_hostname(ssl, hostname.c_str());
+		}
+#endif
+
+#else
+		TORRENT_UNUSED(ec);
+		TORRENT_UNUSED(hostname);
+		TORRENT_UNUSED(s);
 #endif
 	}
 
-	void on_close_socket(socket_type* s, boost::shared_ptr<void> holder)
+#ifdef TORRENT_USE_OPENSSL
+	namespace {
+
+	void on_close_socket(socket_type* s, boost::shared_ptr<void>)
 	{
 #if defined TORRENT_ASIO_DEBUGGING
 		complete_async("on_close_socket");
@@ -124,6 +148,9 @@ namespace libtorrent
 		s->close(ec);
 	}
 
+	} // anonymous namespace
+#endif
+
 	// the second argument is a shared pointer to an object that
 	// will keep the socket (s) alive for the duration of the async operation
 	void async_shutdown(socket_type& s, boost::shared_ptr<void> holder)
@@ -137,6 +164,7 @@ namespace libtorrent
 #else
 #define MAYBE_ASIO_DEBUGGING
 #endif
+
 #define CASE(t) case socket_type_int_impl<ssl_stream<t> >::value: \
 	MAYBE_ASIO_DEBUGGING \
 	s.get<ssl_stream<t> >()->async_shutdown(boost::bind(&on_close_socket, &s, holder)); \
@@ -144,7 +172,7 @@ namespace libtorrent
 
 		switch (s.type())
 		{
-			CASE(stream_socket)
+			CASE(tcp::socket)
 			CASE(socks5_stream)
 			CASE(http_stream)
 			CASE(utp_stream)
@@ -152,17 +180,19 @@ namespace libtorrent
 		}
 #undef CASE
 #else
+		TORRENT_UNUSED(holder);
 		s.close(e);
 #endif // TORRENT_USE_OPENSSL
 	}
 
 	void socket_type::destruct()
 	{
+		typedef tcp::socket tcp_socket;
 		switch (m_type)
 		{
 			case 0: break;
-			case socket_type_int_impl<stream_socket>::value:
-				get<stream_socket>()->~stream_socket();
+			case socket_type_int_impl<tcp::socket>::value:
+				get<tcp::socket>()->~tcp_socket();
 				break;
 			case socket_type_int_impl<socks5_stream>::value:
 				get<socks5_stream>()->~socks5_stream();
@@ -179,8 +209,8 @@ namespace libtorrent
 				break;
 #endif
 #ifdef TORRENT_USE_OPENSSL
-			case socket_type_int_impl<ssl_stream<stream_socket> >::value:
-				get<ssl_stream<stream_socket> >()->~ssl_stream();
+			case socket_type_int_impl<ssl_stream<tcp::socket> >::value:
+				get<ssl_stream<tcp::socket> >()->~ssl_stream();
 				break;
 			case socket_type_int_impl<ssl_stream<socks5_stream> >::value:
 				get<ssl_stream<socks5_stream> >()->~ssl_stream();
@@ -199,47 +229,51 @@ namespace libtorrent
 
 	void socket_type::construct(int type, void* userdata)
 	{
+#ifndef TORRENT_USE_OPENSSL
+		TORRENT_UNUSED(userdata);
+#endif
+
 		destruct();
 		switch (type)
 		{
 			case 0: break;
-			case socket_type_int_impl<stream_socket>::value:
-				new ((stream_socket*)m_data) stream_socket(m_io_service);
+			case socket_type_int_impl<tcp::socket>::value:
+				new (reinterpret_cast<tcp::socket*>(&m_data)) tcp::socket(m_io_service);
 				break;
 			case socket_type_int_impl<socks5_stream>::value:
-				new ((socks5_stream*)m_data) socks5_stream(m_io_service);
+				new (reinterpret_cast<socks5_stream*>(&m_data)) socks5_stream(m_io_service);
 				break;
 			case socket_type_int_impl<http_stream>::value:
-				new ((http_stream*)m_data) http_stream(m_io_service);
+				new (reinterpret_cast<http_stream*>(&m_data)) http_stream(m_io_service);
 				break;
 			case socket_type_int_impl<utp_stream>::value:
-				new ((utp_stream*)m_data) utp_stream(m_io_service);
+				new (reinterpret_cast<utp_stream*>(&m_data)) utp_stream(m_io_service);
 				break;
 #if TORRENT_USE_I2P
 			case socket_type_int_impl<i2p_stream>::value:
-				new ((i2p_stream*)m_data) i2p_stream(m_io_service);
+				new (reinterpret_cast<i2p_stream*>(&m_data)) i2p_stream(m_io_service);
 				break;
 #endif
 #ifdef TORRENT_USE_OPENSSL
-			case socket_type_int_impl<ssl_stream<stream_socket> >::value:
+			case socket_type_int_impl<ssl_stream<tcp::socket> >::value:
 				TORRENT_ASSERT(userdata);
-				new ((ssl_stream<stream_socket>*)m_data) ssl_stream<stream_socket>(m_io_service
-					, *((boost::asio::ssl::context*)userdata));
+				new (reinterpret_cast<ssl_stream<tcp::socket>*>(&m_data)) ssl_stream<tcp::socket>(m_io_service
+					, *static_cast<ssl::context*>(userdata));
 				break;
 			case socket_type_int_impl<ssl_stream<socks5_stream> >::value:
 				TORRENT_ASSERT(userdata);
-				new ((ssl_stream<socks5_stream>*)m_data) ssl_stream<socks5_stream>(m_io_service
-					, *((boost::asio::ssl::context*)userdata));
+				new (reinterpret_cast<ssl_stream<socks5_stream>*>(&m_data)) ssl_stream<socks5_stream>(m_io_service
+					, *static_cast<ssl::context*>(userdata));
 				break;
 			case socket_type_int_impl<ssl_stream<http_stream> >::value:
 				TORRENT_ASSERT(userdata);
-				new ((ssl_stream<http_stream>*)m_data) ssl_stream<http_stream>(m_io_service
-					, *((boost::asio::ssl::context*)userdata));
+				new (reinterpret_cast<ssl_stream<http_stream>*>(&m_data)) ssl_stream<http_stream>(m_io_service
+					, *static_cast<ssl::context*>(userdata));
 				break;
 			case socket_type_int_impl<ssl_stream<utp_stream> >::value:
 				TORRENT_ASSERT(userdata);
-				new ((ssl_stream<utp_stream>*)m_data) ssl_stream<utp_stream>(m_io_service
-					, *((boost::asio::ssl::context*)userdata));
+				new (reinterpret_cast<ssl_stream<utp_stream>*>(&m_data)) ssl_stream<utp_stream>(m_io_service
+					, *static_cast<ssl::context*>(userdata));
 				break;
 #endif
 			default: TORRENT_ASSERT(false);
@@ -295,6 +329,36 @@ namespace libtorrent
 		TORRENT_SOCKTYPE_FORWARD(close(ec))
 	}
 
+	void socket_type::set_close_reason(boost::uint16_t code)
+	{
+		switch (m_type)
+		{
+			case socket_type_int_impl<utp_stream>::value:
+				get<utp_stream>()->set_close_reason(code);
+				break;
+#ifdef TORRENT_USE_OPENSSL
+			case socket_type_int_impl<ssl_stream<utp_stream> >::value:
+				get<ssl_stream<utp_stream> >()->lowest_layer().set_close_reason(code);
+				break;
+#endif
+			default: break;
+		}
+	}
+
+	boost::uint16_t socket_type::get_close_reason()
+	{
+		switch (m_type)
+		{
+			case socket_type_int_impl<utp_stream>::value:
+				return get<utp_stream>()->get_close_reason();
+#ifdef TORRENT_USE_OPENSSL
+			case socket_type_int_impl<ssl_stream<utp_stream> >::value:
+				return get<ssl_stream<utp_stream> >()->lowest_layer().get_close_reason();
+#endif
+			default: return 0;
+		}
+	}
+
 	socket_type::endpoint_type socket_type::local_endpoint(error_code& ec) const
 	{ TORRENT_SOCKTYPE_FORWARD_RET(local_endpoint(ec), socket_type::endpoint_type()) }
 
