@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2009-2014, Arvid Norberg
+Copyright (c) 2009-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -45,15 +45,16 @@ bool compare_less_wrap(boost::uint32_t lhs, boost::uint32_t rhs
 
 boost::uint32_t timestamp_history::add_sample(boost::uint32_t sample, bool step)
 {
-	if (!m_initialized)
+	if (!initialized())
 	{
 		for (int i = 0; i < history_size; ++i)
 			m_history[i] = sample;
 		m_base = sample;
-		m_initialized = true;
+		m_num_samples = 0;
 	}
 
-	++m_num_samples;
+	// don't let the counter wrap
+	if (m_num_samples < 0xfffe) ++m_num_samples;
 
 	// if sample is less than base, update the base
 	// and update the history entry (because it will
@@ -93,7 +94,7 @@ boost::uint32_t timestamp_history::add_sample(boost::uint32_t sample, bool step)
 
 void timestamp_history::adjust_base(int change)
 {
-	TORRENT_ASSERT(m_initialized);
+	TORRENT_ASSERT(initialized());
 	m_base += change;
 	// make sure this adjustment sticks by updating all history slots
 	for (int i = 0; i < history_size; ++i)
