@@ -77,6 +77,8 @@ namespace libtorrent
 
 		TORRENT_ASSERT(is_outgoing());
 
+		TORRENT_ASSERT(!m_torrent.lock()->is_upload_only());
+
 		// we only want left-over bandwidth
 		// TODO: introduce a web-seed default class which has a low download priority
 
@@ -113,9 +115,14 @@ namespace libtorrent
 
 	void web_connection_base::start()
 	{
-		set_upload_only(true);
-		if (is_disconnecting()) return;
+		// avoid calling torrent::set_seed because it calls torrent::check_invariant
+		// which fails because the m_num_connecting count is not consistent until
+		// after we call peer_connection::start
+		m_upload_only = true;
 		peer_connection::start();
+		// disconnect_if_redundant must be called after start to keep
+		// m_num_connecting consistent
+		disconnect_if_redundant();
 	}
 
 	web_connection_base::~web_connection_base()
@@ -140,28 +147,35 @@ namespace libtorrent
 	{
 		request += "Host: ";
 		request += m_host;
-		if (m_first_request || m_settings.get_bool(settings_pack::always_send_user_agent)) {
+		if ((m_first_request || m_settings.get_bool(settings_pack::always_send_user_agent))
+			&& !m_settings.get_bool(settings_pack::anonymous_mode))
+		{
 			request += "\r\nUser-Agent: ";
 			request += m_settings.get_str(settings_pack::user_agent);
 		}
-		if (!m_external_auth.empty()) {
+		if (!m_external_auth.empty())
+		{
 			request += "\r\nAuthorization: ";
 			request += m_external_auth;
-		} else if (!m_basic_auth.empty()) {
+		}
+		else if (!m_basic_auth.empty())
+		{
 			request += "\r\nAuthorization: Basic ";
 			request += m_basic_auth;
 		}
-		if (sett.get_int(settings_pack::proxy_type) == settings_pack::http_pw) {
+		if (sett.get_int(settings_pack::proxy_type) == settings_pack::http_pw)
+		{
 			request += "\r\nProxy-Authorization: Basic ";
 			request += base64encode(sett.get_str(settings_pack::proxy_username)
 				+ ":" + sett.get_str(settings_pack::proxy_password));
 		}
 		for (web_seed_entry::headers_t::const_iterator it = m_extra_headers.begin();
-		     it != m_extra_headers.end(); ++it) {
-		  request += "\r\n";
-		  request += it->first;
-		  request += ": ";
-		  request += it->second;
+			it != m_extra_headers.end(); ++it)
+		{
+			request += "\r\n";
+			request += it->first;
+			request += ": ";
+			request += it->second;
 		}
 		if (using_proxy) {
 			request += "\r\nProxy-Connection: keep-alive";
