@@ -42,8 +42,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/alert_types.hpp"
 #include "libtorrent/create_torrent.hpp"
 #include "libtorrent/socket_io.hpp" // print_endpoint
-#include "libtorrent/socket_type.hpp"
-#include "libtorrent/instantiate_connection.hpp"
 #include "libtorrent/ip_filter.hpp"
 #include "libtorrent/session_stats.hpp"
 #include "libtorrent/thread.hpp"
@@ -59,11 +57,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "test_utils.hpp"
 #include "setup_transfer.hpp"
 
-#ifdef TORRENT_USE_OPENSSL
-#include <boost/asio/ssl/stream.hpp>
-#include <boost/asio/ssl/context.hpp>
-#endif
-
 #ifndef _WIN32
 #include <spawn.h>
 #include <signal.h>
@@ -151,7 +144,7 @@ std::map<std::string, boost::int64_t> get_counters(libtorrent::session& s)
 	return ret;
 }
 
-alert const* wait_for_alert(lt::session& ses, int type, char const* name)
+alert const* wait_for_alert(lt::session& ses, int type, char const* name, int num)
 {
 	time_point end = libtorrent::clock_type::now() + seconds(10);
 	while (true)
@@ -167,14 +160,15 @@ alert const* wait_for_alert(lt::session& ses, int type, char const* name)
 		for (std::vector<alert*>::iterator i = alerts.begin()
 			, end(alerts.end()); i != end; ++i)
 		{
-			fprintf(stderr, "%s: %s: [%s] %s\n", time_now_string(), name
+			fprintf(stdout, "%s: %s: [%s] %s\n", time_now_string(), name
 				, (*i)->what(), (*i)->message().c_str());
-			if ((*i)->type() == type && !ret)
+			if ((*i)->type() == type)
 			{
 				ret = *i;
+				--num;
 			}
 		}
-		if (ret) return ret;
+		if (num == 0) return ret;
 	}
 	return NULL;
 }
@@ -263,7 +257,7 @@ void save_file(char const* filename, char const* data, int size)
 
 bool print_alerts(lt::session& ses, char const* name
 	, bool allow_disconnects, bool allow_no_torrents, bool allow_failed_fastresume
-	, bool (*predicate)(libtorrent::alert const*), bool no_output)
+	, boost::function<bool(libtorrent::alert const*)> predicate, bool no_output)
 {
 	bool ret = false;
 	std::vector<torrent_handle> handles = ses.get_torrents();
@@ -277,14 +271,14 @@ bool print_alerts(lt::session& ses, char const* name
 		if (predicate && predicate(*i)) ret = true;
 		if (peer_disconnected_alert const* p = alert_cast<peer_disconnected_alert>(*i))
 		{
-			fprintf(stderr, "%s: %s: [%s] (%s): %s\n", time_now_string(), name, (*i)->what(), print_endpoint(p->ip).c_str(), p->message().c_str());
+			fprintf(stdout, "%s: %s: [%s] (%s): %s\n", time_now_string(), name, (*i)->what(), print_endpoint(p->ip).c_str(), p->message().c_str());
 		}
 		else if ((*i)->message() != "block downloading"
 			&& (*i)->message() != "block finished"
 			&& (*i)->message() != "piece finished"
 			&& !no_output)
 		{
-			fprintf(stderr, "%s: %s: [%s] %s\n", time_now_string(), name, (*i)->what(), (*i)->message().c_str());
+			fprintf(stdout, "%s: %s: [%s] %s\n", time_now_string(), name, (*i)->what(), (*i)->message().c_str());
 		}
 
 		TEST_CHECK(alert_cast<fastresume_rejected_alert>(*i) == 0 || allow_failed_fastresume);
@@ -292,7 +286,7 @@ bool print_alerts(lt::session& ses, char const* name
 		peer_error_alert const* pea = alert_cast<peer_error_alert>(*i);
 		if (pea)
 		{
-			fprintf(stderr, "%s: peer error: %s\n", time_now_string(), pea->error.message().c_str());
+			fprintf(stdout, "%s: peer error: %s\n", time_now_string(), pea->error.message().c_str());
 			TEST_CHECK((!handles.empty() && h.status().is_seeding)
 				|| pea->error.message() == "connecting to peer"
 				|| pea->error.message() == "closing connection to ourself"
@@ -310,7 +304,7 @@ bool print_alerts(lt::session& ses, char const* name
 		invalid_request_alert const* ira = alert_cast<invalid_request_alert>(*i);
 		if (ira)
 		{
-			fprintf(stderr, "peer error: %s\n", ira->message().c_str());
+			fprintf(stdout, "peer error: %s\n", ira->message().c_str());
 			TEST_CHECK(false);
 		}
 	}
@@ -363,7 +357,7 @@ void wait_for_downloading(lt::session& ses, char const* name)
 	} while (a);
 	if (!downloading_done)
 	{
-		fprintf(stderr, "%s: did not receive a state_changed_alert indicating "
+		fprintf(stdout, "%s: did not receive a state_changed_alert indicating "
 			"the torrent is downloading. waited: %d ms\n"
 			, name, int(total_milliseconds(clock_type::now() - start)));
 	}
@@ -376,7 +370,7 @@ void print_ses_rate(float time
 {
 	if (st1)
 	{
-		fprintf(stderr, "%3.1fs | %dkB/s %dkB/s %d%% %d cc:%d%s", time
+		fprintf(stdout, "%3.1fs | %dkB/s %dkB/s %d%% %d cc:%d%s", time
 			, int(st1->download_payload_rate / 1000)
 			, int(st1->upload_payload_rate / 1000)
 			, int(st1->progress * 100)
@@ -385,7 +379,7 @@ void print_ses_rate(float time
 			, st1->errc ? (" [" + st1->errc.message() + "]").c_str() : "");
 	}
 	if (st2)
-		fprintf(stderr, " : %3.1fs | %dkB/s %dkB/s %d%% %d cc:%d%s", time
+		fprintf(stdout, " : %3.1fs | %dkB/s %dkB/s %d%% %d cc:%d%s", time
 			, int(st2->download_payload_rate / 1000)
 			, int(st2->upload_payload_rate / 1000)
 			, int(st2->progress * 100)
@@ -393,7 +387,7 @@ void print_ses_rate(float time
 			, st2->connect_candidates
 			, st2->errc ? (" [" + st1->errc.message() + "]").c_str() : "");
 	if (st3)
-		fprintf(stderr, " : %3.1fs | %dkB/s %dkB/s %d%% %d cc:%d%s", time
+		fprintf(stdout, " : %3.1fs | %dkB/s %dkB/s %d%% %d cc:%d%s", time
 			, int(st3->download_payload_rate / 1000)
 			, int(st3->upload_payload_rate / 1000)
 			, int(st3->progress * 100)
@@ -401,7 +395,7 @@ void print_ses_rate(float time
 			, st3->connect_candidates
 			, st3->errc ? (" [" + st1->errc.message() + "]").c_str() : "");
 
-	fprintf(stderr, "\n");
+	fprintf(stdout, "\n");
 }
 
 void test_sleep(int milliseconds)
@@ -432,7 +426,7 @@ static std::map<int, proxy_t> running_proxies;
 
 void stop_proxy(int port)
 {
-	fprintf(stderr, "stopping proxy on port %d\n", port);
+	fprintf(stdout, "stopping proxy on port %d\n", port);
 	// don't shut down proxies until the test is
 	// completely done. This saves a lot of time.
 	// they're closed at the end of main() by
@@ -458,7 +452,7 @@ pid_type async_run(char const* cmdline)
 	if (ret == 0)
 	{
 		int error = GetLastError();
-		fprintf(stderr, "failed (%d) %s\n", error, error_code(error, system_category()).message().c_str());
+		fprintf(stdout, "failed (%d) %s\n", error, error_code(error, system_category()).message().c_str());
 		return 0;
 	}
 	return pi.dwProcessId;
@@ -572,13 +566,13 @@ int start_proxy(int proxy_type)
 	char buf[512];
 	snprintf(buf, sizeof(buf), "%s --port %d%s", cmd, port, auth);
 
-	fprintf(stderr, "%s starting proxy on port %d (%s %s)...\n", time_now_string(), port, type, auth);
-	fprintf(stderr, "%s\n", buf);
+	fprintf(stdout, "%s starting proxy on port %d (%s %s)...\n", time_now_string(), port, type, auth);
+	fprintf(stdout, "%s\n", buf);
 	pid_type r = async_run(buf);
 	if (r == 0) abort();
 	proxy_t t = { r, proxy_type };
 	running_proxies.insert(std::make_pair(port, t));
-	fprintf(stderr, "%s launched\n", time_now_string());
+	fprintf(stdout, "%s launched\n", time_now_string());
 	test_sleep(500);
 	return port;
 }
@@ -594,7 +588,8 @@ boost::shared_ptr<T> clone_ptr(boost::shared_ptr<T> const& ptr)
 unsigned char random_byte()
 { return std::rand() & 0xff; }
 
-void create_random_files(std::string const& path, const int file_sizes[], int num_files)
+void create_random_files(std::string const& path, const int file_sizes[], int num_files
+	, file_storage* fs)
 {
 	error_code ec;
 	char* random_data = (char*)malloc(300000);
@@ -612,6 +607,7 @@ void create_random_files(std::string const& path, const int file_sizes[], int nu
 		full_path = combine_path(full_path, filename);
 
 		int to_write = file_sizes[i];
+		if (fs) fs->add_file(full_path, to_write);
 		file f(full_path, file::write_only, ec);
 		if (ec) fprintf(stderr, "failed to create file \"%s\": (%d) %s\n"
 			, full_path.c_str(), ec.value(), ec.message().c_str());
@@ -769,7 +765,7 @@ setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 		}
 		char ih_hex[41];
 		to_hex((char const*)&t->info_hash()[0], 20, ih_hex);
-		fprintf(stderr, "generated torrent: %s tmp1%s/temporary\n", ih_hex, suffix.c_str());
+		fprintf(stdout, "generated torrent: %s tmp1%s/temporary\n", ih_hex, suffix.c_str());
 	}
 	else
 	{
@@ -790,7 +786,7 @@ setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 	torrent_handle tor1 = ses1->add_torrent(param, ec);
 	if (ec)
 	{
-		fprintf(stderr, "ses1.add_torrent: %s\n", ec.message().c_str());
+		fprintf(stdout, "ses1.add_torrent: %s\n", ec.message().c_str());
 		return boost::make_tuple(torrent_handle(), torrent_handle(), torrent_handle());
 	}
 	tor1.super_seeding(super_seeding);
@@ -843,16 +839,16 @@ setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 		if (use_ssl_ports)
 		{
 			port = ses2->ssl_listen_port();
-			fprintf(stderr, "%s: ses2->ssl_listen_port(): %d\n", time_now_string(), port);
+			fprintf(stdout, "%s: ses2->ssl_listen_port(): %d\n", time_now_string(), port);
 		}
 
 		if (port == 0)
 		{
 			port = ses2->listen_port();
-			fprintf(stderr, "%s: ses2->listen_port(): %d\n", time_now_string(), port);
+			fprintf(stdout, "%s: ses2->listen_port(): %d\n", time_now_string(), port);
 		}
 
-		fprintf(stderr, "%s: ses1: connecting peer port: %d\n"
+		fprintf(stdout, "%s: ses1: connecting peer port: %d\n"
 			, time_now_string(), port);
 		tor1.connect_peer(tcp::endpoint(address::from_string("127.0.0.1", ec)
 			, port));
@@ -875,10 +871,10 @@ setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 			if (port == 0) port = ses2->listen_port();
 			if (port2 == 0) port2 = ses1->listen_port();
 
-			fprintf(stderr, "ses3: connecting peer port: %d\n", port);
+			fprintf(stdout, "ses3: connecting peer port: %d\n", port);
 			tor3.connect_peer(tcp::endpoint(
 					address::from_string("127.0.0.1", ec), port));
-			fprintf(stderr, "ses3: connecting peer port: %d\n", port2);
+			fprintf(stdout, "ses3: connecting peer port: %d\n", port2);
 				tor3.connect_peer(tcp::endpoint(
 					address::from_string("127.0.0.1", ec)
 					, port2));
@@ -909,13 +905,13 @@ int start_web_server(bool ssl, bool chunked_encoding, bool keepalive)
 	snprintf(buf, sizeof(buf), "python ../web_server.py %d %d %d %d"
 		, port, chunked_encoding , ssl, keepalive);
 
-	fprintf(stderr, "%s starting web_server on port %d...\n", time_now_string(), port);
+	fprintf(stdout, "%s starting web_server on port %d...\n", time_now_string(), port);
 
-	fprintf(stderr, "%s\n", buf);
+	fprintf(stdout, "%s\n", buf);
 	pid_type r = async_run(buf);
 	if (r == 0) abort();
 	web_server_pid = r;
-	fprintf(stderr, "%s launched\n", time_now_string());
+	fprintf(stdout, "%s launched\n", time_now_string());
 	test_sleep(500);
 	return port;
 }
@@ -923,7 +919,7 @@ int start_web_server(bool ssl, bool chunked_encoding, bool keepalive)
 void stop_web_server()
 {
 	if (web_server_pid == 0) return;
-	fprintf(stderr, "stopping web server\n");
+	fprintf(stdout, "stopping web server\n");
 	stop_process(web_server_pid);
 	web_server_pid = 0;
 }
