@@ -226,7 +226,7 @@ void send_dht_request(node& node, char const* msg, udp::endpoint const& ep
 	int size = bencode(msg_buf, e);
 #if defined TORRENT_DEBUG && TORRENT_USE_IOSTREAM
 // this yields a lot of output. too much
-//	std::cerr << "sending: " <<  e << "\n";
+//	std::cout << "sending: " <<  e << "\n";
 #endif
 
 #ifdef TORRENT_USE_VALGRIND
@@ -236,7 +236,7 @@ void send_dht_request(node& node, char const* msg, udp::endpoint const& ep
 	bdecode_node decoded;
 	error_code ec;
 	bdecode(msg_buf, msg_buf + size, decoded, ec);
-	if (ec) fprintf(stderr, "bdecode failed: %s\n", ec.message().c_str());
+	if (ec) fprintf(stdout, "bdecode failed: %s\n", ec.message().c_str());
 
 	dht::msg m(decoded, ep);
 	node.incoming(m);
@@ -285,7 +285,7 @@ void send_dht_response(node& node, bdecode_node const& request, udp::endpoint co
 	bdecode_node decoded;
 	error_code ec;
 	bdecode(msg_buf, msg_buf + size, decoded, ec);
-	if (ec) fprintf(stderr, "bdecode failed: %s\n", ec.message().c_str());
+	if (ec) fprintf(stdout, "bdecode failed: %s\n", ec.message().c_str());
 
 	dht::msg m(decoded, ep);
 	node.incoming(m);
@@ -337,19 +337,19 @@ void announce_immutable_items(node& node, udp::endpoint const* eps
 			bdecode_node parsed[5];
 			char error_string[200];
 
-//			fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
+//			fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
 			int ret = verify_message(response, desc, parsed, error_string
 				, sizeof(error_string));
 			if (ret)
 			{
 				TEST_EQUAL(parsed[4].string_value(), "r");
 				token = parsed[2].string_value();
-//				fprintf(stderr, "got token: %s\n", token.c_str());
+//				fprintf(stdout, "got token: %s\n", token.c_str());
 			}
 			else
 			{
-				fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
-				fprintf(stderr, "   invalid get response: %s\n", error_string);
+				fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
+				fprintf(stdout, "   invalid get response: %s\n", error_string);
 				TEST_ERROR(error_string);
 			}
 
@@ -378,14 +378,14 @@ void announce_immutable_items(node& node, udp::endpoint const* eps
 			if (ret)
 			{
 				if (parsed2[0].string_value() != "r")
-					fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
+					fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
 
 				TEST_EQUAL(parsed2[0].string_value(), "r");
 			}
 			else
 			{
-				fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
-				fprintf(stderr, "   invalid put response: %s\n", error_string);
+				fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
+				fprintf(stdout, "   invalid put response: %s\n", error_string);
 				TEST_ERROR(error_string);
 			}
 		}
@@ -538,7 +538,7 @@ TORRENT_TEST(dht)
 
 	bdecode_node pong_keys[4];
 
-	fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
+	fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
 	ret = dht::verify_message(response, pong_desc, pong_keys, error_string
 		, sizeof(error_string));
 	TEST_CHECK(ret);
@@ -549,7 +549,7 @@ TORRENT_TEST(dht)
 	}
 	else
 	{
-		fprintf(stderr, "   invalid ping response: %s\n", error_string);
+		fprintf(stdout, "   invalid ping response: %s\n", error_string);
 	}
 
 	// ====== invalid message ======
@@ -563,7 +563,7 @@ TORRENT_TEST(dht)
 
 	bdecode_node err_keys[2];
 
-	fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
+	fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
 	ret = dht::verify_message(response, err_desc, err_keys, error_string
 		, sizeof(error_string));
 	TEST_CHECK(ret);
@@ -582,7 +582,7 @@ TORRENT_TEST(dht)
 	}
 	else
 	{
-		fprintf(stderr, "   invalid error response: %s\n", error_string);
+		fprintf(stdout, "   invalid error response: %s\n", error_string);
 	}
 
 	// ====== get_peers ======
@@ -600,7 +600,7 @@ TORRENT_TEST(dht)
 	bdecode_node peer1_keys[4];
 
 	std::string token;
-	fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
+	fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
 	ret = dht::verify_message(response, peer1_desc, peer1_keys, error_string
 		, sizeof(error_string));
 	TEST_CHECK(ret);
@@ -608,12 +608,12 @@ TORRENT_TEST(dht)
 	{
 		TEST_CHECK(peer1_keys[0].string_value() == "r");
 		token = peer1_keys[2].string_value();
-//		fprintf(stderr, "got token: %s\n", token.c_str());
+//		fprintf(stdout, "got token: %s\n", token.c_str());
 	}
 	else
 	{
-		fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
-		fprintf(stderr, "   invalid get_peers response: %s\n", error_string);
+		fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
+		fprintf(stdout, "   invalid get_peers response: %s\n", error_string);
 	}
 
 	// ====== announce ======
@@ -634,7 +634,7 @@ TORRENT_TEST(dht)
 
 	bdecode_node ann_keys[3];
 
-	fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
+	fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
 	ret = dht::verify_message(response, ann_desc, ann_keys, error_string
 		, sizeof(error_string));
 	TEST_CHECK(ret);
@@ -644,7 +644,7 @@ TORRENT_TEST(dht)
 	}
 	else
 	{
-		fprintf(stderr, "   invalid announce response: %s\n", error_string);
+		fprintf(stdout, "   invalid announce response: %s\n", error_string);
 	}
 
 	init_rand_address();
@@ -664,12 +664,12 @@ TORRENT_TEST(dht)
 		{
 			TEST_CHECK(peer1_keys[0].string_value() == "r");
 			token = peer1_keys[2].string_value();
-//			fprintf(stderr, "got token: %s\n", token.c_str());
+//			fprintf(stdout, "got token: %s\n", token.c_str());
 		}
 		else
 		{
-			fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
-			fprintf(stderr, "   invalid get_peers response: %s\n", error_string);
+			fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
+			fprintf(stdout, "   invalid get_peers response: %s\n", error_string);
 		}
 		response.clear();
 		send_dht_request(node, "announce_peer", source, &response
@@ -698,7 +698,7 @@ TORRENT_TEST(dht)
 
 	bdecode_node peer2_keys[5];
 
-	fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
+	fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
 	ret = dht::verify_message(response, peer2_desc, peer2_keys, error_string
 		, sizeof(error_string));
 	TEST_CHECK(ret);
@@ -712,15 +712,15 @@ TORRENT_TEST(dht)
 		downloaders.from_string(peer2_keys[2].string_ptr());
 		seeds.from_string(peer2_keys[3].string_ptr());
 
-		fprintf(stderr, "seeds: %f\n", seeds.size());
-		fprintf(stderr, "downloaders: %f\n", downloaders.size());
+		fprintf(stdout, "seeds: %f\n", seeds.size());
+		fprintf(stdout, "downloaders: %f\n", downloaders.size());
 
 		TEST_CHECK(fabs(seeds.size() - 50.f) <= 3.f);
 		TEST_CHECK(fabs(downloaders.size() - 50.f) <= 3.f);
 	}
 	else
 	{
-		fprintf(stderr, "   invalid get_peers response: %s\n", error_string);
+		fprintf(stdout, "   invalid get_peers response: %s\n", error_string);
 	}
 
 	// ====== test node ID testing =====
@@ -767,14 +767,14 @@ TORRENT_TEST(dht)
 		}
 		else
 		{
-			fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
+			fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
 			TEST_ERROR("invalid error response");
 		}
 	}
 	else
 	{
-		fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
-		fprintf(stderr, "   invalid error response: %s\n", error_string);
+		fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
+		fprintf(stdout, "   invalid error response: %s\n", error_string);
 	}
 
 	// a node with invalid node-id shouldn't be added to routing table.
@@ -793,7 +793,7 @@ TORRENT_TEST(dht)
 
 	bdecode_node nodes_keys[3];
 
-	fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
+	fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
 	ret = dht::verify_message(response, nodes_desc, nodes_keys, error_string
 		, sizeof(error_string));
 	TEST_CHECK(ret);
@@ -803,8 +803,8 @@ TORRENT_TEST(dht)
 	}
 	else
 	{
-		fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
-		fprintf(stderr, "   invalid error response: %s\n", error_string);
+		fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
+		fprintf(stdout, "   invalid error response: %s\n", error_string);
 	}
 	// node with valid node-id should be added to routing table.
 	TEST_EQUAL(node.size().get<0>(), nodes_num + 1);
@@ -839,8 +839,8 @@ TORRENT_TEST(dht)
 
 	// these are test vectors from BEP 33
 	// http://www.bittorrent.org/beps/bep_0033.html
-	fprintf(stderr, "test.size: %f\n", test.size());
-	fprintf(stderr, "%s\n", to_hex(test.to_string()).c_str());
+	fprintf(stdout, "test.size: %f\n", test.size());
+	fprintf(stdout, "%s\n", to_hex(test.to_string()).c_str());
 	if (supports_ipv6())
 	{
 		TEST_CHECK(fabs(test.size() - 1224.93f) < 0.001);
@@ -910,13 +910,13 @@ TORRENT_TEST(dht)
 	for (int with_salt = 0; with_salt < 2; ++with_salt)
 	{
 		seq = 4;
-		fprintf(stderr, "\nTEST GET/PUT%s \ngenerating ed25519 keys\n\n"
+		fprintf(stdout, "\nTEST GET/PUT%s \ngenerating ed25519 keys\n\n"
 			, with_salt ? " with-salt" : " no-salt");
 		unsigned char seed[32];
 		ed25519_create_seed(seed);
 
 		ed25519_create_keypair((unsigned char*)public_key, (unsigned char*)private_key, seed);
-		fprintf(stderr, "pub: %s priv: %s\n"
+		fprintf(stdout, "pub: %s priv: %s\n"
 			, to_hex(std::string(public_key, item_pk_len)).c_str()
 			, to_hex(std::string(private_key, item_sk_len)).c_str());
 
@@ -930,7 +930,7 @@ TORRENT_TEST(dht)
 		if (with_salt) h.update(salt.first, salt.second);
 		sha1_hash target_id = h.final();
 
-		fprintf(stderr, "target_id: %s\n"
+		fprintf(stdout, "target_id: %s\n"
 			, to_hex(target_id.to_string()).c_str());
 
 		send_dht_request(node, "get", source, &response
@@ -953,14 +953,14 @@ TORRENT_TEST(dht)
 		{
 			TEST_EQUAL(desc_keys[4].string_value(), "r");
 			token = desc_keys[2].string_value();
-			fprintf(stderr, "get response: %s\n"
+			fprintf(stdout, "get response: %s\n"
 				, print_entry(response).c_str());
-			fprintf(stderr, "got token: %s\n", to_hex(token).c_str());
+			fprintf(stdout, "got token: %s\n", to_hex(token).c_str());
 		}
 		else
 		{
-			fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
-			fprintf(stderr, "   invalid get response: %s\n%s\n"
+			fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
+			fprintf(stdout, "   invalid get response: %s\n%s\n"
 				, error_string, print_entry(response).c_str());
 			TEST_ERROR(error_string);
 		}
@@ -985,13 +985,13 @@ TORRENT_TEST(dht)
 			, sizeof(error_string));
 		if (ret)
 		{
-			fprintf(stderr, "put response: %s\n"
+			fprintf(stdout, "put response: %s\n"
 				, print_entry(response).c_str());
 			TEST_EQUAL(desc2_keys[0].string_value(), "r");
 		}
 		else
 		{
-			fprintf(stderr, "   invalid put response: %s\n%s\n"
+			fprintf(stdout, "   invalid put response: %s\n%s\n"
 				, error_string, print_entry(response).c_str());
 			TEST_ERROR(error_string);
 		}
@@ -999,7 +999,7 @@ TORRENT_TEST(dht)
 		send_dht_request(node, "get", source, &response
 			, msg_args().target((char*)&target_id[0]));
 
-		fprintf(stderr, "target_id: %s\n"
+		fprintf(stdout, "target_id: %s\n"
 			, to_hex(target_id.to_string()).c_str());
 
 		key_desc_t desc3[] =
@@ -1019,14 +1019,14 @@ TORRENT_TEST(dht)
 			, sizeof(error_string));
 		if (ret == 0)
 		{
-			fprintf(stderr, "msg: %s\n", print_entry(response).c_str());
-			fprintf(stderr, "   invalid get response: %s\n%s\n"
+			fprintf(stdout, "msg: %s\n", print_entry(response).c_str());
+			fprintf(stdout, "   invalid get response: %s\n%s\n"
 				, error_string, print_entry(response).c_str());
 			TEST_ERROR(error_string);
 		}
 		else
 		{
-			fprintf(stderr, "get response: %s\n"
+			fprintf(stdout, "get response: %s\n"
 				, print_entry(response).c_str());
 			char value[1020];
 			char* ptr = value;
@@ -1048,7 +1048,7 @@ TORRENT_TEST(dht)
 		// break the signature
 		signature[2] ^= 0xaa;
 
-		fprintf(stderr, "PUT broken signature\n");
+		fprintf(stdout, "PUT broken signature\n");
 
 		TEST_CHECK(verify_mutable_item(itemv, salt, seq, public_key, signature) != 1);
 
@@ -1065,14 +1065,14 @@ TORRENT_TEST(dht)
 			, sizeof(error_string));
 		if (ret)
 		{
-			fprintf(stderr, "put response: %s\n", print_entry(response).c_str());
+			fprintf(stdout, "put response: %s\n", print_entry(response).c_str());
 			TEST_EQUAL(desc_error_keys[1].string_value(), "e");
 			// 206 is the code for invalid signature
 			TEST_EQUAL(desc_error_keys[0].list_int_value_at(0), 206);
 		}
 		else
 		{
-			fprintf(stderr, "   invalid put response: %s\n%s\n"
+			fprintf(stdout, "   invalid put response: %s\n%s\n"
 				, error_string, print_entry(response).c_str());
 			TEST_ERROR(error_string);
 		}
@@ -1116,7 +1116,7 @@ TORRENT_TEST(dht)
 
 		TEST_CHECK(item_target_id(salt, public_key) == target_id);
 
-		fprintf(stderr, "PUT CAS 1\n");
+		fprintf(stdout, "PUT CAS 1\n");
 
 		send_dht_request(node, "put", source, &response
 			, msg_args()
@@ -1132,18 +1132,18 @@ TORRENT_TEST(dht)
 			, sizeof(error_string));
 		if (ret)
 		{
-			fprintf(stderr, "put response: %s\n"
+			fprintf(stdout, "put response: %s\n"
 				, print_entry(response).c_str());
 			TEST_EQUAL(desc2_keys[0].string_value(), "r");
 		}
 		else
 		{
-			fprintf(stderr, "   invalid put response: %s\n%s\n"
+			fprintf(stdout, "   invalid put response: %s\n%s\n"
 				, error_string, print_entry(response).c_str());
 			TEST_ERROR(error_string);
 		}
 
-		fprintf(stderr, "PUT CAS 2\n");
+		fprintf(stdout, "PUT CAS 2\n");
 
 		// put the same message again. This should fail because the
 		// CAS hash is outdated, it's not the hash of the value that's
@@ -1162,7 +1162,7 @@ TORRENT_TEST(dht)
 			, sizeof(error_string));
 		if (ret)
 		{
-			fprintf(stderr, "put response: %s\n"
+			fprintf(stdout, "put response: %s\n"
 				, print_entry(response).c_str());
 			TEST_EQUAL(desc_error_keys[1].string_value(), "e");
 			// 301 is the error code for CAS hash mismatch
@@ -1170,7 +1170,7 @@ TORRENT_TEST(dht)
 		}
 		else
 		{
-			fprintf(stderr, "   invalid put response: %s\n%s\nExpected failure 301 (CAS hash mismatch)\n"
+			fprintf(stdout, "   invalid put response: %s\n%s\nExpected failure 301 (CAS hash mismatch)\n"
 				, error_string, print_entry(response).c_str());
 			TEST_ERROR(error_string);
 		}
@@ -1354,7 +1354,7 @@ TORRENT_TEST(dht)
 		//#error test need_refresh
 
 #if defined TORRENT_DEBUG
-		table.print_state(std::cerr);
+		table.print_state(std::cout);
 #endif
 
 		table.for_each_node(node_push_back, nop, &nodes);
@@ -1438,7 +1438,7 @@ TORRENT_TEST(dht)
 			TEST_CHECK((id[2] & 0xf8) == (prefixes[i][2] & 0xf8));
 
 			TEST_CHECK(id[19] == rs[i]);
-			fprintf(stderr, "IP address: %s r: %d node ID: %s\n", ips[i]
+			fprintf(stdout, "IP address: %s r: %d node ID: %s\n", ips[i]
 				, rs[i], to_hex(id.to_string()).c_str());
 		}
 	}
@@ -1511,7 +1511,7 @@ TORRENT_TEST(dht)
 		}
 		else
 		{
-			fprintf(stderr, "   invalid find_node request: %s\n", print_entry(response).c_str());
+			fprintf(stdout, "   invalid find_node request: %s\n", print_entry(response).c_str());
 			TEST_ERROR(error_string);
 			break;
 		}
@@ -1539,7 +1539,7 @@ TORRENT_TEST(dht)
 		}
 		else
 		{
-			fprintf(stderr, "   invalid find_node request: %s\n", print_entry(response).c_str());
+			fprintf(stdout, "   invalid find_node request: %s\n", print_entry(response).c_str());
 			TEST_ERROR(error_string);
 			break;
 		}
@@ -1580,7 +1580,7 @@ TORRENT_TEST(dht)
 		}
 		else
 		{
-			fprintf(stderr, "   invalid get_peers request: %s\n", print_entry(response).c_str());
+			fprintf(stdout, "   invalid get_peers request: %s\n", print_entry(response).c_str());
 			TEST_ERROR(error_string);
 			break;
 		}
@@ -1614,7 +1614,7 @@ TORRENT_TEST(dht)
 		}
 		else
 		{
-			fprintf(stderr, "   invalid get_peers request: %s\n", print_entry(response).c_str());
+			fprintf(stdout, "   invalid get_peers request: %s\n", print_entry(response).c_str());
 			TEST_ERROR(error_string);
 			break;
 		}
@@ -1630,7 +1630,7 @@ TORRENT_TEST(dht)
 		for (std::list<std::pair<udp::endpoint, entry> >::iterator i = g_sent_packets.begin()
 			, end(g_sent_packets.end()); i != end; ++i)
 		{
-//			fprintf(stderr, " %s:%d: %s\n", i->first.address().to_string(ec).c_str()
+//			fprintf(stdout, " %s:%d: %s\n", i->first.address().to_string(ec).c_str()
 //				, i->first.port(), i->second.to_string().c_str());
 			TEST_EQUAL(i->second["q"].string(), "announce_peer");
 		}
@@ -1675,7 +1675,7 @@ TORRENT_TEST(dht)
 		}
 		else
 		{
-			fprintf(stderr, "   invalid get request: %s\n", print_entry(response).c_str());
+			fprintf(stdout, "   invalid get request: %s\n", print_entry(response).c_str());
 			TEST_ERROR(error_string);
 			break;
 		}
@@ -1720,7 +1720,7 @@ TORRENT_TEST(dht)
 		}
 		else
 		{
-			fprintf(stderr, "   invalid get request: %s\n", print_entry(response).c_str());
+			fprintf(stdout, "   invalid get request: %s\n", print_entry(response).c_str());
 			TEST_ERROR(error_string);
 			break;
 		}
@@ -1823,7 +1823,7 @@ TORRENT_TEST(dht)
 				, sizeof(error_string));
 			if (!ret)
 			{
-				fprintf(stderr, "   invalid get request: %s\n", print_entry(response).c_str());
+				fprintf(stdout, "   invalid get request: %s\n", print_entry(response).c_str());
 				TEST_ERROR(error_string);
 				continue;
 			}
@@ -1866,7 +1866,7 @@ TORRENT_TEST(dht)
 			}
 			else
 			{
-				fprintf(stderr, "   invalid immutable put request: %s\n", print_entry(response).c_str());
+				fprintf(stdout, "   invalid immutable put request: %s\n", print_entry(response).c_str());
 				TEST_ERROR(error_string);
 				continue;
 			}
@@ -1920,7 +1920,7 @@ TORRENT_TEST(dht)
 				, sizeof(error_string));
 			if (!ret)
 			{
-				fprintf(stderr, "   invalid get request: %s\n", print_entry(response).c_str());
+				fprintf(stdout, "   invalid get request: %s\n", print_entry(response).c_str());
 				TEST_ERROR(error_string);
 				continue;
 			}
@@ -1968,7 +1968,7 @@ TORRENT_TEST(dht)
 			}
 			else
 			{
-				fprintf(stderr, "   invalid put request: %s\n", print_entry(response).c_str());
+				fprintf(stdout, "   invalid put request: %s\n", print_entry(response).c_str());
 				TEST_ERROR(error_string);
 				continue;
 			}
@@ -2035,7 +2035,7 @@ TORRENT_TEST(dht)
 				, sizeof(error_string));
 			if (!ret)
 			{
-				fprintf(stderr, "   invalid get request: %s\n", print_entry(response).c_str());
+				fprintf(stdout, "   invalid get request: %s\n", print_entry(response).c_str());
 				TEST_ERROR(error_string);
 				continue;
 			}
@@ -2161,7 +2161,7 @@ TORRENT_TEST(verify_message)
 	error_code ec;
 	char const test_msg[] = "d1:A4:test1:Bd2:B15:test22:B25:test3ee";
 	bdecode(test_msg, test_msg + sizeof(test_msg)-1, ent, ec);
-	fprintf(stderr, "%s\n", print_entry(ent).c_str());
+	fprintf(stdout, "%s\n", print_entry(ent).c_str());
 
 	bool ret = verify_message(ent, msg_desc, msg_keys, error_string
 		, sizeof(error_string));
@@ -2179,7 +2179,7 @@ TORRENT_TEST(verify_message)
 
 	char const test_msg2[] = "d1:A4:test1:Cd2:C15:test22:C25:test3ee";
 	bdecode(test_msg2, test_msg2 + sizeof(test_msg2)-1, ent, ec);
-	fprintf(stderr, "%s\n", print_entry(ent).c_str());
+	fprintf(stdout, "%s\n", print_entry(ent).c_str());
 
 	ret = verify_message(ent, msg_desc, msg_keys, error_string
 		, sizeof(error_string));
@@ -2198,38 +2198,38 @@ TORRENT_TEST(verify_message)
 
 	char const test_msg3[] = "d1:Cd2:C15:test22:C25:test3ee";
 	bdecode(test_msg3, test_msg3 + sizeof(test_msg3)-1, ent, ec);
-	fprintf(stderr, "%s\n", print_entry(ent).c_str());
+	fprintf(stdout, "%s\n", print_entry(ent).c_str());
 
 	ret = verify_message(ent, msg_desc, msg_keys, error_string
 		, sizeof(error_string));
 	TEST_CHECK(!ret);
-	fprintf(stderr, "%s\n", error_string);
+	fprintf(stdout, "%s\n", error_string);
 	TEST_EQUAL(error_string, std::string("missing 'A' key"));
 
 	char const test_msg4[] = "d1:A6:foobare";
 	bdecode(test_msg4, test_msg4 + sizeof(test_msg4)-1, ent, ec);
-	fprintf(stderr, "%s\n", print_entry(ent).c_str());
+	fprintf(stdout, "%s\n", print_entry(ent).c_str());
 
 	ret = verify_message(ent, msg_desc, msg_keys, error_string
 		, sizeof(error_string));
 	TEST_CHECK(!ret);
-	fprintf(stderr, "%s\n", error_string);
+	fprintf(stdout, "%s\n", error_string);
 	TEST_EQUAL(error_string, std::string("invalid value for 'A'"));
 
 	char const test_msg5[] = "d1:A4:test1:Cd2:C15:test2ee";
 	bdecode(test_msg5, test_msg5 + sizeof(test_msg5)-1, ent, ec);
-	fprintf(stderr, "%s\n", print_entry(ent).c_str());
+	fprintf(stdout, "%s\n", print_entry(ent).c_str());
 
 	ret = verify_message(ent, msg_desc, msg_keys, error_string
 		, sizeof(error_string));
 	TEST_CHECK(!ret);
-	fprintf(stderr, "%s\n", error_string);
+	fprintf(stdout, "%s\n", error_string);
 	TEST_EQUAL(error_string, std::string("missing 'C2' key"));
 
 	// test empty strings [ { "":1 }, "" ]
 	char const test_msg6[] = "ld0:i1ee0:e";
 	bdecode(test_msg6, test_msg6 + sizeof(test_msg6)-1, ent, ec);
-	fprintf(stderr, "%s\n", print_entry(ent).c_str());
+	fprintf(stdout, "%s\n", print_entry(ent).c_str());
 	TEST_CHECK(ent.type() == bdecode_node::list_t);
 	if (ent.type() == bdecode_node::list_t)
 	{
@@ -2282,8 +2282,8 @@ TORRENT_TEST(routing_table_uniform)
 	// i.e. no more than 5 levels
 	TEST_EQUAL(tbl.num_active_buckets(), 5);
 
-#if defined TORRENT_DHT_VERBOSE_LOGGING || defined TORRENT_DEBUG
-	tbl.print_state(std::cerr);
+#if defined TORRENT_DEBUG
+	tbl.print_state(std::cout);
 #endif
 }
 
@@ -2308,7 +2308,7 @@ TORRENT_TEST(routing_table_balance)
 	TEST_EQUAL(tbl.num_active_buckets(), 2);
 
 #if defined TORRENT_DEBUG
-	tbl.print_state(std::cerr);
+	tbl.print_state(std::cout);
 #endif
 }
 
@@ -2337,7 +2337,7 @@ TORRENT_TEST(routing_table_extended)
 	TEST_EQUAL(tbl.num_active_buckets(), 6);
 
 #if defined TORRENT_DEBUG
-	tbl.print_state(std::cerr);
+	tbl.print_state(std::cout);
 #endif
 }
 
@@ -2372,7 +2372,7 @@ TORRENT_TEST(routing_table_set_id)
 	tbl.for_each_node(boost::bind(&inserter, &original_nodes, _1));
 
 #if defined TORRENT_DEBUG
-	tbl.print_state(std::cerr);
+	tbl.print_state(std::cout);
 #endif
 
 	id = to_hash("ffffffffffffffffffffffffffffffffffffffff");
@@ -2392,7 +2392,7 @@ TORRENT_TEST(routing_table_set_id)
 	TEST_EQUAL(intersection.size(), remaining_nodes.size());
 
 #if defined TORRENT_DEBUG
-	tbl.print_state(std::cerr);
+	tbl.print_state(std::cout);
 #endif
 }
 
@@ -2504,7 +2504,7 @@ TORRENT_TEST(invalid_error_msg)
 	bdecode_node decoded;
 	error_code ec;
 	bdecode(msg_buf, msg_buf + size, decoded, ec);
-	if (ec) fprintf(stderr, "bdecode failed: %s\n", ec.message().c_str());
+	if (ec) fprintf(stdout, "bdecode failed: %s\n", ec.message().c_str());
 
 	dht::msg m(decoded, source);
 	node.incoming(m);
@@ -2563,7 +2563,7 @@ TORRENT_TEST(rpc_invalid_error_msg)
 	bdecode_node decoded;
 	error_code ec;
 	bdecode(msg_buf, msg_buf + size, decoded, ec);
-	if (ec) fprintf(stderr, "bdecode failed: %s\n", ec.message().c_str());
+	if (ec) fprintf(stdout, "bdecode failed: %s\n", ec.message().c_str());
 
 	dht::msg m(decoded, source);
 	node_id nid;
@@ -2647,5 +2647,49 @@ TORRENT_TEST(dht_verify_node_address)
 	TEST_EQUAL(table.size().get<0>(), 1);
 	TEST_EQUAL(nodes.size(), 1);
 }
+
+TORRENT_TEST(routing_table_for_each)
+{
+	dht_settings sett = test_settings();
+	obs observer;
+
+	sett.extended_routing_table = false;
+	node_id id = to_hash("1234876923549721020394873245098347598635");
+
+	dht::routing_table tbl(id, 2, sett, &observer);
+
+	for (int i = 0; i < 32; ++i)
+	{
+		id[4] = i;
+		tbl.node_seen(id, rand_udp_ep(), 20 + (id[19] & 0xff));
+	}
+
+	int nodes;
+	int replacements;
+	boost::tie(nodes, replacements, boost::tuples::ignore) = tbl.size();
+
+	std::printf("num_active_buckets: %d\n", tbl.num_active_buckets());
+	std::printf("live nodes: %d\n", nodes);
+	std::printf("replacements: %d\n", replacements);
+
+	TEST_EQUAL(tbl.num_active_buckets(), 2);
+	TEST_EQUAL(nodes, 2);
+	TEST_EQUAL(replacements, 2);
+
+#if defined TORRENT_DEBUG
+	tbl.print_state(std::cout);
+#endif
+
+	std::vector<node_entry> v;
+	tbl.for_each_node(node_push_back, nop, &v);
+	TEST_EQUAL(v.size(), 2);
+	v.clear();
+	tbl.for_each_node(nop, node_push_back, &v);
+	TEST_EQUAL(v.size(), 2);
+	v.clear();
+	tbl.for_each_node(node_push_back, node_push_back, &v);
+	TEST_EQUAL(v.size(), 4);
+}
+
 #endif
 
