@@ -273,14 +273,20 @@ void do_handshake(tcp::socket& s, sha1_hash const& ih, char* buffer)
 	// check for fast extension support
 	TEST_CHECK(extensions[7] & 0x4);
 
-#ifndef TORRENT_DISABLE_EXTENSIONS
 	// check for extension protocol support
-	TEST_CHECK(extensions[5] & 0x10);
+	bool const lt_extension_protocol = (extensions[5] & 0x10) != 0;
+#ifndef TORRENT_DISABLE_EXTENSIONS
+	TEST_CHECK(lt_extension_protocol == true);
+#else
+	TEST_CHECK(lt_extension_protocol == false);
 #endif
 
-#ifndef TORRENT_DISABLE_DHT
 	// check for DHT support
-	TEST_CHECK(extensions[7] & 0x1);
+	bool const dht_support = (extensions[7] & 0x1) != 0;
+#ifndef TORRENT_DISABLE_DHT
+	TEST_CHECK(dht_support == true);
+#else
+	TEST_CHECK(dht_support == false);
 #endif
 
 	TEST_CHECK(std::memcmp(buffer + 28, ih.begin(), 20) == 0);
@@ -402,7 +408,8 @@ entry read_ut_metadata_msg(tcp::socket& s, char* recv_buffer, int size)
 }
 
 boost::shared_ptr<torrent_info> setup_peer(tcp::socket& s, sha1_hash& ih
-	, boost::shared_ptr<lt::session>& ses, torrent_handle* th = NULL)
+	, boost::shared_ptr<lt::session>& ses, bool incoming = true
+	, int flags = 0, torrent_handle* th = NULL)
 {
 	boost::shared_ptr<torrent_info> t = ::create_torrent();
 	ih = t->info_hash();
@@ -413,12 +420,17 @@ boost::shared_ptr<torrent_info> setup_peer(tcp::socket& s, sha1_hash& ih
 	sett.set_bool(settings_pack::enable_natpmp, false);
 	sett.set_bool(settings_pack::enable_lsd, false);
 	sett.set_bool(settings_pack::enable_dht, false);
+	sett.set_int(settings_pack::in_enc_policy, settings_pack::pe_disabled);
+	sett.set_int(settings_pack::out_enc_policy, settings_pack::pe_disabled);
+	sett.set_bool(settings_pack::enable_outgoing_utp, false);
+	sett.set_bool(settings_pack::enable_incoming_utp, false);
 	ses.reset(new lt::session(sett, lt::session::add_default_plugins));
 
 	error_code ec;
 	add_torrent_params p;
 	p.flags &= ~add_torrent_params::flag_paused;
 	p.flags &= ~add_torrent_params::flag_auto_managed;
+	p.flags |= flags;
 	p.ti = t;
 	p.save_path = "./tmp1_fast";
 
@@ -429,10 +441,29 @@ boost::shared_ptr<torrent_info> setup_peer(tcp::socket& s, sha1_hash& ih
 	if (th) *th = ret;
 
 	// wait for the torrent to be ready
-	wait_for_downloading(*ses, "ses");
+	if ((flags & add_torrent_params::flag_seed_mode) == 0)
+	{
+		wait_for_downloading(*ses, "ses");
+	}
 
-	s.connect(tcp::endpoint(address::from_string("127.0.0.1", ec), ses->listen_port()), ec);
-	if (ec) TEST_ERROR(ec.message());
+	if (incoming)
+	{
+		s.connect(tcp::endpoint(address::from_string("127.0.0.1", ec), ses->listen_port()), ec);
+		if (ec) TEST_ERROR(ec.message());
+	}
+	else
+	{
+		tcp::acceptor l(s.get_io_service());
+		l.open(tcp::v4());
+		l.bind(tcp::endpoint(address_v4::from_string("127.0.0.1")
+			, 3000 + rand() % 60000));
+		l.listen();
+		tcp::endpoint addr = l.local_endpoint();
+
+		ret.connect_peer(addr);
+		print_session_log(*ses);
+		l.accept(s);
+	}
 
 	print_session_log(*ses);
 
@@ -595,7 +626,7 @@ TORRENT_TEST(reject_suggest)
 
 		using namespace libtorrent::detail;
 		char* ptr = recv_buffer + 1;
-		int piece = read_int32(ptr);
+		int const piece = read_int32(ptr);
 
 		std::vector<int>::iterator i = std::find(suggested.begin()
 			, suggested.end(), piece);
@@ -629,6 +660,66 @@ TORRENT_TEST(reject_suggest)
 	print_session_log(*ses);
 }
 
+TORRENT_TEST(suggest_order)
+{
+	std::cerr << "\n === test suggest ===\n" << std::endl;
+
+	sha1_hash ih;
+	boost::shared_ptr<lt::session> ses;
+	io_service ios;
+	tcp::socket s(ios);
+	setup_peer(s, ih, ses);
+
+	char recv_buffer[1000];
+	do_handshake(s, ih, recv_buffer);
+	print_session_log(*ses);
+	send_have_all(s);
+	print_session_log(*ses);
+
+	std::vector<int> suggested;
+	suggested.push_back(0);
+	suggested.push_back(1);
+	suggested.push_back(2);
+	suggested.push_back(3);
+
+	std::for_each(suggested.begin(), suggested.end()
+		, boost::bind(&send_suggest_piece, boost::ref(s), _1));
+	print_session_log(*ses);
+
+	send_unchoke(s);
+	print_session_log(*ses);
+
+	int fail_counter = 100;
+	while (!suggested.empty() && fail_counter > 0)
+	{
+		print_session_log(*ses);
+		int len = read_message(s, recv_buffer, sizeof(recv_buffer));
+		if (len == -1) break;
+		print_message(recv_buffer, len);
+		int msg = recv_buffer[0];
+		fail_counter--;
+
+		// we're just interested in requests
+		if (msg != 0x6) continue;
+
+		using namespace libtorrent::detail;
+		char* ptr = recv_buffer + 1;
+		int const piece = read_int32(ptr);
+
+		// make sure we receive the requests inverse order of sending the suggest
+		// messages. The last suggest should be the highest priority
+		int const expected_piece = suggested.back();
+		suggested.pop_back();
+		TEST_EQUAL(piece, expected_piece);
+	}
+	print_session_log(*ses);
+	TEST_CHECK(fail_counter > 0);
+
+	s.close();
+	test_sleep(500);
+	print_session_log(*ses);
+}
+
 TORRENT_TEST(multiple_bitfields)
 {
 	std::cerr << "\n === test multiple bitfields ===\n" << std::endl;
@@ -706,7 +797,7 @@ TORRENT_TEST(dont_have)
 	boost::shared_ptr<lt::session> ses;
 	io_service ios;
 	tcp::socket s(ios);
-	boost::shared_ptr<torrent_info> ti = setup_peer(s, ih, ses, &th);
+	boost::shared_ptr<torrent_info> ti = setup_peer(s, ih, ses, true, 0, &th);
 
 	char recv_buffer[1000];
 	do_handshake(s, ih, recv_buffer);
@@ -875,6 +966,55 @@ TORRENT_TEST(invalid_request)
 	send_request(s, req);
 }
 
+void have_all_test(bool const incoming)
+{
+	sha1_hash ih;
+	boost::shared_ptr<lt::session> ses;
+	io_service ios;
+	tcp::socket s(ios);
+	setup_peer(s, ih, ses, incoming, add_torrent_params::flag_seed_mode);
+
+	char recv_buffer[1000];
+	do_handshake(s, ih, recv_buffer);
+	print_session_log(*ses);
+
+	// expect to receive a have-all (not a bitfield)
+	// since we advertised support for FAST extensions
+	for (;;)
+	{
+		int const len = read_message(s, recv_buffer, sizeof(recv_buffer));
+		if (len == -1)
+		{
+			TEST_ERROR("failed to receive have-all despite advertising support for FAST");
+			break;
+		}
+		print_message(recv_buffer, len);
+		int const msg = recv_buffer[0];
+		if (msg == 0xe) // have-all
+		{
+			// success!
+			break;
+		}
+		if (msg == 5) // bitfield
+		{
+			TEST_ERROR("received bitfield from seed despite advertising support for FAST");
+			break;
+		}
+	}
+}
+
+TORRENT_TEST(outgoing_have_all)
+{
+	std::cerr << "\n === test outgoing have-all ===\n" << std::endl;
+	have_all_test(true);
+}
+
+TORRENT_TEST(incoming_have_all)
+{
+	std::cerr << "\n === test outgoing have-all ===\n" << std::endl;
+	have_all_test(false);
+}
+
 #endif // TORRENT_DISABLE_EXTENSIONS
 
 // TODO: test sending invalid requests (out of bound piece index, offsets and
