@@ -97,7 +97,7 @@ TORRENT_TEST(file_status)
 	TEST_CHECK(!ec);
 
 	int diff = int(st2.mtime - st1.mtime);
-	fprintf(stderr, "timestamp difference: %d seconds. expected approx. 3 seconds\n"
+	fprintf(stdout, "timestamp difference: %d seconds. expected approx. 3 seconds\n"
 		, diff);
 
 	TEST_CHECK(diff >= 2 && diff <= 4);
@@ -108,7 +108,7 @@ TORRENT_TEST(directory)
 	error_code ec;
 
 	create_directory("file_test_dir", ec);
-	if (ec) fprintf(stderr, "create_directory: %s\n", ec.message().c_str());
+	if (ec) fprintf(stdout, "create_directory: %s\n", ec.message().c_str());
 	TEST_CHECK(!ec);
 
 	std::string cwd = current_working_directory();
@@ -123,7 +123,7 @@ TORRENT_TEST(directory)
 		std::string f = i.file();
 		TEST_CHECK(files.count(f) == 0);
 		files.insert(f);
-		fprintf(stderr, " %s\n", f.c_str());
+		fprintf(stdout, " %s\n", f.c_str());
 	}
 
 	TEST_CHECK(files.count("abc") == 1);
@@ -140,13 +140,13 @@ TORRENT_TEST(directory)
 		std::string f = i.file();
 		TEST_CHECK(files.count(f) == 0);
 		files.insert(f);
-		fprintf(stderr, " %s\n", f.c_str());
+		fprintf(stdout, " %s\n", f.c_str());
 	}
 
 	remove_all("file_test_dir", ec);
-	if (ec) fprintf(stderr, "remove_all: %s\n", ec.message().c_str());
+	if (ec) fprintf(stdout, "remove_all: %s\n", ec.message().c_str());
 	remove_all("file_test_dir2", ec);
-	if (ec) fprintf(stderr, "remove_all: %s\n", ec.message().c_str());
+	if (ec) fprintf(stdout, "remove_all: %s\n", ec.message().c_str());
 }
 
 // test path functions
@@ -289,20 +289,20 @@ TORRENT_TEST(file)
 	TEST_CHECK(f.open("test_file", file::read_write, ec));
 #endif
 	if (ec)
-		fprintf(stderr, "open failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
+		fprintf(stdout, "open failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
 	TEST_EQUAL(ec, error_code());
-	if (ec) fprintf(stderr, "%s\n", ec.message().c_str());
+	if (ec) fprintf(stdout, "%s\n", ec.message().c_str());
 	file::iovec_t b = {(void*)"test", 4};
 	TEST_EQUAL(f.writev(0, &b, 1, ec), 4);
 	if (ec)
-		fprintf(stderr, "writev failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
+		fprintf(stdout, "writev failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
 	TEST_CHECK(!ec);
 	char test_buf[5] = {0};
 	b.iov_base = test_buf;
 	b.iov_len = 4;
 	TEST_EQUAL(f.readv(0, &b, 1, ec), 4);
 	if (ec)
-		fprintf(stderr, "readv failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
+		fprintf(stdout, "readv failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
 	TEST_EQUAL(ec, error_code());
 	TEST_CHECK(strcmp(test_buf, "test") == 0);
 	f.close();
@@ -319,26 +319,26 @@ TORRENT_TEST(hard_link)
 	file f;
 	TEST_CHECK(f.open("original_file", file::read_write, ec));
 	if (ec)
-		fprintf(stderr, "open failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
+		fprintf(stdout, "open failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
 	TEST_EQUAL(ec, error_code());
 
 	file::iovec_t b = {(void*)"abcdefghijklmnopqrstuvwxyz", 26};
 	TEST_EQUAL(f.writev(0, &b, 1, ec), 26);
 	if (ec)
-		fprintf(stderr, "writev failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
+		fprintf(stdout, "writev failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
 	TEST_EQUAL(ec, error_code());
 	f.close();
 
 	hard_link("original_file", "second_link", ec);
 
 	if (ec)
-		fprintf(stderr, "hard_link failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
+		fprintf(stdout, "hard_link failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
 	TEST_EQUAL(ec, error_code());
 
 
 	TEST_CHECK(f.open("second_link", file::read_write, ec));
 	if (ec)
-		fprintf(stderr, "open failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
+		fprintf(stdout, "open failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
 	TEST_EQUAL(ec, error_code());
 
 	char test_buf[27] = {0};
@@ -346,18 +346,18 @@ TORRENT_TEST(hard_link)
 	b.iov_len = 27;
 	TEST_EQUAL(f.readv(0, &b, 1, ec), 26);
 	if (ec)
-		fprintf(stderr, "readv failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
+		fprintf(stdout, "readv failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
 	TEST_EQUAL(ec, error_code());
 	TEST_CHECK(strcmp(test_buf, "abcdefghijklmnopqrstuvwxyz") == 0);
 	f.close();
 
 	remove("original_file", ec);
 	if (ec)
-		fprintf(stderr, "remove failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
+		fprintf(stdout, "remove failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
 
 	remove("second_link", ec);
 	if (ec)
-		fprintf(stderr, "remove failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
+		fprintf(stdout, "remove failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
 }
 
 TORRENT_TEST(coalesce_buffer)
@@ -366,13 +366,13 @@ TORRENT_TEST(coalesce_buffer)
 	file f;
 	TEST_CHECK(f.open("test_file", file::read_write, ec));
 	if (ec)
-		fprintf(stderr, "open failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
+		fprintf(stdout, "open failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
 	TEST_EQUAL(ec, error_code());
-	if (ec) fprintf(stderr, "%s\n", ec.message().c_str());
+	if (ec) fprintf(stdout, "%s\n", ec.message().c_str());
 	file::iovec_t b[2] = {{(void*)"test", 4}, {(void*)"foobar", 6}};
 	TEST_EQUAL(f.writev(0, b, 2, ec, file::coalesce_buffers), 4 + 6);
 	if (ec)
-		fprintf(stderr, "writev failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
+		fprintf(stdout, "writev failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
 	TEST_CHECK(!ec);
 	char test_buf1[5] = {0};
 	char test_buf2[7] = {0};
@@ -383,7 +383,7 @@ TORRENT_TEST(coalesce_buffer)
 	TEST_EQUAL(f.readv(0, b, 2, ec), 4 + 6);
 	if (ec)
 	{
-		fprintf(stderr, "readv failed: [%s] %s\n"
+		fprintf(stdout, "readv failed: [%s] %s\n"
 			, ec.category().name(), ec.message().c_str());
 	}
 	TEST_EQUAL(ec, error_code());
