@@ -36,7 +36,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/socket.hpp"
 #include "libtorrent/random.hpp"
 #include "libtorrent/socket_io.hpp"
-#include "libtorrent/aux_/session_impl.hpp"
+#include "libtorrent/aux_/session_interface.hpp"
 #include "libtorrent/broadcast_socket.hpp" // for supports_ipv6()
 #include "setup_transfer.hpp" // for rand_v4
 
@@ -45,12 +45,12 @@ using namespace libtorrent;
 bool cast_vote(ip_voter& ipv, address ext_ip, address voter)
 {
 	bool new_ip = ipv.cast_vote(ext_ip, 1, voter);
-	fprintf(stderr, "%15s -> %-15s\n"
+	fprintf(stdout, "%15s -> %-15s\n"
 		, print_address(voter).c_str()
 		, print_address(ext_ip).c_str());
 	if (new_ip)
 	{
-		fprintf(stderr, "   \x1b[1mnew external IP: %s\x1b[0m\n"
+		fprintf(stdout, "   \x1b[1mnew external IP: %s\x1b[0m\n"
 			, print_address(ipv.external_address()).c_str());
 	}
 	return new_ip;
@@ -165,8 +165,8 @@ TORRENT_TEST(externa_ip_1)
 	TEST_CHECK(!ec);
 	for (int i = 0; i < 50; ++i)
 	{
-		ipv1.cast_vote(real_external, aux::session_impl::source_dht, rand_v4());
-		ipv1.cast_vote(rand_v4(), aux::session_impl::source_dht, malicious);
+		ipv1.cast_vote(real_external, aux::session_interface::source_dht, rand_v4());
+		ipv1.cast_vote(rand_v4(), aux::session_interface::source_dht, malicious);
 	}
 	TEST_CHECK(ipv1.external_address(rand_v4()) == real_external);
 }
@@ -199,12 +199,12 @@ TORRENT_TEST(externa_ip_2)
 	TEST_CHECK(!ec);
 	for (int i = 0; i < 50; ++i)
 	{
-		ipv2.cast_vote(real_external1, aux::session_impl::source_dht, rand_v4());
+		ipv2.cast_vote(real_external1, aux::session_interface::source_dht, rand_v4());
 #if TORRENT_USE_IPV6
 		if (supports_ipv6())
-			ipv2.cast_vote(real_external2, aux::session_impl::source_dht, rand_v6());
+			ipv2.cast_vote(real_external2, aux::session_interface::source_dht, rand_v6());
 #endif
-		ipv2.cast_vote(malicious_external, aux::session_impl::source_dht, malicious);
+		ipv2.cast_vote(malicious_external, aux::session_interface::source_dht, malicious);
 	}
 	TEST_CHECK(ipv2.external_address(rand_v4()) == real_external1);
 #if TORRENT_USE_IPV6
