@@ -30,6 +30,8 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
+#include "test.hpp"
+
 #ifndef TORRENT_DISABLE_EXTENSIONS
 
 #include "libtorrent/session.hpp"
@@ -37,15 +39,17 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/hasher.hpp"
 #include "libtorrent/extensions/ut_pex.hpp"
 #include "libtorrent/thread.hpp"
+#include "libtorrent/ip_filter.hpp"
+#include "libtorrent/torrent_status.hpp"
 #include <boost/tuple/tuple.hpp>
 
-#include "test.hpp"
 #include "setup_transfer.hpp"
 #include <iostream>
 
 void test_pex()
 {
 	using namespace libtorrent;
+	namespace lt = libtorrent;
 
 	// these are declared before the session objects
 	// so that they are destructed last. This enables
@@ -54,9 +58,38 @@ void test_pex()
 	session_proxy p2;
 	session_proxy p3;
 
-	session ses1(fingerprint("LT", 0, 1, 0, 0), std::make_pair(48200, 49000), "0.0.0.0", 0);
-	session ses2(fingerprint("LT", 0, 1, 0, 0), std::make_pair(49200, 50000), "0.0.0.0", 0);
-	session ses3(fingerprint("LT", 0, 1, 0, 0), std::make_pair(50200, 51000), "0.0.0.0", 0);
+	int mask = alert::all_categories
+		& ~(alert::progress_notification
+			| alert::performance_warning
+			| alert::stats_notification);
+
+	// this is to avoid everything finish from a single peer
+	// immediately. To make the swarm actually connect all
+	// three peers before finishing.
+	settings_pack pack;
+	pack.set_int(settings_pack::alert_mask, mask);
+	pack.set_int(settings_pack::download_rate_limit, 2000);
+	pack.set_int(settings_pack::upload_rate_limit, 2000);
+	pack.set_int(settings_pack::max_retry_port_bind, 800);
+	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:48200");
+
+	pack.set_bool(settings_pack::enable_dht, false);
+	pack.set_bool(settings_pack::enable_upnp, false);
+	pack.set_bool(settings_pack::enable_natpmp, false);
+
+	pack.set_int(settings_pack::out_enc_policy, settings_pack::pe_forced);
+	pack.set_int(settings_pack::in_enc_policy, settings_pack::pe_forced);
+
+	lt::session ses1(pack);
+
+	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:49200");
+
+	lt::session ses3(pack);
+
+	// make the peer connecting the two worthless to transfer
+	// data, to force peer 3 to connect directly to peer 1 through pex
+	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:50200");
+	lt::session ses2(pack);
 
 	ses1.add_extension(create_ut_pex_plugin);
 	ses2.add_extension(create_ut_pex_plugin);
@@ -67,42 +100,7 @@ void test_pex()
 
 	boost::tie(tor1, tor2, tor3) = setup_transfer(&ses1, &ses2, &ses3, true, false, false, "_pex");
 
-	int mask = alert::all_categories
-		& ~(alert::progress_notification
-			| alert::performance_warning
-			| alert::stats_notification);
-	ses1.set_alert_mask(mask);
-	ses2.set_alert_mask(mask);
-	ses3.set_alert_mask(mask);
-
-	// this is to avoid everything finish from a single peer
-	// immediately. To make the swarm actually connect all
-	// three peers before finishing.
-	session_settings set = ses1.settings();
-	set.download_rate_limit = 2000;
-	set.upload_rate_limit = 2000;
-	ses1.set_settings(set);
-
-	// make the peer connecting the two worthless to transfer
-	// data, to force peer 3 to connect directly to peer 1 through pex
-	set = ses2.settings();
-	set.ignore_limits_on_local_network = false;
-	set.rate_limit_utp = true;
-	ses2.set_settings(set);
-
-	set = ses3.settings();
-	set.download_rate_limit = 0;
-	set.upload_rate_limit = 0;
-	ses3.set_settings(set);
-
-#ifndef TORRENT_DISABLE_ENCRYPTION
-	pe_settings pes;
-	pes.out_enc_policy = pe_settings::forced;
-	pes.in_enc_policy = pe_settings::forced;
-	ses1.set_pe_settings(pes);
-	ses2.set_pe_settings(pes);
-	ses3.set_pe_settings(pes);
-#endif
+	ses2.apply_settings(pack);
 
 	test_sleep(100);
 
@@ -115,7 +113,7 @@ void test_pex()
 	torrent_status st1;
 	torrent_status st2;
 	torrent_status st3;
-	for (int i = 0; i < 80; ++i)
+	for (int i = 0; i < 610; ++i)
 	{
 		print_alerts(ses1, "ses1");
 		print_alerts(ses2, "ses2");
@@ -148,9 +146,11 @@ void test_pex()
 	p2 = ses2.abort();
 	p3 = ses3.abort();
 }
+#endif // TORRENT_DISABLE_EXTENSIONS
 
-int test_main()
+TORRENT_TEST(pex)
 {
+#ifndef TORRENT_DISABLE_EXTENSIONS
 	using namespace libtorrent;
 
 	// in case the previous run was terminated
@@ -160,15 +160,11 @@ int test_main()
 	remove_all("tmp3_pex", ec);
 
 	test_pex();
-	
+
 	remove_all("tmp1_pex", ec);
 	remove_all("tmp2_pex", ec);
 	remove_all("tmp3_pex", ec);
-
-	return 0;
+#endif // TORRENT_DISABLE_EXTENSIONS
 }
 
-#else
-int test_main() { return 0; }
-#endif // TORRENT_DISABLE_EXTENSIONS
 
