@@ -192,6 +192,63 @@ void default_tests(torrent_status const& s)
 	TEST_EQUAL(s.completed_time, 1348);
 }
 
+void test_file_sizes(bool allocate)
+{
+	error_code ec;
+	remove_all("test_resume", ec);
+
+	lt::settings_pack pack = settings();
+	// we're not testing the hash check, just accept the data we write
+	pack.set_bool(settings_pack::disable_hash_checks, true);
+	lt::session ses(pack);
+	boost::shared_ptr<torrent_info> ti = generate_torrent();
+	add_torrent_params p;
+	p.ti = ti;
+	p.save_path = ".";
+	if (allocate) p.storage_mode = storage_mode_allocate;
+	torrent_handle h = ses.add_torrent(p);
+
+	wait_for_downloading(ses, "ses");
+
+	std::vector<char> piece(ti->piece_length(), 0);
+	h.add_piece(0, piece.data());
+
+	h.save_resume_data();
+	alert const* a = wait_for_alert(ses, save_resume_data_alert::alert_type);
+	TEST_CHECK(a);
+
+	save_resume_data_alert const* ra = alert_cast<save_resume_data_alert>(a);
+	TEST_CHECK(ra);
+	if (ra)
+	{
+		fprintf(stderr, "%s\n", ra->resume_data->to_string().c_str());
+		bool const has_file_sizes = ra->resume_data->dict().count("file sizes") == 1;
+		TEST_CHECK(has_file_sizes);
+		if (!has_file_sizes) return;
+		// { 'file sizes': [ [ size, timestamp], [...], ... ] }
+		boost::int64_t const file_size = (*ra->resume_data)["file sizes"].list()
+			.front().list().front().integer();
+		if (allocate)
+		{
+			TEST_EQUAL(file_size, ti->files().file_size(0));
+		}
+		else
+		{
+			TEST_EQUAL(file_size, ti->piece_length());
+		}
+	}
+}
+
+TORRENT_TEST(file_sizes_allocate)
+{
+	test_file_sizes(true);
+}
+
+TORRENT_TEST(file_sizes)
+{
+	test_file_sizes(false);
+}
+
 TORRENT_TEST(piece_priorities)
 {
 	lt::session ses(settings());
@@ -208,7 +265,9 @@ TORRENT_TEST(piece_priorities)
 	alert const* a = wait_for_alert(ses, save_resume_data_alert::alert_type);
 
 	TEST_CHECK(a);
-	if (save_resume_data_alert const* ra = alert_cast<save_resume_data_alert>(a))
+	save_resume_data_alert const* ra = alert_cast<save_resume_data_alert>(a);
+	TEST_CHECK(ra);
+	if (ra)
 	{
 		fprintf(stderr, "%s\n", ra->resume_data->to_string().c_str());
 		entry::string_type prios = (*ra->resume_data)["piece_priority"].string();
@@ -249,6 +308,45 @@ TORRENT_TEST(file_priorities_default)
 	TEST_EQUAL(file_priorities[2], 4);
 }
 
+// As long as the add_torrent_params priorities are empty, the file_priorities
+// from the resume data should take effect
+TORRENT_TEST(file_priorities_in_resume)
+{
+	lt::session ses(settings());
+	std::vector<int> file_priorities = test_resume_flags(ses, 0, "", "123").file_priorities();
+
+	TEST_EQUAL(file_priorities.size(), 3);
+	TEST_EQUAL(file_priorities[0], 1);
+	TEST_EQUAL(file_priorities[1], 2);
+	TEST_EQUAL(file_priorities[2], 3);
+}
+
+// if both resume data and add_torrent_params has file_priorities, the
+// add_torrent_params one take precedence
+TORRENT_TEST(file_priorities_in_resume_and_params)
+{
+	lt::session ses(settings());
+	std::vector<int> file_priorities = test_resume_flags(ses, 0, "456", "123").file_priorities();
+
+	TEST_EQUAL(file_priorities.size(), 3);
+	TEST_EQUAL(file_priorities[0], 4);
+	TEST_EQUAL(file_priorities[1], 5);
+	TEST_EQUAL(file_priorities[2], 6);
+}
+
+// if we set flag_override_resume_data, it should no affect file priorities
+TORRENT_TEST(file_priorities_override_resume)
+{
+	lt::session ses(settings());
+	std::vector<int> file_priorities = test_resume_flags(ses
+		, add_torrent_params::flag_override_resume_data, "", "123").file_priorities();
+
+	TEST_EQUAL(file_priorities.size(), 3);
+	TEST_EQUAL(file_priorities[0], 1);
+	TEST_EQUAL(file_priorities[1], 2);
+	TEST_EQUAL(file_priorities[2], 3);
+}
+
 TORRENT_TEST(file_priorities_resume_seed_mode)
 {
 	// in share mode file priorities should always be 0
