@@ -40,6 +40,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/torrent.hpp"
 #include "libtorrent/peer_info.hpp"
 #include "libtorrent/extensions.hpp"
+#include "libtorrent/magnet_uri.hpp"
 #include "settings.hpp"
 #include <boost/tuple/tuple.hpp>
 #include <boost/make_shared.hpp>
@@ -75,7 +76,7 @@ void test_running_torrent(boost::shared_ptr<torrent_info> info, boost::int64_t f
 	torrent_handle h = ses.add_torrent(p, ec);
 	if (ec)
 	{
-		fprintf(stderr, "add_torrent: %s\n", ec.message().c_str());
+		fprintf(stdout, "add_torrent: %s\n", ec.message().c_str());
 		return;
 	}
 
@@ -392,4 +393,141 @@ TORRENT_TEST(torrent_total_size_zero)
 	TEST_CHECK(ec);
 }
 
+void test_queue(add_torrent_params p)
+{
+	lt::settings_pack pack = settings();
+	// we're not testing the hash check, just accept the data we write
+	pack.set_bool(settings_pack::disable_hash_checks, true);
+	lt::session ses(pack);
+
+	std::vector<torrent_handle> torrents;
+	for(int i = 0; i < 6; i++)
+	{
+		file_storage fs;
+		std::stringstream file_path;
+		file_path << "test_torrent_dir4/queue" << i;
+		fs.add_file(file_path.str(), 1024);
+		libtorrent::create_torrent t(fs, 128 * 1024, 6);
+
+		std::vector<char> buf;
+		bencode(std::back_inserter(buf), t.generate());
+		boost::shared_ptr<torrent_info> ti = boost::make_shared<torrent_info>(&buf[0], buf.size());
+		p.ti = ti;
+		p.save_path = ".";
+		torrents.push_back(ses.add_torrent(p));
+	}
+
+	std::vector<int> pieces = torrents[5].piece_priorities();
+	std::vector<std::pair<int, int> > piece_prios;
+	for (int i = 0; i < int(pieces.size()); ++i) {
+		piece_prios.push_back(std::make_pair(i,0));
+	}
+	torrents[5].prioritize_pieces(piece_prios);
+	torrent_handle finished = torrents[5];
+
+	// add_torrent should be ordered
+	TEST_EQUAL(finished.queue_position(), -1);
+	TEST_EQUAL(torrents[0].queue_position(), 0);
+	TEST_EQUAL(torrents[1].queue_position(), 1);
+	TEST_EQUAL(torrents[2].queue_position(), 2);
+	TEST_EQUAL(torrents[3].queue_position(), 3);
+	TEST_EQUAL(torrents[4].queue_position(), 4);
+
+	// test top and bottom
+	torrents[2].queue_position_top();
+	torrents[1].queue_position_bottom();
+
+	TEST_EQUAL(finished.queue_position(), -1);
+	TEST_EQUAL(torrents[2].queue_position(), 0);
+	TEST_EQUAL(torrents[0].queue_position(), 1);
+	TEST_EQUAL(torrents[3].queue_position(), 2);
+	TEST_EQUAL(torrents[4].queue_position(), 3);
+	TEST_EQUAL(torrents[1].queue_position(), 4);
+
+	// test set pos
+	torrents[0].queue_position_set(0);
+	torrents[1].queue_position_set(1);
+	// torrent 2 should be get moved down by 0 and 1 to pos 2
+
+	TEST_EQUAL(finished.queue_position(), -1);
+	TEST_EQUAL(torrents[0].queue_position(), 0);
+	TEST_EQUAL(torrents[1].queue_position(), 1);
+	TEST_EQUAL(torrents[2].queue_position(), 2);
+	TEST_EQUAL(torrents[3].queue_position(), 3);
+	TEST_EQUAL(torrents[4].queue_position(), 4);
+
+	//test strange up and down commands
+	torrents[0].queue_position_up();
+	torrents[4].queue_position_down();
+
+	TEST_EQUAL(finished.queue_position(), -1);
+	TEST_EQUAL(torrents[0].queue_position(), 0);
+	TEST_EQUAL(torrents[1].queue_position(), 1);
+	TEST_EQUAL(torrents[2].queue_position(), 2);
+	TEST_EQUAL(torrents[3].queue_position(), 3);
+	TEST_EQUAL(torrents[4].queue_position(), 4);
+
+	torrents[1].queue_position_up();
+	torrents[3].queue_position_down();
+	finished.queue_position_up();
+
+	TEST_EQUAL(finished.queue_position(), -1);
+	TEST_EQUAL(torrents[1].queue_position(), 0);
+	TEST_EQUAL(torrents[0].queue_position(), 1);
+	TEST_EQUAL(torrents[2].queue_position(), 2);
+	TEST_EQUAL(torrents[4].queue_position(), 3);
+	TEST_EQUAL(torrents[3].queue_position(), 4);
+
+	torrents[1].queue_position_down();
+	torrents[3].queue_position_up();
+	finished.queue_position_down();
+
+
+	TEST_EQUAL(finished.queue_position(), -1);
+	TEST_EQUAL(torrents[0].queue_position(), 0);
+	TEST_EQUAL(torrents[1].queue_position(), 1);
+	TEST_EQUAL(torrents[2].queue_position(), 2);
+	TEST_EQUAL(torrents[3].queue_position(), 3);
+	TEST_EQUAL(torrents[4].queue_position(), 4);
+
+	// test set pos on not existing pos
+	torrents[3].queue_position_set(10);
+	finished.queue_position_set(10);
+
+	TEST_EQUAL(finished.queue_position(), -1);
+	TEST_EQUAL(torrents[0].queue_position(), 0);
+	TEST_EQUAL(torrents[1].queue_position(), 1);
+	TEST_EQUAL(torrents[2].queue_position(), 2);
+	TEST_EQUAL(torrents[4].queue_position(), 3);
+	TEST_EQUAL(torrents[3].queue_position(), 4);
+}
+
+TORRENT_TEST(queue)
+{
+	test_queue(add_torrent_params());
+}
+
+TORRENT_TEST(queue_paused)
+{
+	add_torrent_params p;
+	p.flags |= add_torrent_params::flag_paused;
+	p.flags &= ~add_torrent_params::flag_auto_managed;
+	test_queue(p);
+}
+
+TORRENT_TEST(test_move_storage_no_metadata)
+{
+	lt::session ses(settings());
+	add_torrent_params p;
+	p.save_path = "save_path";
+	error_code ec;
+	parse_magnet_uri("magnet?xt=urn:btih:abababababababababababababababababababab", p, ec);
+	torrent_handle h = ses.add_torrent(p);
+
+	TEST_EQUAL(h.status().save_path, complete("save_path"));
+
+	h.move_storage("save_path_1");
+
+	TEST_EQUAL(h.status().save_path, complete("save_path_1"));
+}
 
