@@ -134,6 +134,8 @@ static test_torrent_t test_torrents[] =
 	{ "unordered.torrent" },
 	{ "symlink_zero_size.torrent" },
 	{ "pad_file_no_path.torrent" },
+	{ "invalid_filename.torrent" },
+	{ "invalid_filename2.torrent" },
 };
 
 struct test_failing_torrent_t
@@ -326,6 +328,11 @@ TORRENT_TEST(sanitize_path_trailing_spaces)
 TORRENT_TEST(sanitize_path)
 {
 	std::string path;
+
+	sanitize_append_path_element(path, "\0\0\xed\0\x80", 5);
+	TEST_EQUAL(path, "_");
+
+	path.clear();
 	sanitize_append_path_element(path, "/a/", 3);
 	sanitize_append_path_element(path, "b", 1);
 	sanitize_append_path_element(path, "c", 1);
@@ -468,7 +475,7 @@ TORRENT_TEST(sanitize_path)
 	// 5-byte utf-8 sequence (not allowed)
 	path.clear();
 	sanitize_append_path_element(path, "filename\xf8\x9f\x9f\x9f\x9f" "foobar", 19);
-	TEST_EQUAL(path, "filename_____foobar");
+	TEST_EQUAL(path, "filename_foobar");
 
 	// redundant (overlong) 2-byte sequence
 	// ascii code 0x2e encoded with a leading 0
@@ -487,6 +494,34 @@ TORRENT_TEST(sanitize_path)
 	path.clear();
 	sanitize_append_path_element(path, "filename\xf0\x80\x80\xae", 12);
 	TEST_EQUAL(path, "filename_");
+
+	// a filename where every character is filtered is not replaced by an understcore
+	path.clear();
+	sanitize_append_path_element(path, "//\\", 3);
+	TEST_EQUAL(path, "");
+
+	// make sure suspicious unicode characters are filtered out
+	path.clear();
+	// that's utf-8 for U+200e LEFT-TO-RIGHT MARK
+	sanitize_append_path_element(path, "foo\xe2\x80\x8e" "bar", 9);
+	TEST_EQUAL(path, "foobar");
+
+	// make sure suspicious unicode characters are filtered out
+	path.clear();
+	// that's utf-8 for U+202b RIGHT-TO-LEFT EMBEDDING
+	sanitize_append_path_element(path, "foo\xe2\x80\xab" "bar", 9);
+	TEST_EQUAL(path, "foobar");
+}
+
+TORRENT_TEST(sanitize_path_zeroes)
+{
+	std::string path;
+	sanitize_append_path_element(path, "\0foo", 4);
+	TEST_EQUAL(path, "foo");
+
+	path.clear();
+	sanitize_append_path_element(path, "\0\0\0\0", 4);
+	TEST_EQUAL(path, "");
 }
 
 TORRENT_TEST(verify_encoding)
@@ -503,71 +538,77 @@ TORRENT_TEST(verify_encoding)
 	// valid 2-byte sequence
 	test = "filename\xc2\xa1";
 	TEST_CHECK(verify_encoding(test));
-	fprintf(stderr, "%s\n", test.c_str());
+	fprintf(stdout, "%s\n", test.c_str());
 	TEST_CHECK(test == "filename\xc2\xa1");
 
 	// truncated 2-byte sequence
 	test = "filename\xc2";
 	TEST_CHECK(!verify_encoding(test));
-	fprintf(stderr, "%s\n", test.c_str());
+	fprintf(stdout, "%s\n", test.c_str());
 	TEST_CHECK(test == "filename_");
 
 	// valid 3-byte sequence
 	test = "filename\xe2\x9f\xb9";
 	TEST_CHECK(verify_encoding(test));
-	fprintf(stderr, "%s\n", test.c_str());
+	fprintf(stdout, "%s\n", test.c_str());
 	TEST_CHECK(test == "filename\xe2\x9f\xb9");
 
 	// truncated 3-byte sequence
 	test = "filename\xe2\x9f";
 	TEST_CHECK(!verify_encoding(test));
-	fprintf(stderr, "%s\n", test.c_str());
+	fprintf(stdout, "%s\n", test.c_str());
 	TEST_CHECK(test == "filename_");
 
 	// truncated 3-byte sequence
 	test = "filename\xe2";
 	TEST_CHECK(!verify_encoding(test));
-	fprintf(stderr, "%s\n", test.c_str());
+	fprintf(stdout, "%s\n", test.c_str());
 	TEST_CHECK(test == "filename_");
 
 	// valid 4-byte sequence
 	test = "filename\xf0\x9f\x92\x88";
 	TEST_CHECK(verify_encoding(test));
-	fprintf(stderr, "%s\n", test.c_str());
+	fprintf(stdout, "%s\n", test.c_str());
 	TEST_CHECK(test == "filename\xf0\x9f\x92\x88");
 
 	// truncated 4-byte sequence
 	test = "filename\xf0\x9f\x92";
 	TEST_CHECK(!verify_encoding(test));
-	fprintf(stderr, "%s\n", test.c_str());
+	fprintf(stdout, "%s\n", test.c_str());
 	TEST_CHECK(test == "filename_");
 
 	// 5-byte utf-8 sequence (not allowed)
 	test = "filename\xf8\x9f\x9f\x9f\x9f""foobar";
 	TEST_CHECK(!verify_encoding(test));
-	fprintf(stderr, "%s\n", test.c_str());
+	fprintf(stdout, "%s\n", test.c_str());
 	TEST_CHECK(test == "filename_____foobar");
 
 	// redundant (overlong) 2-byte sequence
 	// ascii code 0x2e encoded with a leading 0
 	test = "filename\xc0\xae";
 	TEST_CHECK(!verify_encoding(test));
-	fprintf(stderr, "%s\n", test.c_str());
+	fprintf(stdout, "%s\n", test.c_str());
 	TEST_CHECK(test == "filename__");
 
 	// redundant (overlong) 3-byte sequence
 	// ascii code 0x2e encoded with two leading 0s
 	test = "filename\xe0\x80\xae";
 	TEST_CHECK(!verify_encoding(test));
-	fprintf(stderr, "%s\n", test.c_str());
+	fprintf(stdout, "%s\n", test.c_str());
 	TEST_CHECK(test == "filename___");
 
 	// redundant (overlong) 4-byte sequence
 	// ascii code 0x2e encoded with three leading 0s
 	test = "filename\xf0\x80\x80\xae";
 	TEST_CHECK(!verify_encoding(test));
-	fprintf(stderr, "%s\n", test.c_str());
+	fprintf(stdout, "%s\n", test.c_str());
 	TEST_CHECK(test == "filename____");
+
+	// missing byte header
+	test = "filename\xed\0\x80";
+	TEST_CHECK(!verify_encoding(test));
+	fprintf(stdout, "%s\n", test.c_str());
+	TEST_CHECK(test == "filename_");
 }
 
 TORRENT_TEST(parse_torrents)
@@ -618,12 +659,12 @@ TORRENT_TEST(parse_torrents)
 	std::string root_dir = parent_path(current_working_directory());
 	for (int i = 0; i < int(sizeof(test_torrents)/sizeof(test_torrents[0])); ++i)
 	{
-		fprintf(stderr, "loading %s\n", test_torrents[i].file);
+		fprintf(stdout, "loading %s\n", test_torrents[i].file);
 		std::string filename = combine_path(combine_path(root_dir, "test_torrents")
 			, test_torrents[i].file);
 		boost::shared_ptr<torrent_info> ti(new torrent_info(filename, ec));
 		TEST_CHECK(!ec);
-		if (ec) fprintf(stderr, " loading(\"%s\") -> failed %s\n", filename.c_str()
+		if (ec) fprintf(stdout, " loading(\"%s\") -> failed %s\n", filename.c_str()
 			, ec.message().c_str());
 
 		if (std::string(test_torrents[i].file) == "whitespace_url.torrent")
@@ -730,6 +771,14 @@ TORRENT_TEST(parse_torrents)
 			TEST_EQUAL(ti->num_files(), 2);
 			TEST_EQUAL(ti->files().file_path(1), combine_path(".pad", "0"));
 		}
+		else if (std::string(test_torrents[i].file) == "invalid_filename.torrent")
+		{
+			TEST_EQUAL(ti->num_files(), 2);
+		}
+		else if (std::string(test_torrents[i].file) == "invalid_filename2.torrent")
+		{
+			TEST_EQUAL(ti->num_files(), 3);
+		}
 
 		file_storage const& fs = ti->files();
 		for (int i = 0; i < fs.num_files(); ++i)
@@ -737,7 +786,7 @@ TORRENT_TEST(parse_torrents)
 			int first = ti->map_file(i, 0, 0).piece;
 			int last = ti->map_file(i, (std::max)(fs.file_size(i)-1, boost::int64_t(0)), 0).piece;
 			int flags = fs.file_flags(i);
-			fprintf(stderr, "  %11" PRId64 " %c%c%c%c [ %4d, %4d ] %7u %s %s %s%s\n"
+			fprintf(stdout, "  %11" PRId64 " %c%c%c%c [ %4d, %4d ] %7u %s %s %s%s\n"
 				, fs.file_size(i)
 				, (flags & file_storage::flag_pad_file)?'p':'-'
 				, (flags & file_storage::flag_executable)?'x':'-'
@@ -769,12 +818,13 @@ TORRENT_TEST(parse_torrents)
 	for (int i = 0; i < int(sizeof(test_error_torrents)/sizeof(test_error_torrents[0])); ++i)
 	{
 		error_code ec;
-		fprintf(stderr, "loading %s\n", test_error_torrents[i].file);
+		fprintf(stdout, "loading %s\n", test_error_torrents[i].file);
 		boost::shared_ptr<torrent_info> ti(new torrent_info(combine_path(
 			combine_path(root_dir, "test_torrents"), test_error_torrents[i].file), ec));
-		fprintf(stderr, "E:        \"%s\"\nexpected: \"%s\"\n", ec.message().c_str()
+		fprintf(stdout, "E:        \"%s\"\nexpected: \"%s\"\n", ec.message().c_str()
 			, test_error_torrents[i].error.message().c_str());
 		TEST_CHECK(ec.message() == test_error_torrents[i].error.message());
+		TEST_EQUAL(ti->is_valid(), false);
 	}
 }
 
@@ -861,7 +911,7 @@ void test_resolve_duplicates(int test_case)
 	{
 		std::string p = ti.files().file_path(i);
 		convert_path_to_posix(p);
-		fprintf(stderr, "%s == %s\n", p.c_str(), filenames[test_case][i]);
+		fprintf(stdout, "%s == %s\n", p.c_str(), filenames[test_case][i]);
 
 		TEST_EQUAL(p, filenames[test_case][i]);
 	}
@@ -920,7 +970,7 @@ TORRENT_TEST(copy)
 		std::string p = a->files().file_path(i);
 		convert_path_to_posix(p);
 		TEST_EQUAL(p, expected_files[i]);
-		fprintf(stderr, "%s\n", p.c_str());
+		fprintf(stdout, "%s\n", p.c_str());
 
 		TEST_EQUAL(a->files().hash(i), file_hashes[i]);
 	}
@@ -942,7 +992,7 @@ TORRENT_TEST(copy)
 		std::string p = b->files().file_path(i);
 		convert_path_to_posix(p);
 		TEST_EQUAL(p, expected_files[i]);
-		fprintf(stderr, "%s\n", p.c_str());
+		fprintf(stdout, "%s\n", p.c_str());
 
 		TEST_EQUAL(b->files().hash(i), file_hashes[i]);
 	}
