@@ -33,6 +33,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/upnp.hpp"
 #include "libtorrent/socket.hpp"
 #include "libtorrent/socket_io.hpp" // print_endpoint
+#include "libtorrent/http_parser.hpp"
 #include "test.hpp"
 #include "setup_transfer.hpp"
 #include <fstream>
@@ -182,8 +183,8 @@ void run_upnp_test(char const* root_filename, char const* router_model, char con
 	std::cerr << "router: " << upnp_handler->router_model() << std::endl;
 	TEST_EQUAL(upnp_handler->router_model(), router_model);
 
-	int mapping1 = upnp_handler->add_mapping(upnp::tcp, 500, 500);
-	int mapping2 = upnp_handler->add_mapping(upnp::udp, 501, 501);
+	int mapping1 = upnp_handler->add_mapping(upnp::tcp, 500, ep("127.0.0.1", 500));
+	int mapping2 = upnp_handler->add_mapping(upnp::udp, 501, ep("127.0.0.1", 501));
 
 	for (int i = 0; i < 40; ++i)
 	{
