@@ -47,6 +47,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <iostream>
 
 using namespace libtorrent;
+namespace lt = libtorrent;
 using boost::tuples::ignore;
 
 void test_transfer()
@@ -62,56 +63,56 @@ void test_transfer()
 	session_proxy p1;
 	session_proxy p2;
 
-	session ses1(fingerprint("LT", 0, 1, 0, 0), std::make_pair(48885, 49930), "0.0.0.0", 0);
-	session ses2(fingerprint("LT", 0, 1, 0, 0), std::make_pair(49885, 50930), "0.0.0.0", 0);
+	const int mask = alert::all_categories
+		& ~(alert::progress_notification
+			| alert::performance_warning
+			| alert::stats_notification);
 
-	session_settings sett;
+	settings_pack pack;
+	pack.set_bool(settings_pack::enable_lsd, false);
+	pack.set_bool(settings_pack::enable_natpmp, false);
+	pack.set_bool(settings_pack::enable_upnp, false);
+	pack.set_bool(settings_pack::enable_dht, false);
+	pack.set_int(settings_pack::alert_mask, mask);
+	pack.set_int(settings_pack::out_enc_policy, settings_pack::pe_disabled);
+	pack.set_int(settings_pack::in_enc_policy, settings_pack::pe_disabled);
+	pack.set_bool(settings_pack::enable_outgoing_tcp, false);
+	pack.set_bool(settings_pack::enable_incoming_tcp, false);
+	pack.set_bool(settings_pack::announce_to_all_trackers, true);
+	pack.set_bool(settings_pack::announce_to_all_tiers, true);
+	pack.set_bool(settings_pack::prefer_udp_trackers, false);
+	pack.set_int(settings_pack::min_reconnect_time, 1);
+	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:48885");
+	lt::session ses1(pack);
 
-	sett.enable_outgoing_tcp = false;
-	sett.min_reconnect_time = 1;
-	sett.announce_to_all_trackers = true;
-	sett.announce_to_all_tiers = true;
-	// make sure we announce to both http and udp trackers
-	sett.prefer_udp_trackers = false;
-
-	// speed up loopback connections (by using the full MTU)
-	sett.utp_dynamic_sock_buf = true;
-
-	// for performance testing
-//	sett.disable_hash_checks = true;
-//	sett.utp_delayed_ack = 0;
-
-	// disable this to use regular size packets over loopback
-//	sett.utp_dynamic_sock_buf = false;
-
-	ses1.set_settings(sett);
-	ses2.set_settings(sett);
-
-#ifndef TORRENT_DISABLE_ENCRYPTION
-	pe_settings pes;
-	pes.out_enc_policy = pe_settings::disabled;
-	pes.in_enc_policy = pe_settings::disabled;
-	ses1.set_pe_settings(pes);
-	ses2.set_pe_settings(pes);
-#endif
+	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:49885");
+	lt::session ses2(pack);
 
 	torrent_handle tor1;
 	torrent_handle tor2;
 
 	create_directory("./tmp1_utp", ec);
 	std::ofstream file("./tmp1_utp/temporary");
-	boost::intrusive_ptr<torrent_info> t = ::create_torrent(&file, 128 * 1024, 6, false);
+	boost::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 128 * 1024, 6, false);
 	file.close();
 
 	// for performance testing
 	add_torrent_params atp;
+	atp.flags &= ~add_torrent_params::flag_paused;
+	atp.flags &= ~add_torrent_params::flag_auto_managed;
 //	atp.storage = &disabled_storage_constructor;
 
 	// test using piece sizes smaller than 16kB
 	boost::tie(tor1, tor2, ignore) = setup_transfer(&ses1, &ses2, 0
 		, true, false, true, "_utp", 0, &t, false, &atp);
 
-	for (int i = 0; i < 6; ++i)
+#ifdef TORRENT_USE_VALGRIND
+	const int timeout = 16;
+#else
+	const int timeout = 8;
+#endif
+
+	for (int i = 0; i < timeout; ++i)
 	{
 		print_alerts(ses1, "ses1", true, true, true);
 		print_alerts(ses2, "ses2", true, true, true);
@@ -138,16 +139,14 @@ void test_transfer()
 	p2 = ses2.abort();
 }
 
-int test_main()
+TORRENT_TEST(utp)
 {
 	using namespace libtorrent;
 
 	test_transfer();
-	
+
 	error_code ec;
 	remove_all("./tmp1_utp", ec);
 	remove_all("./tmp2_utp", ec);
-
-	return 0;
 }
 
